/*
 * (C) Copyright 2005-2007 Hewlett-Packard Development Company, L.P.
 */

//
//  Filename: $Source: /usr/local/vse/glib/src/web/js/addonList.js,v $
//  Version:  $Revision: 1.6 $
//  Date:     $Date: 2008/05/16 20:32:29 $
//
//  Hewlett-Packard Company Confidential(C) Copyright 2005-2007 Hewlett-Packard
//  Development Company, L.P.

/**
 * Add the empty first row of the table of an AddonListG.
 * @param tableName Name of the table to be adding to.
 * @param numCols The number of columns in this table.
 */
function addInitialRow(tableName,numCols)
{
        vsValidateArgs(['string','number']);
        var table = document.getElementById(tableName);
                var tbody = table.getElementsByTagName('TBODY')[0];
        var rows = tbody.getElementsByTagName('tr');

        // If only the header row exists, then put in the empty label.
        if (rows.length==1) {
                var tr = document.createElement('tr');
                var td = document.createElement('td');
                td.setAttribute("colSpan", numCols + 1); // add one for the selector column
                var emptyLabelField = document.getElementById(tableName + "_emptyLabel");
                var emptyLabel = emptyLabelField.value;
                td.appendChild(document.createTextNode(emptyLabel));
                tr.appendChild(td);
                tbody.appendChild(tr);
        }
}

/**
 * Remove the empty first row of the table of an AddonListG.
 * @param tableName Name of the table to remove row from.
 */
function removeInitialRow(tableName)
{
        vsValidateArgs(['string']);
        var table = document.getElementById(tableName);
        var tbody = table.getElementsByTagName('TBODY')[0];
//
//        var rows = tbody.getElementsByTagName('tr');
//        var inputs = tbody.getElementsByTagName('input');

        // Every row in an AddonList has one <input>, including the header.
        // The only row that doesn't have in <input> is the empty initial row.
        // If there is more then just the header row, and there is only the header rows input.
        // Then remove the row below the header row.
//        if (rows.length>1) {
//                if(inputs.length==1){
//                        tbody.deleteRow(1);
//                }
//        }
        var rows = ourTableManager.getSelectableRows(table);
        if(rows.length == 0)
             tbody.deleteRow(1);
                
}

/**
 * Returns true if the number of rows in the AddonListG matches
 * the passed-in maxEntries value; false otherwise.
 * @return true if the table is full; false otherwise.
 */
function isFullAddonListGTable(tableName, maxEntries) {
	vsValidateArgs(['string','number']);
	var table = document.getElementById(tableName);
	var tbody = table.getElementsByTagName('TBODY')[0];
	var inputs = tbody.getElementsByTagName('input');

	// Since every row in the AddonListG (including the header) has one
	// input, we have filled up the list when the list size exceeds our
	// maximum number of entries.
	return (inputs.length > maxEntries);
}

/**
 * Returns true if the AddonListG table is empty
 * @param tableName Name of the table to check
 * @return true if the table is empty, false otherwise.
 */
function isEmptyAddonListGTable(tableName) {
	vsValidateArgs(['string']);
	var table = document.getElementById(tableName);
	var tbody = table.getElementsByTagName('TBODY')[0];
	var inputs = tbody.getElementsByTagName('input');

	// Every row in an AddonListG (including the header) has one input,
	// except for the empty initial row.  So if we have just one input,
	// then we know that only the header row is displayed.
	return (inputs.length <= 2);
}


/**
 * Add a hidden input value representing data in given cell of AddonList.
 * This allows transporting data back to the server. A table isn't included in a post,
 * so the data put in the table needs to be stored in something that will be posted.
 * @param gadgetName Name of the AddonListG object, and the table.
 * @param rowNum Which row.
 * @param colNum Which column.
 * @param value What value should be stored.
 */
function addOnlistGAddHiddenInput(gadgetName, rowNum, colNum, value)
{
        vsValidateArgs(['string','number','number','string']);
    var inputName = addOnlistGGetHiddenInputName(gadgetName, rowNum, colNum);
        var form = document.getElementById('mainForm');
        var hiddenInputCounter = gadgetName + '.numHiddenDataInputs';

        // If the input already exists, just put the value in and return.
        var inputElement = getField(form, inputName);
        if (inputElement) {
            inputElement.value = value;
                return;
        }

        // --- Need to add another hidden input. ---

        // Increase the recorded number of hidden inputs.
        getField(form, hiddenInputCounter).value++;

        // Now create the hidden input and add it to the form.
    var input = document.createElement('input');
    input.setAttribute('type', 'hidden');
    input.setAttribute('name', inputName);
    input.setAttribute('value', value);
        form.appendChild(input);
}

/**
 * Clear value of hidden input representing data in given cell of AddonList.
 * Hidden input was added, but now needs to be removed, since the row in the
 * table was removed.
 * @param tableName Name of table to remove row from.
 * @param rowNum Which row.
 */
function addOnlistGClearHiddenInput(tableName, rowNum)
{
    vsValidateArgs(['string','number']);
    var table = document.getElementById(tableName);
    var rows = table.getElementsByTagName('tr');
    var lastRowNum = rows.length-1;
    //
    // Can't use index 0, because that row only has 'th' tags
    //
    var cols = rows[1].getElementsByTagName('td');
    var form = document.getElementById('mainForm');
    var thisElem;
    var nextElem;

    //
    // We need to shift the values up by one row for every input
    // beyond the row being deleted.  Finally, clear the inputs
    // in the last row.  This way the hidden inputs will line
    // up with the real rows of data in the table.
    //
    for (var i=rowNum; i <= lastRowNum; i++) {
        for (var j=0; j < cols.length-1; j++) {
            thisElem = addOnlistGGetInputElement(form, tableName, i, j);
            if (i == lastRowNum) {
                thisElem.value = "";
            } else {
                nextElem = addOnlistGGetInputElement(form, tableName, i+1, j);
                thisElem.value = nextElem.value;
            }

        }
    }
}

/**
 * Return the hidden AddonListG inputElement in the form corresponding to
 * a form, tableName, row and column.
 *
 * @param formName Form the input should exist in.
 * @param tableName Name of table to remove row from.
 * @param rowNum Which row.
 * @param colNum Which column.
 */
function addOnlistGGetInputElement(form, tableName, rowNum, colNum)
{
    var inputName = addOnlistGGetHiddenInputName(tableName,
                                                 rowNum,
                                                 colNum);

    var inputElement = getField(form, inputName);
    vsAssert(inputElement!=null,"hidden input couldn't be found");
    return inputElement;
}

/**
 * Create the appropriate name for the hidden input for a table cells data.
 * @param gadgetName Name of the AddonListG object, and the table.
 * @param rowNum Which row.
 * @param colNum Which column.
 * @return Name for the HiddenInput.
 */
function addOnlistGGetHiddenInputName(gadgetName, rowNum, colNum)
{
    vsValidateArgs(["string","number","number"]);
    return gadgetName + ".row_" + rowNum + "_col_" + colNum;
}

/**
 * Add a row to the AddonList table.
 * This is the function that is directly called when AddonListG's AddButton is pushed.
 * @param tableName Name of the table to add a row to.
 * @param rbID ID of the radio button for this row.
 * @param inputValues Array of values, corresponding to the cells in the row.
 */
function addonListGAddRowByValue(tableName, rbID, inputValues)
{
        vsValidateArgs(["string", "string", "Array"]);

        // Remove the initial row
        removeInitialRow(tableName);

        // Look up the table and create a row for it.
        var table = document.getElementById(tableName);
        var tbody = table.getElementsByTagName('TBODY')[0];
        
        // Keep a handle on the first row (which contains the column headings)
        // We'll check these cells later to see if we need to hide cells
        // being added to the table.
        var firstRow = table.getElementsByTagName('tr')[0];
        
        
        var newRowNum = table.getElementsByTagName('tr').length;
        var newRowNum0 = newRowNum-1; // Java code uses 0-based row indexes
        var tr = document.createElement('tr');
        if((newRowNum0 % 2) == 0)
                        appendClassName(tr, "altRowColor");

        // Create the radio button or checkbox
        var rowSelector;
        rowSelector=document.getElementById(rbID).cloneNode(true);
        rowSelector.setAttribute('rowselector', 'yes'); // Affects row, GUIDO specific
        rowSelector.style.display='';
		// In IE, radiobuttons must have unique IDs or the newly
		// created radio button will become selected for some reason.
        rowSelector.id = rbID + "_" + newRowNum;
        var selectorValue = "" + newRowNum0;
        rowSelector.value = selectorValue;
        rowSelector.setAttribute('selTypes', ''); 
        rowSelector.setAttribute('selLabel', ''); 

        // Put the radio button or checkbox in the row
        var td0 = document.createElement('td');
        td0.setAttribute('class', 'checkboxCell'); // checkboxCell is used for RadioButtons and checkboxes.
        td0.appendChild(rowSelector);
        tr.appendChild(td0);

        // Copy array entries into cells in row.
        for (var i=0; i < inputValues.length; i++) {
            var td = document.createElement('td');
            td.appendChild(document.createTextNode(inputValues[i]));
            
            // If column this data is going into is hidden set the
            // display on this cell to be hidden too.
            // (must use firstRow.cells[i+1] because the radio button has
            // already been added to the row)
            if(firstRow.cells[i+1].style.display == "none") {
        		td.style.display = "none";
        	}
            
            tr.appendChild(td);
            addOnlistGAddHiddenInput(tableName, newRowNum, i, inputValues[i]);
        }

        // Add the new row to the table
        tbody.appendChild(tr);
        ourTableManager.initTableRow(table, tr);
        ourTableManager.setRowSelected(tr, true, true);
}

/**
 * Updates the selected row of AddonList table.
 * This is the function that is directly called when AddonListG's Update button is pushed.
 * @param tableName Name of the table to add a row to.
 * @param inputValues Array of values, corresponding to the cells in the row.
 */
function addonListGUpdateRowByValue(tableName, inputValues) {
    vsValidateArgs(["string", "Array"]);
    var table = document.getElementById(tableName);
    var form = document.getElementById("mainForm");
    var rowIndex = findSelectedTableRow(table);    
    var selRow = getAbsoluteTableRowIndex(rowIndex);
    if(selRow > 0) {
        for (var i = 0; i < inputValues.length; i++) {
            var cell = table.rows[selRow].cells[i+1];
            var text = cell.childNodes[0];
            text.data = inputValues[i];
                        var input = addOnlistGGetInputElement(form, tableName, selRow, i);
                        input.value = inputValues[i];
                }
    }

}

/**
 * Returns logical index of the first selected table row or -1 if no row is selected.
 * The index is 0-based.
 */
function findSelectedTableRow(table) {
    var rows = ourTableManager.getSelectableRows(table);
    for (var i = 0; i < rows.length; i++) {
            if(ourTableManager.isRowSelected(rows[i]))
                return i;
    }
    return -1;
}

/** Returns the absolute index of the specified logical row index */
function getAbsoluteTableRowIndex(rowIndex) {
    return rowIndex+1;
}

/**
 * Remove a row from the AddonList table.
 * This is the function that is directly called when AddonListG's RemoveButton is pushed.
 * @param tableName Name of the table from which to remove a row.
 * @param numCols how many columns are in this row.
 */
function addOnlistGRmRow(tableName, numCols) {
    vsValidateArgs(["string", "number"]);
    var table = document.getElementById(tableName);
    var rowIndex = findSelectedTableRow(table);
    if(rowIndex >= 0) {
        deleteTableRow(tableName, table, numCols, rowIndex);
    }
}

/** Deletes the table row specified by logical row index */
function deleteTableRow(tableName, table, numCols, rowIndex) {
    vsValidateArgs(["string", "object", "number", "number"]);
    vsAssert(rowIndex >= 0, "invalid row index");
    var selRow = getAbsoluteTableRowIndex(rowIndex);    
    ourTableManager.setRowSelected(table.rows[selRow], false, true);   
    addOnlistGClearHiddenInput(tableName, selRow);
    table.deleteRow(selRow);
    addInitialRow(tableName,numCols);
} 

/**
 * Enable the up button if an Up operation is valid on the currently selected row
 *
 * @param tableName Name of the table
  * @param upButtonName Name of the up button to enable/disable
 */
function addOnlistGEnableUpButton(tableName, upButtonName) {
    vsValidateArgs(["string"]);
    var table = document.getElementById(tableName);
    var rowIndex = findSelectedTableRow(table);
    var row = getAbsoluteTableRowIndex(rowIndex);
    if(row > 1 && row < table.rows.length) {
         ourButtonManager.enableButtonById(upButtonName);
         return;
    }
   ourButtonManager.disableButtonById(upButtonName);
}

/**
 * Enable the down button if a Down operation is valid on the currently selected row
 *
 * @param tableName Name of the table
 * @param downButtonName Name of the down button to enable/disable
 */
function addOnlistGEnableDownButton(tableName, downButtonName) {
    vsValidateArgs(["string"]);
    var table = document.getElementById(tableName);
    var rowIndex = findSelectedTableRow(table);
    var row = getAbsoluteTableRowIndex(rowIndex);
    if(row >= 1 && row < (table.rows.length - 1)) {
       ourButtonManager.enableButtonById(downButtonName);
       return;
    }
    ourButtonManager.disableButtonById(downButtonName);
}

/**
 * Swap the selected row in the table with the one above, and 
 * update the selection to match.
 *
 * @param tableName Name of the table from which to remove a row.
 * @param numCols how many columns are in this row.
 */
function addOnlistGUpRow(tableName, numCols) {
    vsValidateArgs(["string", "number"]);
    var table = document.getElementById(tableName);
    var rowIndex = findSelectedTableRow(table);
    if(rowIndex >= 1) {
        addOnListSwap(tableName, numCols, rowIndex, rowIndex-1);
        ourTableManager.setRowSelected(table.rows[getAbsoluteTableRowIndex(rowIndex-1)], true, true);   
    }
}

/**
 * Swap the selected row in the table with the one below, and 
 * update the selection to match.
 *
 * @param tableName Name of the table from which to remove a row.
 * @param numCols how many columns are in this row.
 */
function addOnlistGDownRow(tableName, numCols) {
    vsValidateArgs(["string", "number"]);
    var table = document.getElementById(tableName);
    var rowIndex = findSelectedTableRow(table);
    var rows = ourTableManager.getSelectableRows(table);
    if(rowIndex >= 0 && rowIndex < rows.length) {
        addOnListSwap(tableName, numCols, rowIndex, rowIndex+1);
        ourTableManager.setRowSelected(table.rows[getAbsoluteTableRowIndex(rowIndex+1)], true, true);   
    }
}

/**
 * Swap two rows in the list
 * This is the function that is indirectly called when AddonListG's UpRow/DownRow
 * method is called
 * @param tableName Name of the table from which to remove a row.
 * @param numCols how many columns are in this row.
 * @param src source logical row index
 * @param dest destinatino logical row index
 */
function addOnListSwap(tableName, numCols, src, dest) {
    vsValidateArgs(["string", "number", "number", "number"]);
    var table = document.getElementById(tableName);
    var srcRow = getAbsoluteTableRowIndex(src);
    var destRow = getAbsoluteTableRowIndex(dest);
    if(srcRow > 0 && destRow > 0) {
        for (var i = 0; i < numCols; i++) {
            var srcCell = table.rows[srcRow].cells[i+1];
            var destCell = table.rows[destRow].cells[i+1];
            var srcChild = null;
            if (srcCell.firstChild != null)
                srcChild = srcCell.removeChild(srcCell.firstChild);
            var destChild = null;
            if (destCell.firstChild != null) 
                destChild=destCell.removeChild(destCell.firstChild);
            if (srcChild != null)               
                destCell.appendChild(srcChild);
            if (destChild != null) 
                srcCell.appendChild(destChild);
                
            var srcHidden = addOnlistGGetInputElement(document.mainForm, tableName, srcRow, i);
            var destHidden = addOnlistGGetInputElement(document.mainForm, tableName, destRow, i);
            var tmp = destHidden.value;
            destHidden.value = srcHidden.value;
            srcHidden.value = tmp;
            }
    }
}


/**
 * Returns a string containing all list values separated by a given delimiter.
 * If a value contains the delimiter, it's  removed from the value.
 * @param tableName - the gadget name
 * @param numCols - number of columns in addon list
 * @param delim - the delimiter to be used
 */
function addOnlistGEncode(tableName, numCols, delim) {
    vsValidateArgs(["string", "number", "string"]);
    var table = ourTableManager.getTableByName(tableName);
    var rowCount = ourTableManager.getRowCount(table);
    var result = "";
    for(var i = 1; i <= rowCount; i++) {
        for(var j = 0; j < numCols; j++) {
            var input = addOnlistGGetInputElement(document.mainForm, tableName, i, j);
            var s = input.value;
            s = s.replace(delim, ""); // just in case, remove the delimiter if it exists in the string
            if(i > 1 || j > 0)
                result += delim;
            result += s;
        }
    }
    return result;
}

function addOnlistGClear(tableName, numCols) {
    var table = ourTableManager.getTableByName(tableName);
    var rowCount = ourTableManager.getRowCount(table);
    for(var i = rowCount-1; i >= 0; i--) {
        deleteTableRow(tableName, table, numCols, i);
    }
}

/**
 * Populates an add-on list from an encoded string of values.
 */
function addOnlistPopulate(tableName, numCols, delim, values) {
    vsValidateArgs(["string", "number", "string", "string"]);
    addOnlistGClear(tableName, numCols);
    var splitValues = values.split(delim);
    var rowCount = splitValues.length / numCols;
    var idx = 0;
    for(var i = 1; i <= rowCount; i++) {
        if(splitValues.length < idx + numCols)
            break;
        var rowValues = new Array();
        for(var j = 0; j < numCols; j++) {
            rowValues[j] = splitValues[idx++];
        }
        addonListGAddRowByValue(tableName, tableName+".rs.0", rowValues); 
    }
}

function getField (form, fieldName) {
  if (!document.all) {
          return form[fieldName];
  } else { // IE has a bug not adding dynamically created field
           // as named properties so we loop through the elements array
    for (var e = 0; e < form.elements.length; e++) {
      if (form.elements[e].name == fieldName) {
                  return form.elements[e];
          }
        }
    return null;
  }
}

