/*
 * (C) Copyright 2001-2006 Hewlett-Packard Development Company, L.P.
 */

/**
 * java scripts for the create npar wizard.
 * <p>
 * This file contains mostly validation scripts for various wizard pages.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/js/create_npar.js,v 1.20 2006/01/12 22:44:11 shunr Exp $
 * @risk Medium
 * @task name = Define hardware partition object for create/modify npar
 *       cycle = 2
 *       phase = CODE
 *       status = Done
 *       owner = Ming
 *       description = none
 */

/*
 * make sure there is a partition name and that it is not longer
 * than 64 characters, nor does it have unallowed characters.  If the
 * validation fails, a pop up error message will appear indicating which 
 * fields need to be corrected.
 *
 * @return true or false depending on if the checks where successful or not.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect a message dialog will be invoked with the error message if the
 *             user entered an inapropriate value.
 */
function validate_par_name(){
    var re=/^[a-z0-9\._\-\ ]*$/i;
    var name=document.setParName.parName.value;
    var msg = "Only alphanumeric,\" \", \".\", \"_\", and \"-\" characters are allowed in the partition name";

    // validate field values
    if (name.search(re)) {
	// there is a non-allowed character
	alert(msg);
	return false;
    }

    return true;
}

/*
 * run the cell option checks and display any error messages.
 *
 * @param myForm a form object that contains the cell options selectors.
 * @return true or false depending on if the checks where successful or not.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect a message dialog will be invoked with the error message if the
 *             user entered an inapropriate value.
 */
function validateCellOptions(myForm){
    var cells = new Array();
    var re = /coreCellChoice/;
    var cellRe = /\d+/;
    var j = 0;

    //
    // Loop through the elements on the form.  The core cell choice is 
    // identified with any row in the table that has a selector for core cell
    // choices.  The actual choice does not matter.
    //
    for (var i = 0; i < myForm.elements.length; ++i)
    {
	// get the elements name
	var name = myForm.elements[i].name;

	//
	// If this element is a coreCellChoice element store
	// the cell number in the cells array
	//
	if (name.search(re) != -1)
	{
	   cells[j++] = myForm.elements[i].name.match(cellRe); 
	}
    }

    var msg = checkUseOnNextBoot(myForm, cells) + 
	      checkFailureUsage(myForm, cells) + 
	      checkCoreChoices();

    if (msg.length != 0){
	alert(msg);
	return false;
    }

    return true;
}

/*
 * Make sure there is at least one viable core cell that will be used on 
 * failure.
 *
 * @param myForm a form object that contains the cell options selectors.
 * @return an empty string or an error message if the check failed.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
function checkFailureUsage(myForm, cells){
    // global defines for cell usage.
    BASE = "3";
    BASE_NO_INTERLEAVE = "4";
    FLOATING = "5";
    DISABLE = "6";

    var numDisabledCells = 0;
    var msg = "There must be at least one viable core cell whose \"failure usage\" option is NOT set to \"disable\".\n\n";

    //
    // Loop through the elements on the form and see if there is at least
    // one core cell that will be used on next boot.
    //
    for (i = 0; i < myForm.elements.length; ++i)
    {
        //
        // Loop through the array of cells created above.  If
	// one of the cells in the array set to be used on
	// next boot return true.  Otherwise we will fall
	// through the loop.
        //
	for (j = 0; j < cells.length; ++j)
	{
	    var cellFailureUsage = "failureUsage" + cells[j];
	    if ((myForm.elements[i].name == cellFailureUsage) && 
		(myForm.elements[i].value == DISABLE))
	    {
		++numDisabledCells;
	    }
	}
    }

    if (numDisabledCells == cells.length){
	return msg;
    }

    return "";
}

/*
 * make sure there is at least one viable core cell that will be used on next 
 * boot
 *
 * @param myForm a form object that contains the cell options selectors.
 * @return an empty string or an error message if the check failed.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
function checkUseOnNextBoot(myForm, cells){
    var msg = "There must be one viable core cell whose \"use on next boot\" option is set to \"yes\".\n\n";

    //
    // Loop through the elements on the form and see if there is at least
    // one core cell that will be used on next boot.
    //
    for (i = 0; i < myForm.elements.length; ++i)
    {
        //
        // Loop through the array of cells created above.  If
	// one of the cells in the array set to be used on
	// next boot return true.  Otherwise we will fall
	// through the loop.
        //
	for (j = 0; j < cells.length; ++j)
	{
	    var cellUseOnNextBoot = "useOnNextBoot" + cells[j];
	    if ((myForm.elements[i].name == cellUseOnNextBoot) && 
		(myForm.elements[i].value == "true"))
	    {
		return "";
	    }
	}
    }

    return msg;
}

/*
 * make sure there is no duplicate.  If there is a duplicate, then the cell 
 * that didn't trigger the event will have it's core choice set to none.
 *
 * @param selectedIndex the index number of the selected option.
 * @param numSelections the number options in the select menu.
 * @param rowIndex the row number of the cell in the cell table.
 * 
 * @return None.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect The core cell choices for cells showing on the screen might
 *	       be modified to "none".
 */
function validate_core_choices(selectedIndex, numSelections, rowIndex){

    //
    // when the user sets a core choice, make sure to shift the core choices
    // around when neccessary so there will not be duplicate core choices.
    //
    if (selectedIndex == 0){
	return true;
    }

    var re=/coreCellChoice/;
    var elements = document.setCellOptions.elements;
    var eLength = elements.length;
    var cells = new Array();
    var j = 0;

    //
    // get the list of viable core cell choices, less the cell that triggered
    // this event.
    //
    for (var i = 0; i < eLength; ++i){

	var name = elements[i].name;

	if (name.search(re) != -1){

	    var row = elements[i].parentNode.parentNode;

	    if ((row.rowIndex != rowIndex) && 

		(elements[i].selectedIndex == selectedIndex)){

                //
		// this is a different core cell viable cell than the one
		// that triggered the onChange event, and it has the same
		// core choice. Set this cells choice to none.
                //
		elements[i].selectedIndex = 0;
	    }
	}
    }
}

/*
 * Check to see if there are any holes in core cell choices, i.e. 1, 2, and 4
 * but no 3.
 *
 * @return an empty string or an error message if the check failed.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
function checkCoreChoices(){
    var re=/coreCellChoice/;
    var elements = document.setCellOptions.elements;
    var eLength = elements.length;
    var numSelections = 0;
    var coreNone = 0;
    var numCoreCells = 0;
    var msg = "There is a discontinuity in the core cell choices.  All specified core cell choices must be in consecutive order before continuing.";

    // use flags for core cell choices.
    var choices = [false, false, false, false, false];

    // find out which cells have a core cell choice, and what is it.
    for (var i = 0; i < eLength; ++i){
	var name = elements[i].name;
	if (name.search(re) != -1){
	    // get the number of core cell choices showing on the screen.
	    if (numSelections == 0){
		numSelections = elements[i].length;
	    }
	    // set the used flag for this core cell choice to true.
	    choices[elements[i].selectedIndex] = true;

	    // find out if this core capable cell does not have a core choice.
	    if (elements[i].selectedIndex == 0){
		++coreNone;
	    }

	    // get the number of core capable cells.
	    ++numCoreCells;
	}
    }

    // if all of the core capable cells have "none" as the selection, then
    // return and don't go through the continuity check below.  
    if (coreNone == numCoreCells){
	return "";
    }

    // check to see if a higher choice is used without a lower choice being
    // used.  Choice 0 does not matter because it is the choice for not using a
    // cell as a core cell choice.
    for (var i = numSelections; i > 0; --i){
	if (choices[i]){
	   for(var j = 1; j < i; ++j){
	     if(!choices[j]){
		return msg;
	     }
	   }
	}
    }

    return "";
}

/*
 * Adjust the failure usage options based on the user's cell type
 * selection.  If the users sets a cell's type to floating, only
 * give two failure usage options: "floating" or "do not use".
 *
 * @param type the user selection for the cell type
 * @param failureUsage the select object for setting the failure usage for
 *        this cell.
 * @return None.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect The failure usage choices for cells showing on the screen will
 *	       be modified.
 */
function adjustCellFailureUsageOptions(type, failureUsage){


    TYPE_FLOATING="1";

    // get the current user selected failure usage.
    var sFailureUsage = failureUsage.selectedIndex;

    if (type.selectedIndex == TYPE_FLOATING){

	failureUsage.options[1].text = "floating";
	failureUsage.options[1].value = FLOATING;
	failureUsage.options[0] = null;
	failureUsage.options[1] = null;
	// in this case, the user has changed the cell from type equals base
	// to type equals floating.  So a selected index of 2 would be 
	// "floating", and index of 3 would be "disabled".
	if (sFailureUsage == 2) {
	    sFailureUsage = 0;
	} else if (sFailureUsage == 3) {
	    sFailureUsage = 1;
	} else {
	    sFailureUsage = 0;
	}

    } else {

	failureUsage.options[0].text = "base";
	failureUsage.options[0].value = BASE;
	failureUsage.options[1].text = "base, no interleave";
	failureUsage.options[1].value = BASE_NO_INTERLEAVE;
	failureUsage.options[2] = new Option ("floating", FLOATING);
	failureUsage.options[3] = new Option ("disable", DISABLE);
	// in this case, the user has changed the cell from type equals floating
	// to type equals base.  So a selected index of 0 would be "floating", 
	// and index of 1 would be "disabled".
	if (sFailureUsage == 0) {
	    sFailureUsage = 2;
	} else if (sFailureUsage == 1) {
	    sFailureUsage = 3;
	} else {
	    sFailureUsage = 0;
	}
    }

    failureUsage.selectedIndex = sFailureUsage;
}

/*
 * Round a memory amount to the nearest .5 GB, for example:
 *	3.2 -> 3.0
 *	3.4 -> 3.5
 *
 * @param amount the number that should be rounded.
 * @return a float number that is rounded to the nearest .5 .
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
function roundMemAmount(amount){

    return Math.round(amount * 2) / 2.0;
}

/*
 * Format a number to a floating point number by * adding ".0" to the number 
 * if the number is a whole number. This is because * a float such as "3.0" 
 * will print to the screen as "3" by javascript.
 *
 * @param amount the number to be formatted.
 * @return a string holding a float number formatted to 1 decimal point.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
function formatFloat(amount){

    if (parseInt(amount) == parseFloat(amount)){

	return amount+".0";

    } else {

	return amount;
    }
}

/*
 * Update the values in the "CLM amount" column of a cells
 * table based on the index into a selection box.  It is
 * assumed that the selection box allow the user to choose
 * the ratio for CLM for all base cells in increments of 
 * an eight ("1/8").
 */
function updateFraction(tableName, selectedIndex, selectList){
    var CELL_TYPE_COL = 1;
    var CELL_TOTAL_MEM_COL = 2;
    var CELL_CLM_AMOUNT_COL = 3
    var fraction = selectList.options[selectedIndex].value;

    var myTable = eval(getDocument() + tableName);
    var re = /floating/;
    var totalInterleaved = document.memConfig.totalInterleaved;
    var totalBaseMem = eval(getDocument() + "totalBaseMem");

    var total = 0;

    //
    // Loop through the rows in the table
    //
    for (var i = 1; i < myTable.rows.length; i++){

	var r = myTable.rows[i];
	var cellType = r.cells[CELL_TYPE_COL].innerHTML;

        //
	// If the cell is a base cell update the
        // CLM Amount column.
	//
	if (cellType.search(re) == -1){

	    var CLM = parseFloat(r.cells[CELL_TOTAL_MEM_COL].innerHTML) * 
				    fraction / 8;
	    r.cells[CELL_CLM_AMOUNT_COL].innerHTML = formatFloat(CLM);

	} else {

	    total += parseFloat(r.cells[CELL_CLM_AMOUNT_COL].innerHTML);
	}
    }

    var totalFromBase = parseFloat(totalBaseMem.innerHTML) * fraction / 8;
    totalInterleaved.value = formatFloat(parseFloat(totalBaseMem.innerHTML) -
			     totalFromBase);

    total += totalFromBase;
    setTotalCLM(myTable, total);
}

//
// Update the values in the "CLM Amount" column of a cells
// table based on input into a text box indicating the amount
// of CLM for each base cell.
//
function updateTotals(tableName, amountStr){
    var CELL_TYPE_COL = 1;
    var CELL_TOTAL_MEM_COL = 2;
    var CELL_CLM_AMOUNT_COL = 3

    var myTable = eval(getDocument() + tableName);
    var totalInterleaved = document.memConfig.totalInterleaved;
    var totalBaseMem = eval(getDocument() + "totalBaseMem");

    // round to nearest .5 GB
    var amount = roundMemAmount(amountStr);
    var re = /floating/;

    // keep a running total based on user settings.
    // by pass header row
    var total = 0;
    var totalFromBase = 0;

    //
    // Loop through each cell row.
    //
    for (var i = 1; i < myTable.rows.length; i++){

	var r = myTable.rows[i];
	var cellType = r.cells[CELL_TYPE_COL].innerHTML;

	//
	// Set the "CLM Amount" column for base cells only.
	//
	if (cellType.search(re) == -1){

	    var cellMem=parseFloat(r.cells[CELL_TOTAL_MEM_COL].innerHTML);

	    //
	    // If the amount of memory on the cell is greater
	    // than the amount specified for CLM then use
	    // the amount specified.  Otherwise, use on the 
	    // memory on the cell.
	    //
	    if (cellMem > amount){

		r.cells[CELL_CLM_AMOUNT_COL].innerHTML=formatFloat(amount);
		totalFromBase += amount;

	    } else {

		r.cells[CELL_CLM_AMOUNT_COL].innerHTML=formatFloat(cellMem);
		totalFromBase += cellMem;
	    }

	} else {

	    // add the floating cells memory to total CLM amount
	    total += parseFloat(r.cells[CELL_CLM_AMOUNT_COL].innerHTML);
	}
    }

    totalInterleaved.value = formatFloat(parseFloat(totalBaseMem.innerHTML) -
			     totalFromBase);

    total += totalFromBase;
    setTotalCLM(myTable, total);
}

//
// Get the total CLM value by reaping it from the table's caption.
//
function getTotalCLM(tableName){

    var myTable = eval(getDocument() + tableName);
 
    // CLM value will be in float format
    var re=/\d+\.\d+/;

    // Get the caption
    var tableCaption = myTable.caption;
    var totalCLMCaption = tableCaption.innerHTML;

    // The the numerical part of the caption
    var totalCLMStr = totalCLMCaption.match(re);

    return parseFloat(totalCLMStr[0]);
}

//
// Set the total CLM value for a table
//
function setTotalCLM(myTable, totalCLM){

    myTable.caption.innerHTML = "total amount of CLM: " + 
			formatFloat(totalCLM) + " GB";
}
    
// invoked by the onFocus event for entering per cell CLM. It remembers the
// old per cell CLM amount and saves it in a global for use later.
var gCellAmount = 0.0;
function store_cell_amount(amountStr){
    gCellAmount = parseFloat(amountStr);
}

//
// Update the total CLM value for the table based on
// changes in the CLM ratio column in a cells table.
//
function updatePerCellTotals(tableName, amountStr){ 

    var myTable = eval(getDocument() + tableName);
    var amount = parseFloat(amountStr); 
    var totalInterleaved = document.memConfig.totalInterleaved;

    // get new total interleaved amount
    var newTotalInterleaved = parseFloat(totalInterleaved.value) + 
			      gCellAmount - amount;
    totalInterleaved.value = formatFloat(newTotalInterleaved);

    // get new CLM total
    var totalCLM = getTotalCLM(tableName) - gCellAmount + amount;
    setTotalCLM(myTable, totalCLM);
}

// invoked by the onFocus event for entering per cell CLM. It remembers the
// old per cell CLM ratio and saves it in a global for use later.
var gFractionIndex;
function store_cell_fraction(fractionIndex) {
    gFractionIndex = fractionIndex;
}

//
// Update the value in the "CLM Amount" column of a cells
// table based on changes to the "CLM Ratio".  Also, update
// the total CLM amount for the entire table.
//
function updatePerCellFraction(selectList, fractionIndex, rowIndex){
    var fraction = selectList.options[fractionIndex].value;

    var myTable = eval(getDocument() + "cellRatioTable");
    var myRow = myTable.rows[rowIndex];
    var cellTotalMem = parseFloat(myRow.cells[2].innerHTML);
    var newCellCLM = cellTotalMem * fraction / 8.0;

    gCellAmount = parseFloat(myRow.cells[4].innerHTML);
    myRow.cells[4].innerHTML = formatFloat(newCellCLM);

    updatePerCellTotals('cellRatioTable', newCellCLM);
}

function validateCLMSetting(totalStr, memStr){
    var msg="The amount of total interleaved memory requested is greater than the amount of total memory from base cells.";
    var totalBaseMem = parseFloat(totalStr);
    var totalInterleaved = parseFloat(memStr);
    if (totalBaseMem < totalInterleaved){
	alert(msg);
	return false;
    }

    return true;
}

//
// Validate the value in the text box in the CLM Amount column
// of a cells table.  If the CLM Amount is greater than the
// total memory on the cell change the CLM Amount to that value.
//
function validateCLMAmount(total, mem){

    var totalMem = parseFloat(total);
    var newMem = roundMemAmount(mem.value);

    if ((newMem < 0) || isNaN(newMem)){
	mem.value = formatFloat(0);
	return;
    }

    if (newMem > totalMem){
	mem.value = formatFloat(totalMem);
    } else {
	mem.value = formatFloat(newMem);
    }
}

//
// Helper function to show an object.
//
function showObject(optionName){
    var opt;
    if (isIE()){
	opt = eval(getDocument() + optionName);
    } else {
	opt = document.getElementById(optionName);
    }
    opt.className = 'mem-show';
}

//
// Helper function to hide an object.
//
function hideObject(optionName){
    var opt;
    if (isIE()){
	opt = eval(getDocument() + optionName);
    } else {
	opt = document.getElementById(optionName);
    }
    opt.className = 'mem-hide';
}

//
// Restore the total amount of interleaved memory on a document
// to the default value (i.e. the total amount of base memory).
//
function restoreTotalInterleaved(){
    var totalBaseMem = eval(getDocument() + "totalBaseMem");
    document.memConfig.totalInterleaved.value=totalBaseMem.innerHTML;
}

//
// Restore the total CLM amount for a table.
//
function restoreTotalCLM(tableName){

    var myTable = eval(getDocument() + tableName);
    var totalCLM = getTotalCLM('cellTable');

    setTotalCLM(myTable, totalCLM);
}

//
// Restore all the CLM amounts and fractions to zero.
//
function restoreSameForAll(){
    document.memConfig.clmFraction.selectedIndex = 0;
    document.memConfig.clmAmount.value = formatFloat(0.0);
}

//
// Restore all the per cell amounts in a cell memory
// table to zero.
//
function restoreCellAmountTable(){

    var re=/clmAmount\d+/;
    var elements = document.memConfig.elements;
    var eLength = elements.length;

    for (var i = 0; i < eLength; ++i){

	var name = elements[i].name;

	if (name.search(re) != -1){
	    elements[i].value = formatFloat(0);
	}
    }
}

//
// Restore all the per cell ratios in a cell memory
// table.
//
function restoreCellRatioTable(){

    var re=/clmFraction\d+/;
    var elements = document.memConfig.elements;
    var eLength = elements.length;

    for (var i = 0; i < eLength; ++i){

	var name = elements[i].name;

	if (name.search(re) != -1){

	    elements[i].selectedIndex = 0;
	    var row = elements[i].parentNode.parentNode;
	    row.cells[4].innerHTML = formatFloat(0);
	}
    }
}

//
//  Restore all the default values on a "specify CLM" page.
//
function restoreDefaultValues(){

    restoreTotalInterleaved();
    restoreSameForAll();

    // erases all the cell CLM amounts in the cellTable.  This has the
    // side effect of calculating the default total CLM and total interleaved
    // memory amounts.
    updateFraction('cellTable', 0, document.memConfig.clmFraction);
    restoreCellRatioTable();
    restoreCellAmountTable();
    restoreTotalCLM('cellRatioTable');
    restoreTotalCLM('cellAmountTable');
}

function changeOptionConfirm(){
    var msg = "Changing options will erase current memory settings.";
    return confirm(msg);
}

//
// Adjust the available options based on whether the user
// is specifying CLM values for cells individually or
// as a whole.
//
function showHideOptions(radioElement){
    if (radioElement.checked){
	// this element is already selected, there is no change in
	// selection, just return.
	return;
    }

    radioElement.checked = true;

    if (document.memConfig.sameForAll[0].checked) {

	showObject("cellTableGroup");
	hideObject("cellRatioTableGroup");
	hideObject("cellAmountTableGroup");

	if (document.memConfig.useFraction[1].checked){

	    showObject("clmFractionGroup");
	    hideObject("clmAmountGroup");
	} else {

	    hideObject("clmFractionGroup");
	    showObject("clmAmountGroup");
	}

    } else {

	hideObject("clmFractionGroup");
	hideObject("clmAmountGroup");
	hideObject("cellTableGroup");

	if (document.memConfig.useFraction[1].checked){

	    showObject("cellRatioTableGroup");
	    hideObject("cellAmountTableGroup");
	} else {

	    hideObject("cellRatioTableGroup");
	    showObject("cellAmountTableGroup");
	}

    }
    restoreDefaultValues();
}
//
// Show the advanced options on the "specify CLM" page.
//
function showAdvanced(){
    if (!changeOptionConfirm()){
	return;
    }

    hideObject("showAdvanced");
    showObject("advancedOptions");
    showObject("hideAdvanced");
    restoreTotalInterleaved();

    // also set the total interleaved memory input box to readonly.
    document.memConfig.totalInterleavedMem.readOnly = true;
    document.memConfig.useAdvanced.value = "true";
}

//
// Hide the advanced options on the "specify CLM" page.
//
function hideAdvanced(){
    if (!changeOptionConfirm()){
	return;
    }

    showObject("showAdvanced");
    hideObject("advancedOptions");
    hideObject("hideAdvanced");
    restoreDefaultValues();
    document.memConfig.totalInterleavedMem.readOnly = false;
    document.memConfig.useAdvanced.value = "false";
}

//
// used to display any errors from cell selection checks.
//
function displayErr(pageNumber, fromURL){
    var msg = document.forms[0].errorMsg.value;

    if ((msg != "null") && (msg.length != 0)){
	alert(msg);
	// the only error we can encounter when the first page loads is
	// there is no free viable core.  Need to cancel out of the 
	// crate npar task.
	if (pageNumber == 1){
	    window.location.replace(fromURL);
	}
    }
}

// myDoc is a global for referring to the current document
var myDoc="";
function getDocument(){
    var NETSCAPE = "Netscape";
    if (myDoc===""){
	if (isIE()){
	    myDoc = "document.all.";
	} else {
	    myDoc = "document.";
	}
    }

    return myDoc;
}

var ieFlag="";
function isIE(){
    var NETSCAPE = "Netscape";

    if (ieFlag == ""){
	if (navigator.appName == NETSCAPE){
	    ieFlag = false;
	} else {
	    ieFlag = true;
	}
    } 

    return ieFlag;
}
