//
//Filename: $Source: /usr/local/vse/glib/src/web/js/hierTableG.js,v $
//Version:  $Revision: 1.2 $
//Date:     $Date: 2007/06/14 20:24:11 $
//
//(C) Copyright 2007 Hewlett-Packard Development Company, L.P.
//

// This file lives on the CMS in /opt/hpwebadmin/webapps/vse.war/js/hierTableG.js

// GLOBALS

var PLUS_IMAGE  = "/images/tree_open.gif";
var MINUS_IMAGE = "/images/tree_close.gif";
var DEPTH_COLUMN = 0;
var STATE_COLUMN = 1;
var STATE_CONTRACTED = "1";
var STATE_EXPANDED   = "0";
var INVALID_TEXT = "#INVALID_TEXT#";
var INVALID_DEPTH = -1;
var INVALID_STATE = -1;
var TOP_LEVEL_DEPTH = 0;

function dbg(string) {
	alert("dbg> " + string);
}

function elementById(id) {
        var elem = document.getElementById(id); // no N4,IE4
        if (elem !== null) {
                return elem;
        }
        if (document.all !== null) {
                return document.all[id]; // IE only
        }
        if (document.layers !== null) {
                return document.layers[id]; // N4 only
        }
        return null;
}

// Given an element, find the nearest parent of the specified type.
// Returns the element found, or null.
// If the element passed in is of the specified type, that is NOT a match:
// the search begins with the parent of the element passed in.
function findAncestorElementByType(elem, type)
{
	var tagName;
	elem = elem.parentNode;
	while (elem !== null) {
		tagName = elem.tagName;
		if (tagName == type) {
			return elem;
		}
		elem = elem.parentNode;
	}
	return null;
}

// Returns 1 if there is a leading selector (radio button/checkbox).
// Else returns 0.
// This allows other methods to skip past the selector colum by
// incrementing a column number: columnNumber += radioOffset(row).
// Note: we can be sure that if column zero contains
// an INPUT, this is a selector (radio button or checkbox).
// If there were not a selector, the first column would contain 
// the depth of the row.
function radioOffset(row) {
	var cells = getCellsOfRow(row);
	if (cells.length <= 0) {
		return 0;
	}

	var inputs = cells[0].getElementsByTagName('INPUT');
	if (inputs.length == 1) {
		return 1;
	} else if (inputs.length == 0) {
		return 0;
	} else {
		alert("cell 0 of this row contains more than one INPUT: " + row.id);
		return 0;
	}
}

// Given a row of the HierTableG, find the cell in the row that
// contains a table with the id "hier_table_id". This is the 
// "button table", which contains the +/- button.
// We get here because the user clicked on the + or - button.
// Therefore, there must be a button table in this row.
function getButtonTable(row) {
	
	var tables = row.getElementsByTagName('TABLE');
	var table;
	var i;
	for (i = 0; i< tables.length; i++) {
		table = tables[i];
		if (table.id.lastIndexOf('HIER_TABLE_ID') > -1) {
			return table;
		}
	}
	alert("getButtonTable: cannot find the button table in this row: " + row.id);
	return null;
}

// We receive a ColumnTableG row and return the hier button image element.
function getButtonImageElement(row) {
	
	var table = getButtonTable(row);
	var images = table.getElementsByTagName('IMG');
	var image;
	var i;
	for (i = 0; i< images.length; i++) {
		image = images[i];
		if (image.id === 'hier_button') {
			return image;
		}
	}
	alert("getButtomImageElement: cannot find the button img in this table: " + table.id);
	return null;
}

// This function modifies the image (+/-) of the current row to use the 
// specified image.
function setRowHierarchyButtonImage(row, imageName) {
	var cells = row.getElementsByTagName('TD');
	if (cells.length <= 0) {
		return; // This is probably a Table Header (TH) row, not a TD row.
	}

	var img = getButtonImageElement(row);

	var src = img.src;
	if (src.indexOf(imageName) > -1) {
		// It's already the correct imageName.
		return;
	}
	if (imageName.indexOf(PLUS_IMAGE) > -1) {
		// We are contracting this row.
		src = src.replace(MINUS_IMAGE, imageName);
	} else if (imageName.indexOf(MINUS_IMAGE) > -1) {
		// We are expanding this row.
		src = src.replace(PLUS_IMAGE, imageName);
	}
	img.src = src;
}

// If the element specified is a row, return the same element (the row).
// Else, find the ancestor (up the tree) that is a row.
// This is used, for example, to find the row of the image the user clicked.
function getRowOfElement(elem) {
	if (elem.tagName == 'TR') {
		return elem;
	} else {
		return findAncestorElementByType(elem, 'TR');
	}
}

// Get the string from the indicated cell of the row.
// If there is a radio button in the zeroth column, skip over it.
// Used by getRowDepth and getRowState.
function getRowCellText(row, cellIndex) {
	if (row === null) {
		return INVALID_TEXT;
	}

	var cells = row.getElementsByTagName('TD');
	if (cells.length <= 0) {
		return INVALID_TEXT;
	}
	var text = cells[cellIndex + radioOffset(row)].innerHTML;
	if (text === null || text === "") {
		return INVALID_TEXT;
	} else {
		return text;
	}
}
 
// Given a table row element, fetch the value of the depth cell.
// This holds the numeric "depth" of the row.
// Depth is zero-based. 
function getRowDepth(row) {
	var text = getRowCellText(row, DEPTH_COLUMN);
	if (text === INVALID_TEXT) {
		return INVALID_DEPTH;
	} else {
		return text;
	}
}

// Given a table row element, fetch the value of the state cell.
// This holds the boolean "state" of the row (either STATE_COLLAPSED
// or STATE_EXPANDED).
function getRowState(row) {
	var text = getRowCellText(row, STATE_COLUMN);
	if (text === INVALID_TEXT) {
		alert("getRowState invalid 1");
		return INVALID_STATE;
	} else {
		if (text != STATE_COLLAPSED && text != STATE_EXPANDED) {
			alert("getRowState invalid 2");
			return INVALID_STATE;
		} else {
			return text;
		}
	}
}

// Given a table row element, and the row state (either STATE_COLLAPSED or 
// STATE_EXPANDED), set the value of the state cell.
function setRowState(row, rowState) {
	if (row === null) {
		return;
	}

	var cells = row.getElementsByTagName('TD');
	if (cells.length <= 0) {
		return;
	}
	var cell = cells[STATE_COLUMN + radioOffset(row)];
	var inputs = cell.getElementsByTagName('INPUT');
	var input = inputs[0];
	input.setAttribute("value", rowState);
}

// Given a TR element, find the next TR element in the same table.
// Return null if no next TR exists.
function getNextRow(row) {
	while ((row = row.nextSibling) !== null) {
		if (row.tagName == "TR") {
			return row;
		}
	}
	return row; // null
}

// Recursive function that contracts the given row and all
// of its subordinate rows (those with greater depth).
function contractRow(row) {
	var nextRow = getNextRow(row);
	var currDepth = getRowDepth(row);
	var nextDepth = getRowDepth(nextRow);
	
	while (nextDepth > currDepth) {
		contractRow(nextRow);
		setRowHierarchyButtonImage(row, PLUS_IMAGE);
		nextRow = getNextRow(nextRow);
		nextDepth = getRowDepth(nextRow);
	}
	setRowState(row, STATE_CONTRACTED);
	row.style.display = 'none';
}

// Recursive function that expands the given row and all
// of its subordinate rows (those with greater depth).
function expandRow(row) {
	var nextRow = getNextRow(row);
	var currDepth = getRowDepth(row);
	var nextDepth = getRowDepth(nextRow);
	
	while (nextDepth > currDepth) {
		expandRow(nextRow);
		setRowHierarchyButtonImage(row, MINUS_IMAGE);
		nextRow = getNextRow(nextRow);
		nextDepth = getRowDepth(nextRow);
	}
	setRowState(row, STATE_EXPANDED);
	row.style.display = '';
}

// Given an IMG element, see if it is expanded or contracted.
// We determine this by looking at the name of the image that the
// user clicked on.
function isExpanded(img) {
	if (img.src.indexOf(PLUS_IMAGE) > -1) {
		return false;
	} else {
		return true;
	}
}

// Given a TR element, return an array of all the TD elements.
function getCellsOfRow(row) {
	var myArray = new Array();
	var nodes = row.childNodes;
	var i;
	var index = 0;
	var elem;
	var tagName;
	for (i = 0; i < nodes.length; i ++) {

		elem = nodes[i];
		tagName = elem.tagName;
		if (tagName == 'TD') {
			myArray[index++] = elem;
		}
	}

	return (myArray);
}

// Given a table element, return an array of all the TR elements.
function getRowsOfTable(table) {
	var tbodies = table.getElementsByTagName('TBODY');
	var tbody = tbodies[0];
	var myArray = new Array();
	var nodes = tbody.childNodes;
	var i;
	var index = 0;
	var elem;
	var tagName;
	for (i = 0; i < nodes.length; i ++) {

		elem = nodes[i];
		tagName = elem.tagName;
		if (tagName == 'TR') {
			myArray[index++] = elem;
		}
	}
	return (myArray);
}

// Either expand or contract all rows in the table.
// You get here if you click on "++" or "--".
function changeExpansionAllRows(tableId, direction) {
	// If a (TR) row.style.display="", it's visible.
	// If it is  row.style.display="none", it's hidden.
	var table = elementById(tableId);
	var rows = getRowsOfTable(table);
	var row;
	var depth;
	for (var i=0; i < rows.length; i++) {
		row = rows[i];
		depth = getRowDepth(row);
		if (depth == TOP_LEVEL_DEPTH) {
			if (direction == "contract") {
				contractRow(row);
				// Make this top-level row visible, even though all rows 
				// subordinate to it are hidden.
				row.style.display = '';
			} else if (direction == "expand") {
				expandRow(row);
			} else {
				alert("changeExpansionAllRows, invalid direction: " + direction);
			}
		}
	}
}

// This is the function that is invoked when the user clicks on the 
// "hierarchy image", that is, a "+" or "-" button on a HierTableG row.
function toggleVisibilityRows(img) {
	// Get the row containing this image.  This is the row of the 
	// button table (containing just the +/- image and the user's text).
	var row = getRowOfElement(img);
	// Get the row containing the button table. This is a HierTableG row.
	row = getRowOfElement(row.parentNode);
	if (isExpanded(img)) {
		contractRow(row);
		// Make this row visible, even though all rows 
		// subordinate to it are hidden. When you click on a row, you
		// show or hide the rows subordinate to it.
		row.style.display = '';
	} else {
		expandRow(row);
	}
}

// This method is invoked when the user clicks on the "expand all" button 
// in the table's Caption.
function expandAllRows(tableId) {
	changeExpansionAllRows(tableId, "expand");
}

// This method is invoked when the user clicks on the "contract all" button 
// in the table's Caption.
function contractAllRows(tableId) {
	changeExpansionAllRows(tableId, "contract");
}
