/*
 * (C) Copyright 2005-2006 Hewlett-Packard Development Company, L.P.
 */
function TableManager() {
        // a global var holding a roster of all scrollableTables onscreen. Used in windowResize, for the following reason.  If a tableis scrolled horizontally, and the window is widened, you basically need this to realign the columns as the table is sort of unscrollbarring itself.  onscroll on the div will not fire for us, although you might think it should.
        this.scrollableTableIds = new Array();
}

/** Initializes all tables on the page */
TableManager.prototype.init = function() {
    tableNodeCollection = window.document.getElementsByTagName("table");
    for (var i=0; i<tableNodeCollection.length; i++ ) {
        var table = tableNodeCollection[i];
        if (table.className.indexOf("dataTable")!=-1) {
            this.initTable(table);
            // dealing with scrollingTables
            if (table.className.indexOf("scrollingTable")!=-1) {
                var tableId = table.getAttribute('id');
                // file it away for use by the windowResize function
                this.scrollableTableIds[this.scrollableTableIds.length] = tableId;
                // this is the big function that aligns the divs with the table columns
                table.initialWidth = table.offsetWidth;
                this.resizeTableColumns(tableId);
                // a nice way to leverage anonymous functions to allow us to keep event handler code out of the html. A nice side effect is that event handlers arent in place until the page is loaded, at which point they generally also become safe to run.
                eval("document.getElementById(tableId + '_scrollingTableDiv').onscroll=function() {ourTableManager.slideColumns('"+tableId+"')}");
            }
        }
    }
}

/** Initializes a table */
TableManager.prototype.initTable = function(table) {

    // attach event handler to "check all"
    var chk = this.getToggleAllSelector(table);
    if(chk != null) {
        chk.onclick = function(mozEvent) {ourTableManager.tableCheckboxToggleAll(mozEvent, this);};
    }

    // attach event handlers to the row and the row selector
    var rows = this.getSelectableRows(table);
    for (var i=0; i < rows.length; i++ ) {
        this.initTableRow(table, rows[i]);
    }

    this.updateToggleAllState(table);

    // extract the selection notifier.
    // AOR: is it ok to call it before all tables are initialized?
    var selNotifierText = table.getAttribute("selectionNotifier");
    if(selNotifierText != null && selNotifierText != undefined) {
        var onSelectionChange;
        eval("onSelectionChange = function(table) {" + selNotifierText + "}");
        table.setAttribute("onSelectionChange", onSelectionChange);
    }

    // attach event listeners to table headers
    var headers = table.getElementsByTagName("TH");
    for(var i = 0; i < headers.length; i++) {
        var header = headers[i];
        if(header.getAttribute("sortable") == "true")
            header.onclick = function(mozEvent) {return ourTableManager.processHeaderClick(mozEvent, this);};
    }
}

/**
 * Attaches event listeners to a table row. This method should be called once for
 * every existing table row and for every dynamically-added table row.
 */
TableManager.prototype.initTableRow = function(table, row) {
    var rowSelector = this.getRowSelector(row);
    rowSelector.onclick = function(mozEvent) {return ourTableManager.onRowSelectorValueChange(this, true);};
    row.onclick = function(mozEvent) {return ourTableManager.processRowClick(mozEvent, this);};
    if (this.isPropertyTable(table)) {
        if (!document.all) {
            // an elegant way to disable ctrl-click on these elements for Mozilla browsers.
            // IE unfortunately has to use less subtle methods, disabling all ctrl-selection by setting document.onselectstart to return false if CTRL key is pressed. This occurs in global.js if the TableManager is loaded.
            row.onmousedown = function(mozEvent) {return false;};
        }
    }
    // if the table is synchronized with the global selection
    // then update row selector state according to the global selection
    this.setRowSelectionState(table, rowSelector, this.isRowSelected(row));
    this.updateRowHighlight(row);

}

TableManager.prototype.processHeaderClick = function(mozEvent, header) {
    vsAssertTagName("TH", header);
    var table = header.parentNode.parentNode.parentNode;
    vsAssertTagName("TABLE", table);
    var sortOrderField = document.getElementById(table.id+".sortOrder");
    vsAssertTagName("INPUT", sortOrderField);
    vsAssertTypes(["hidden"], sortOrderField);
    var ascSortOrder = header.getAttribute("colIndex")+";"+"A";
    var descSortOrder = header.getAttribute("colIndex")+";"+"D";
    if(sortOrderField.value != ascSortOrder)
        sortOrderField.value = ascSortOrder;
    else
        sortOrderField.value = descSortOrder;
    document.mainForm.submit();
}

/** Updates global selection for the specified row */
TableManager.prototype.setRowSelectionState = function(table, rowSelector, selectionState) {
    var selectorType = rowSelector.getAttribute("type");
    if(table.getAttribute("modifySelection") == "true") {
        var id = rowSelector.value;
        var guid = rowSelector.getAttribute("guid");
        var objectTypes = rowSelector.getAttribute("selTypes").split(",");
        var label = rowSelector.getAttribute("selLabel");
        if(selectionState)
            selectionAdd(id, guid, objectTypes, label);
        else
            selectionRemove(id, guid, objectTypes, label);
    }
}

TableManager.prototype.getSelectionEntry = function(rowSelector) {
    var id = rowSelector.value;
    var guid = rowSelector.getAttribute("guid");
    var objectTypes = rowSelector.getAttribute("selTypes").split(",");
    var label = rowSelector.getAttribute("selLabel");
    return selectionEntry(id, guid, objectTypes, label);
}

/** Retturns the state of row selection in the global (aka "VSE") selection */
TableManager.prototype.getRowSelectionState = function(row) {
    var rowSelector = this.getRowSelector(row);
    var id = rowSelector.value;
    var guid = rowSelector.getAttribute("guid");
    var objectTypes = rowSelector.getAttribute("selTypes").split(",");
    return isSelected(id, guid, objectTypes);
}

TableManager.prototype.clearSelectionState = function(table) {
    // attach event handlers to the row and the row selector
    var rows = this.getSelectableRows(table);
    for (var i=0; i < rows.length; i++ ) {
        this.setRowSelectionState(table, this.getRowSelector(rows[i]), false);
    }
}

TableManager.prototype.selectionChangedNotifiersCalling = false;


/**
 * This method is called when row selector value is changed
 * @param rowSector The row selector
 * @param canSubmit if true and the table has "submitOnClick" attribute set then
 *                  this event will cause the form to be submitted. This parameter
 *                  need to be false if the row change is part of a transaction
 *                  or when table selection is synchronized with the global selection.
 */
TableManager.prototype.onRowSelectorValueChange = function(rowSelector, canSubmit) {
    var rowElement = this.getRowElement(rowSelector);
    var table = this.getTable(rowElement);
    var selectorType = rowSelector.getAttribute("type");

    // if this is a radio button then we don't know which row was unselected so we
    // update global selection for all table rows
    if(selectorType == "radio") {
        var rows = this.getSelectableRows(table);
        for (var i=0; i < rows.length; i++ ) {
            var rs = this.getRowSelector(rows[i]);
            this.setRowSelectionState(table, rs, rs.checked);
            this.updateRowHighlight(rows[i]);
        }
    } else {
        this.setRowSelectionState(table, rowSelector, rowSelector.checked);
        this.updateRowHighlight(rowElement);
    }
    this.updateToggleAllState(table);

    if(canSubmit) {
        // table selection might need to be synchronized back with global selection after
        // calls to setRowSelectionState()
        if(!selectionChangedCalling)
            selectionChangedNotifiers(null);
        this.fireSelectionChangeEvent(table);
        this.submitOnClick(table);
    }
}

TableManager.prototype.submitOnClick = function(table) {
    if(table.getAttribute("submitOnClick") == "true")
        document.mainForm.submit();
}

// Release event handlers put in place by the init function
// NOTE: any changes to init() or rowInit() should be mirrored here
TableManager.prototype.unInit = function() {
    tableNodeCollection = window.document.getElementsByTagName("table");
    for (var i=0; i<tableNodeCollection.length; i++ ) {
        var table = tableNodeCollection[i];
        if (table.className.indexOf("dataTable")!=-1) {
            this.unInitTable(table);
            if (tableNodeCollection[i].className.indexOf("scrollingTable")!=-1) {
                    var tableId = tableNodeCollection[i].getAttribute('id');
                    document.getElementById(tableId + '_scrollingTableDiv').onscroll=null;
            }
        }
    }
}

TableManager.prototype.unInitTable = function(table) {
    var chk = this.getToggleAllSelector(table);
    if(chk != null) {
        chk.onclick = null;
    }

    // remove event handlers to the row and the row selector
    var rows = this.getSelectableRows(table);
    for (var i=0; i < rows.length; i++ ) {
        this.unInitTableRow(table, rows[i]);
    }

    table.setAttribute("onSelectionChange", null);

    // remove event listeners to table headers
    var headers = table.getElementsByTagName("TH");
    for(var i = 0; i < headers.length; i++) {
        var header = headers[i];
        header.onclick = null;
    }
}

TableManager.prototype.unInitTableRow = function(table, row) {
    var rowSelector = this.getRowSelector(row);
    rowSelector.onclick = null;
    row.onclick = null;
    row.onmousedown = null;
}

TableManager.prototype.tableCheckboxToggleAllCalling = false;

/** triggered by onclick on the master checkbox itself. */
TableManager.prototype.tableCheckboxToggleAll = function(mozEvent, masterCheckbox) {
    var tableid = masterCheckbox.getAttribute("tableid");
    vsAssert(tableid != null && tableid != undefined, "no tableid");
    var table = this.getTableByName(tableid);
    var rows = this.getSelectableRows(table);
    var changed = false;
    selectionBeginBulkUpdates();
    this.tableCheckboxToggleAllCalling = true;
    try {
        for(var i = 0; i < rows.length; i++) {
            var rowSelector = this.getRowSelector(rows[i]);
            if(this.setRowSelected(rows[i], masterCheckbox.checked, false))
            changed = true;
        }
    } finally {
        this.tableCheckboxToggleAllCalling = false;
        selectionEndBulkUpdates();
    }
    if(changed) {
        this.fireSelectionChangeEvent(table);
        this.submitOnClick(table);
    }
}

/**
 * Called by onclick on the table row.
 * This function will only be attached as an onclick handler to a row if either it contains a rowselector
 * form element, or if the parent table is of class "propertyViewTable"
 */
TableManager.prototype.processRowClick = function(mozEvent,rowElement) {
        var eventSource = getEventOriginator(mozEvent);
        // a list of various elements within tree rows upon which clicks should not toggle highlight state.
        if (eventSource.tagName=="IMG" || eventSource.tagName=="A" || eventSource.tagName=="DIV" ||
            eventSource.tagName == "INPUT" || eventSource.tagName == "SELECT" || eventSource.tagName == "LABEL" || ((eventSource.tagName=="DIV") && (eventSource.className=="treeControl"))) {
            return;
        }
	
        // If the row contains the noselect attribute, then never select this row.
        // This feature introduced for HierTableG.
        if (rowElement.getAttribute('noselect') == "true") {
            return;
        }

        var table = this.getTable(rowElement);
        var rowSelector = this.getRowSelector(rowElement);
        var selectorType = rowSelector.getAttribute("type");

        if (selectorType == "checkbox") {
            // manually flip the checkbox.
            this.setRowSelected(rowElement, !this.isRowSelected(rowElement), true);
        } else if (selectorType == "radio") {
            // manually turn on the radioButton if it isn't on.
            var changed = this.setRowSelected(rowElement, true, false);
            if(changed) {
                // need to update other rows because the previous statement implicitly unselected another row
                this.updateHighlight(table);
                this.fireSelectionChangeEvent(table);
                this.submitOnClick(table);
            }
        } else if (selectorType == "hidden") {
            var ctrlKey = (document.all) ? window.event.ctrlKey :mozEvent.ctrlKey;
            if (!ctrlKey) {  // clear the selection and select this row
                this.clearSelection(table, false);
                this.setRowSelected(rowElement, true, false);
            } else {        // flip the selection of the row
                this.setRowSelected(rowElement, !this.isRowSelected(rowElement), false);
            }
            this.fireSelectionChangeEvent(table);
            this.submitOnClick(table);
        }
}

/** Returns the row selector of the row */
TableManager.prototype.getRowSelector = function(rowElement) {
    var inputNodeCollection = rowElement.getElementsByTagName("INPUT");
    for (var i=0; i<inputNodeCollection.length; i++ ) {
        if (inputNodeCollection[i].getAttribute("rowselector")=="yes") {
            vsAssertTypes(["checkbox", "radio", "hidden"],inputNodeCollection[i]);
            return inputNodeCollection[i];
        }
    }
    return null;
}

/** Returns the row element of the row selector */
TableManager.prototype.getRowElement = function(rowSelector) {
    vsAssertTagName("INPUT", rowSelector);
    vsAssertTypes(["checkbox", "radio", "hidden"],rowSelector);
    var rowElement = rowSelector.parentNode.parentNode;
    vsAssertTagName("TR",rowElement);
    return rowElement;
}

/** Returns the table for the row */
TableManager.prototype.getTable = function(rowElement) {
    vsAssertTagName("TR",rowElement);
    var table = rowElement.parentNode.parentNode;
    vsAssertTagName("TABLE",table);
    return table;
}

/** Returns all table rows that have row selectors */
TableManager.prototype.getSelectableRows = function(table) {
    vsAssertTagName("TABLE",table);
    var result = new Array();
    var rows = table.getElementsByTagName("TR");
    for (var i = 0; i < rows.length; i++) {
        if (this.getRowSelector(rows[i]) != null) {
            result[result.length] = rows[i];
        }
    }
    return result;
}

/** Returns all table rows that have row selectors */
TableManager.prototype.getToggleAllSelector = function(table) {
    vsAssertTagName("TABLE",table);
    var input = document.getElementById(table.id + "_headerButton");
    return input;
}

/**
 * Clears the selection.
 * @return true if table selection has changed
 */
TableManager.prototype.clearSelection = function(table, canSubmit) {
    vsAssertTagName("TABLE",table);
    var rows = this.getSelectableRows(table);
    var changed = false;
    for (var i = 0; i < rows.length; i++) {
        if(this.setRowSelected(rows[i], false, false))
            changed = true;
    }
    if(changed && canSubmit) {
        this.fireSelectionChangeEvent(table);
        this.submitOnClick(table);
    }
    return changed;
}

/** Updates table row hilighting after an event for a row element */
TableManager.prototype.updateHighlight = function(table) {
    var rowElements = this.getSelectableRows(table);
    for(var i = 0; i < rowElements.length; i++)
        this.updateRowHighlight(rowElements[i]);
}

/** Changes row color based on the value of row selector */
TableManager.prototype.updateRowHighlight = function(rowElement) {
    vsAssertTagName("TR", rowElement);
    if(this.isRowSelected(rowElement)) {
        appendClassName(rowElement, "rowHighlight");
    } else {
        removeClassName(rowElement, "rowHighlight");
    }
}

/** Returns true if the table is a property table */
TableManager.prototype.isPropertyTable = function(table) {
    vsAssertTagName("TABLE",table);
    return table.className.indexOf("propertyViewTable") >= 0;
}

//function isCaller(f) {
//    var elem = arguments.callee.caller;
//    while(elem != null) {
//        if(elem == f)
//            return true;
//		elem = elem.caller;
//    }
//    return false;
//}
//
///** AOR: this is essentially commented out for debugging */
//function isCaller0(f) {
//    for(var elem = arguments.callee.caller; elem != null; elem = elem.caller) {
//        if(elem == f)
//            return true;
//    }
//    return false;
//}

/** */
TableManager.prototype.isRowSelected = function(rowElement) {
    var rowSelector = this.getRowSelector(rowElement);
    return rowSelector != null && rowSelector.checked;
}

/**
 * Changes row selection. Returns true if row selection has changed.
 * @pram canSubmit if true then onSelectionChange event will be called.
 * @return true if table selection has changed
 */
TableManager.prototype.setRowSelected = function(rowElement, selected, canSubmit) {
    var rowSelector = this.getRowSelector(rowElement);
    if(rowSelector != null && rowSelector.checked != selected) {
        rowSelector.checked = selected;
        this.onRowSelectorValueChange(rowSelector, canSubmit);
        return true;
    } else {
        return false;
    }
}

TableManager.prototype.updateToggleAllState = function(table) {
    vsAssertTagName("TABLE",table);
    var toggleAll = this.getToggleAllSelector(table);
    if(toggleAll == null)
        return;
    if(this.tableCheckboxToggleAllCalling)
        return;
    var rows = this.getSelectableRows(table);
    var allSelected = rows.length > 0;
    for (var i = 0; i < rows.length && allSelected; i++) {
        allSelected = this.isRowSelected(rows[i]);
    }
    toggleAll.checked = allSelected;
}

/** Fires selection change event */
TableManager.prototype.fireSelectionChangeEvent = function(table) {
    //var onSelectionChange = table.getAttribute("onSelectionChange");
    //if(onSelectionChange != null)
    //  onSelectionChange(table);
    var s = table.getAttribute("selectionNotifier");
    if(s != null)
        eval(s);
}

/** Returns true if table selection is empty. */
TableManager.prototype.isSelectionEmpty = function(table) {
    var rowElements = this.getSelectableRows(table);
    for(var i = 0; i < rowElements.length; i++)
        if(this.isRowSelected(rowElements[i]))
            return false;
    return true;
}


/*  SET OF FUNCTIONS FOR SCROLLING TABLES */

// Although in most cases of window resizing it's not necessary to recalculate column positions, if a table is partially scrolled horizontally, resizing the window can effectively scroll the div, and unfortunately in IE6 this does not trigger onscroll, so this amounts to a patch.
TableManager.prototype.windowResize = function() {
    // cut down the size of the scrolling divs, so that their containers have an opportunity to contract.
    // this is done through classnames liquidHeight and liquidWidth, on the div around a ScrollingTable.
    // two passes are made through.   On the first pass, heights and widths are temporarily set to lower values.
    // this gives containers on the page the opportunity to contract.
    for (var i=0; i<this.scrollableTableIds.length; i++) {
        var visualHeaderScrollingDiv    = document.getElementById(this.scrollableTableIds[i]+"_headerDiv");
        var tableScrollingDiv       = document.getElementById(this.scrollableTableIds[i]+"_scrollingTableDiv");

        if (tableScrollingDiv.className.indexOf("liquidHeight")!=-1) {
            tableScrollingDiv.style.height = Math.max(20,this.getContainersAvailableHeight(tableScrollingDiv) - visualHeaderScrollingDiv.offsetHeight-100);
        }
        if (tableScrollingDiv.className.indexOf("liquidWidth")!=-1) {
            var availableWidth = this.getContainersAvailableWidth(tableScrollingDiv);
            visualHeaderScrollingDiv.style.width = parseInt(availableWidth/2)+"px";
            tableScrollingDiv.style.width = parseInt(availableWidth/2)+"px";
        }
    }
    // now since all scrolling tabledivs have been shrunk a bit, we are guaranteed that containers have an opportunity to shrink.  (the troublesome case requiring these two loops to be seperate, is the case where a container has more than one scrollingTable in them.)
    // proceeding with setting widths and heights to actually match containers
    for (var i=0; i<this.scrollableTableIds.length; i++) {
        var visualHeaderScrollingDiv    = document.getElementById(this.scrollableTableIds[i]+"_headerDiv");
        var tableScrollingDiv       = document.getElementById(this.scrollableTableIds[i]+"_scrollingTableDiv");

        if (tableScrollingDiv.className.indexOf("liquidHeight")!=-1) {
            tableScrollingDiv.style.height = Math.max(20,this.getContainersAvailableHeight(tableScrollingDiv) - visualHeaderScrollingDiv.offsetHeight);
        }
        if (tableScrollingDiv.className.indexOf("liquidWidth")!=-1) {
            var availableWidth = this.getContainersAvailableWidth(tableScrollingDiv);
            visualHeaderScrollingDiv.style.width = availableWidth+"px";
            tableScrollingDiv.style.width = availableWidth+"px";
        }
        // you have to call this onresize, as there are a couple hard-to-find resize cases where it does effectively induce scrolling on the
        // div, but not all browsers will fire onscroll.    Fortunately it's not a very expensive operation.
        this.slideColumns(this.scrollableTableIds[i]);
    }
}

// checking tableScrollingDiv.parent's offsetWidth returns a number that doesnt account for the parent's border and padding.  This function attempts to determine that.
TableManager.prototype.getContainersAvailableWidth = function(element) {
    var parent = element.offsetParent;
    var paddingAndBorder =0;
    if (document.all)  {
        if (parseInt(parent.currentStyle.paddingLeft)) {
            paddingAndBorder += parseInt(parent.currentStyle.paddingLeft);
        }
        if (parseInt(parent.currentStyle.paddingRight)) {
            paddingAndBorder += parseInt(parent.currentStyle.paddingRight);
        }
        if (parseInt(parent.currentStyle.borderLeftWidth)) {
            paddingAndBorder += parseInt(parent.currentStyle.borderLeftWidth);
        }
        if (parseInt(parent.currentStyle.borderRightWidth)) {
            paddingAndBorder += parseInt(parent.currentStyle.borderRightWidth);
        }
    }
    else {
        paddingAndBorder =
            parseInt(document.defaultView.getComputedStyle(parent   , '').getPropertyValue("padding-left")) +
            parseInt(document.defaultView.getComputedStyle(parent   , '').getPropertyValue("padding-right")) +
            parseInt(document.defaultView.getComputedStyle(parent   , '').getPropertyValue("border-left-width")) +
            parseInt(document.defaultView.getComputedStyle(parent   , '').getPropertyValue("border-right-width"));
    }
    if (isNaN(paddingAndBorder)) paddingAndBorder = 0;
    return Math.max(0,parent.offsetWidth - paddingAndBorder);
}

// checking tableScrollingDiv.parent's offsetHeight returns a number that doesnt account for the parent'sborder and padding.  This function attempts to determine that.
TableManager.prototype.getContainersAvailableHeight = function(element) {
    var parent = element.offsetParent;
    var paddingAndBorder = 0;
    if (document.all)  {
        if (parseInt(parent.currentStyle.paddingTop)) {
            paddingAndBorder += parseInt(parent.currentStyle.paddingTop);
        }
        if (parseInt(parent.currentStyle.paddingBottom)) {
            paddingAndBorder += parseInt(parent.currentStyle.paddingBottom);
        }
        if (parseInt(parent.currentStyle.borderTopWidth)) {
            paddingAndBorder += parseInt(parent.currentStyle.borderTopWidth);
        }
        if (parseInt(parent.currentStyle.borderBottomWidth)) {
            paddingAndBorder += parseInt(parent.currentStyle.borderBottomWidth);
        }
    }
    else {
        paddingAndBorder =
            parseInt(document.defaultView.getComputedStyle(parent   , '').getPropertyValue("padding-left")) +
            parseInt(document.defaultView.getComputedStyle(parent   , '').getPropertyValue("padding-right")) +
            parseInt(document.defaultView.getComputedStyle(parent   , '').getPropertyValue("border-left-width")) +
            parseInt(document.defaultView.getComputedStyle(parent   , '').getPropertyValue("border-right-width"));
    }
    if (isNaN(paddingAndBorder)) paddingAndBorder = 0;
    return Math.max(1,parent.offsetHeight - paddingAndBorder);
}

// this is only called once from init, and doesnt need to run again.
// Note however, it runs once per ScrollingTable
TableManager.prototype.resizeTableColumns = function(tableId) {

    var tableDummyRow                   = document.getElementById(tableId+"_dummyRow");
    var visualHeaderDivParent   = document.getElementById(tableId+"_headerDiv");
    var visualHeaderDiv             = visualHeaderDivParent.childNodes[0];
    var table                               = document.getElementById(tableId);
    var tableScrollingDiv       = document.getElementById(tableId+"_scrollingTableDiv");

    // GUIDO CODE EDITED !!!
        // columnTableG formats the two divs nicely, which causes mozilla to see two extra children
        visualHeaderDiv = getChildNodebyClassName( visualHeaderDivParent.childNodes, "headerDivInner");


	// this will basically take the table's and headerDiv's 100% width styles, at whatever integer values they got at page load, and cast them as pixels.
	// note that the table value in particular will change later, as any long labels in visualHeaderDivs get factored into the column layout.
	table.style.width = table.offsetWidth+"px";
	// and we need to give the visualHeadeDiv's parent a fixed width too, because without that, it's overflow properties wont work, (thanks w3c).
	// Thus, if we waited until the end to do this, the little width assignments to it's visualHeader children would cause it to blow out its container.
	visualHeaderDivParent.style.width = visualHeaderDivParent.offsetWidth+"px";

	//	you need to set this to a high number, or else the visualHeaderDiv will start to wrap.
	visualHeaderDiv.style.width = 2*table.offsetWidth +"px";
	var maxScreenDivHeight = 0;
	for (var i=0; i<visualHeaderDiv.childNodes.length; i++) {
		if (visualHeaderDiv.childNodes[i].nodeType==1)  {
			maxScreenDivHeight = Math.max(maxScreenDivHeight,visualHeaderDiv.childNodes[i].offsetHeight);
		}
	}
	visualHeaderDiv.style.height = maxScreenDivHeight+"px";
	// need two extra pixels on the first one to get even.  One just cause the header border design has them one over,  and the other because the first headerDiv has no border, so needs one px extra width to line up.
	var extraWidthForFirstHeader = 2;
	// for the moment however, only IE really needs this.
	if (!document.all) extraWidthForFirstHeader = 0;
	// the purpose of this loop is to normalize the widths of table columns with their respective header divs.
	for (var i=0; i<tableDummyRow.childNodes.length; i++) {
		var tableColumnWidth      = tableDummyRow.childNodes[i].offsetWidth;
		var visualHeaderDivWidth  = visualHeaderDiv.childNodes[i].offsetWidth;

		// skip over text/whitespace nodes
		if (tableDummyRow.childNodes[i].nodeType==1)  {
			// the case where the visual Header is wider than the table cells

			if (visualHeaderDivWidth > tableColumnWidth) {

				// we need to bump up the whole table's width property each time a column is changed. This will always increase the size of the table, never decrease it. background: if this is not done until the end, then some of the columns along the way would have refused to accept the larger widths, and those assignments would actually fail. Although IE might do ok if this is omitted, mozilla does need it.
				var widthIncrement  = visualHeaderDivWidth - tableColumnWidth;
				table.style.width   = table.offsetWidth + widthIncrement + "px";

				// set the width of the div's within the column header cells.  background: if you try the straightforward approach of setting the width of the cells, the tableElement's constant desire to balance things against cell contents will occasionally rebuff you. This way, since you're making assignments to cell contents, the table is forced to listen.
				var childDiv = getChildNodebyNodeName(tableDummyRow.childNodes[i].childNodes, "DIV");
				if (childDiv != null) {
					childDiv.style.width =   (visualHeaderDivWidth - 10)+"px";
				}

				// However, a belt-and-suspenders approach is necessary, to prevent some strange behaviour (triggered sometimes by inputs or images), where the dummy element's expansion actually causes the table column to expand by that and THEN SOME.  I dont see how it could be useful, and i think a genuine bug, however this assignment to the TD seems to correct for it.
				tableDummyRow.childNodes[i].style.width =   (visualHeaderDivWidth - 10) +"px";
			}
			// the other, much easier case, where the table columns are larger than the header divs.
			// easier cause we just make the header div wider.
			else {
				// everyone except IE gets a pixel removed, to account for the border.
				visualHeaderDiv.childNodes[i].style.width = tableDummyRow.childNodes[i].offsetWidth + extraWidthForFirstHeader - ((document.all)?0:1)+ "px";
			}
			extraWidthForFirstHeader = 0;
			// this is just to make sure the header div's all have borders that extend all the way down.
			visualHeaderDiv.childNodes[i].style.height = maxScreenDivHeight +"px";
//			alert(maxScreenDivHeight +" ," + visualHeaderDiv.childNodes[i].offsetHeight)
		}
	}
}


// this function is triggered by onscroll on the scrolling div.  the onscroll event handler is attached during init
// what will happen is this:   visualHeaderDiv is a very long div, cropped within the shorter visualDivParent, courtesy of its overflow property.
// this function will look at the scrolling of the tableScrollingDiv, and use that value to slide visualHeaderDiv within it's parent, sort of like a big sliderule.
TableManager.prototype.slideColumns = function(tableId) {
    var visualHeaderDivParent   = document.getElementById(tableId+"_headerDiv");
    var slidingColumnHeaders = visualHeaderDivParent.childNodes[0];
    var scrollingDiv    = document.getElementById(tableId+"_scrollingTableDiv");

    // GUIDO CODE EDITED !!!
        // columnTableG formats the two divs nicely, which causes mozilla to see two extra children
        slidingColumnHeaders = getChildNodebyClassName( visualHeaderDivParent.childNodes, "headerDivInner");

    slidingColumnHeaders.style.left = -scrollingDiv.scrollLeft;
    var tableScrollingDiv       = document.getElementById(tableId+"_scrollingTableDiv");
    // Removed by AOR. The following statement was causing the table to grow in IE.
    //visualHeaderDivParent.style.width = tableScrollingDiv.offsetWidth+"px";

}


function tableManager_windowResize() {
        ourTableManager.windowResize();
}

function findTagIndex(childNodes, tagName) {
    for(var index = 0; index < childNodes.length; index++) {
        if(tagName == childNodes[index].nodeName)
            return index;
    }
    return -1;
}

/* Returns the number of selectable rows in the table */
TableManager.prototype.getRowCount = function(table) {
    var rows = this.getSelectableRows(table);
    return rows.length;
}

/** Returns a table object given the name of a ColumnTableG gadget */
TableManager.prototype.getTableByName = function(tableName) {
    var table = document.getElementById(tableName);
    vsAssertTagName("TABLE",table);
    return table;
}


