<%--
/**
 * The JSP page for breaking locks.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/break_lock.jsp,v 1.12 2008/03/05 12:16:10 sridharav Exp $
 * @risk Medium
 *
 * @task name = Break complex lock GUI
 *	cycle = 4
 *	phase = TEST
 *	owner = Ming
 *	status = DONE
 *	description = GUI to break a profile lock by force.
 *
 * @task name = integrate break lock into error handling
 *	cycle = 6
 *	phase = TEST
 *	owner = Ming
 *	status = Done
 *	description = none.
 *
 * @task name = wait for lock to release
 *	cycle = 6
 *	phase = TEST
 *	owner = Ming
 *	status = Done
 *	description = none.
 */
--%>

<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<%@ page errorPage="exception.jsp" %>

<%@ page import="com.hp.parmgr.model.Discovery,
                 com.hp.parmgr.model.Complex,
                 com.hp.parmgr.model.Cell,
                 com.hp.parmgr.model.IOChassis,
                 com.hp.parmgr.model.HWPartition,
                 com.hp.parmgr.model.LockableObject,
                 com.hp.parmgr.model.LockedObjectError,
                 com.hp.parmgr.util.BreakLockUtil,
                 com.hp.parmgr.util.Log,
                 java.util.List" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<jsp:useBean id="parsession" scope="session" class="com.hp.parmgr.ui.ParmgrSession" />

<jsp:useBean id="lockUtil" scope="session" class="com.hp.parmgr.util.BreakLockUtil" />

<i18n:bundle baseName="com.hp.parmgr.ui.BreakProfileLock" id="bundle" />

<%
    String helpPage = "breakLock";

    Discovery discovery = parsession.getDiscovery();
    String pageTitle = bundle.getString("breakLockTitle.str");
%>

<HTML>

<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<%@ include file="session_keep_alive.jsp" %>
    <link type="text/css" rel="stylesheet" href="<%=Util.URL_PARMGR_CSS%>">
    <link type="text/css" rel="stylesheet" href="<%=Util.URL_MXTABLE_CSS%>">
    <script type='text/javascript' language='JavaScript'
            src='<%=Util.URL_PARMGR_JS%>'></script>

    <title>
        <%=pageTitle%>
    </title>
</HEAD>

<%
    String okButton = request.getParameter("ok");
    String cancelButton = request.getParameter("cancel");

    // the show locks button only exists when the user got an error breaking a
    // lock, or canceling from the "go to wait for" confirmation dialog after
    // all the selected locks were successfully broken.
    String showLocksButton = request.getParameter("showLocks");

    if ((okButton == null) &&
        (cancelButton == null)) {
        if (showLocksButton == null){
            // this is the first time we got in here.  log start of task.
            Log.summary(bundle.getString("breakLock.start.log"));

            // intialize the lock util bean.
            lockUtil.reset();
            lockUtil.setDiscovery(discovery);
            lockUtil.setOut(out);
            lockUtil.setBundle(bundle);

            String fromURL = (String) session.getAttribute("fromURL");
            lockUtil.setFromURL(fromURL);
            LockedObjectError lockError =
                (LockedObjectError) session.getAttribute("lockError");
            session.removeAttribute("lockError");
            session.removeAttribute("fromURL");

            List lockedObjects = lockError.getLockedObjects();

            // assert(!lockedObjects.isEmpty());
            lockUtil.setLockedObjects(lockedObjects);
        }
        else {
                        lockUtil.setOut(out);
        }
%>

<BODY>

<p align="left" class="subwindow-heading">
    <%=pageTitle%>
</p>

<form method="post" action="break_lock.jsp" name="breakLock">

<%
        out.println(lockUtil.getConfirmMessage());
        out.println("<p>");
        lockUtil.printLockedObjectsTable();
        int lockType = lockUtil.getLockType();

        out.println("<p>");
        out.println(bundle.getString("breakLock.instructions.str"));
        out.println("</p>");

        // There won't be a cancel button if the lock is a group A lock, so
        // only print the cancel message if it's another type of lock.
        if (BreakLockUtil.COMPLEX != lockType){
            out.println("<p>");
            out.println(bundle.getString("breakLock.cancel.str"));
            out.println("</p>");
        }
%>
    <p align=left>

    <INPUT class="button" type="submit"
           value="<%=bundle.getString("button.ok.str")%>"
           name="ok" />

<% if (BreakLockUtil.COMPLEX != lockType){ %>
    <INPUT class="button" type="submit"
           value="<%=bundle.getString("button.cancel.str")%>"
           name="cancel"/>
<% } %>

    <INPUT class="button" type="button"
           value="<%=bundle.getString("button.help.str")%>"
           name="help"
           onClick="showHelp('<%=parsession.getHelpURL(helpPage)%>')" />
    </p>

</form>

<%
    } else if (cancelButton != null){
        // need to set the jsp writer again since we posted and this is a
        // different writer than before.
        lockUtil.setOut(out);

        // log that the user canceled, then exit.
        lockUtil.doCancel();
    } else if (okButton != null){
%>

<p align="left" class="subwindow-heading">
    <%=pageTitle%>
</p>

<%
        // need to set the jsp writer again since we posted and this is a
        // different writer than before.
        lockUtil.setOut(out);

        // the user made a choice to either wait or break the lock.
        lockUtil.processRequest(request);

        // will break the selected locks and let the users know which
        // locks have been broken and the command executed. If success, and
        // there are other locks that need to be waited on, the user will
        // get an okay button that once clicked, will take them to the wait
        // for dialog.  Otherwise continue with the task.  If failure,
        // give an ok button that behaves like cancel.
        lockUtil.doBreakAndOrWait(parsession);
    }
%>

</BODY>
</HTML>
