<%--
/**
 * Wrapper URL to generate URL for launching CAT functionality.
 * The JSP is called either by clicking on an I/O slot in an
 * I/O chassis or by selecting an I/O action in menu.jsp.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/cat.jsp,v 1.32 2008/03/05 12:16:10 sridharav Exp $
 * @risk: Low
 *
 * @task name = Create JSP to launch CAT pages.
 *       cycle = 5
 *       owner = Paul
 *       phase = TEST
 *       status = Done
 *       description = None
 *
 */
--%>

<%@ page errorPage="exception.jsp" %>
<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<i18n:bundle baseName="com.hp.parmgr.ui.Parmgr" id="parmgrBundle" />
<i18n:bundle baseName="com.hp.parmgr.ui.options" id="optionsBundle" />

<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.Complex" %>
<%@ page import="com.hp.parmgr.model.HWPartition" %>
<%@ page import="com.hp.parmgr.ui.SelectionManager" %>
<%@ page import="com.hp.parmgr.ui.DialogException" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page import="com.hp.parmgr.util.IOSlotUtil" %>
<%@ page import="com.hp.parmgr.util.LoginCredentials" %>
<%@ page import="com.hp.parmgr.util.StringFormatUtil" %>
<%@ page import="java.text.MessageFormat" %>
<%@ page import="java.util.TreeSet" %>
<%@ page import="java.util.Iterator" %>
<%@ page import="java.util.Map" %>
<%@ page import="java.net.Socket" %>
<%@ page import="java.io.IOException" %>
<%@ page import="java.net.MalformedURLException" %>
<%@ page import="java.net.UnknownServiceException" %>
<%@ page import="java.net.InetAddress" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<jsp:useBean id="parsession" scope="session" class="com.hp.parmgr.ui.ParmgrSession" />

<%

    // JAGaf88550: guard against cross site scripting vulnerabilities,
    // by encoding parameters that may be output to the HTML stream:
     String slotID =
         StringFormatUtil.encodeHtmlEntities(request.getParameter("slot"));
     String actionID =
         StringFormatUtil.encodeHtmlEntities(request.getParameter("actionID"));
     String hostname =
         StringFormatUtil.encodeHtmlEntities(request.getParameter("hostname"));

     String catURL = ""; /* URL of CAT page */
     String fromURL = parsession.getFromURL(); /* URL of referring page
                                                * within parmgr; does not
                                                * include hostname, port
                                                * number, and /parmgr/jsp.
                                                */
     String currentURL = ""; /* Complete URL of referring page within parmgr;
                              * includes hostname, port number, and
                              * /parmgr/jsp.
                              */
     StringBuffer thisURL = new StringBuffer(); /* URL of the current page. */

     Discovery discovery = parsession.getDiscovery();
     Complex complex = discovery.getComplex();
     SelectionManager selectedCheckboxes = parsession.getSelectedCheckboxes();
     HWPartition nPartition = null;
     int parID = IOSlotUtil.INVALID_PARTITION_ID;
     boolean badParID = false;
     boolean badSlotID = false;
     final int STANDARD_PDWEB_SERVER_PORT = 1188;
     final int AUTOSTART_PDWEB_SERVER_PORT = 1110;
     final int STANDARD_SMH_SERVER_PORT = 2381;
     final int AUTOSTART_SMH_SERVER_PORT = 2301;
     final String HTTP = "http";
     final String HTTPS = "https";

     if (slotID == null) {
        /* Retrieve the slot ID from the selectedCheckboxes object.  I/O
         * card actions are single-select, but an I/O slot may not
         * currently be visible on the page so it's necessary to find
         * the selected I/O slot that is actually visible.
         */
        TreeSet ioSlotIDs = selectedCheckboxes.getSelectedIOSlots();
        Map visibleCheckboxes = selectedCheckboxes.getVisibleCheckboxes();
        badSlotID = true;
        for (Iterator iter = ioSlotIDs.iterator(); iter.hasNext();) {
           /* Slot IDs returned from the getSelectedIOSlots() method
            * are of the form "x-x-x-x" while checkbox IDs from the
            * getVisibleCheckboxes() method have the IO_SLOT_PREFIX
            * prepended to the slot ID.  It's necessary, then, to
            * convert the slot ID into a checkbox ID before checking
            * to see if a given slot's checkbox is within the list of
            * visible checkboxes.
            */
           slotID = (String) iter.next();
           String slotCheckboxID =
              SelectionManager.getIOSlotCheckboxID(slotID);
           if (visibleCheckboxes.containsKey(slotCheckboxID)) {
              badSlotID = false;
              break;
           }
        }
     }

     if (!badSlotID) {
        parID = IOSlotUtil.getParIDForIOSlot(discovery, complex, slotID);
        if (parID != IOSlotUtil.INVALID_PARTITION_ID) {
           nPartition = discovery.getPartition(parID);
           if ((nPartition == null) || (nPartition.isErrorObject())) {
              badParID = true;
           }
        } else {
           /* A valid partition is required in order to perform CAT
            * functionality.
            */
           badParID = true;
        }
     }

     if (badSlotID || badParID) {
        /* Throw a dialog exception which indicates that an error
         * has occurred.  Because the exception window must then be
         * closed, do not pass along a fromURL in the constructor
         * for the dialog exception. Instead, send the javascript
         * window.close() directive.
         */
        String errorMessage = null;
        String errorTitle = null;
        if (badSlotID) {
           errorMessage =
              parmgrBundle.getString("error.cat.invalid.slotid");
           errorTitle = "error.cat.invalid.slotid.title";
        } else {
           errorMessage =
              parmgrBundle.getString("error.cat.invalid.partition");
           errorTitle = "error.cat.invalid.partition.title";
        }
        DialogException de =
           new DialogException(errorMessage, "javascript:window.close()", "com.hp.parmgr.ui.Parmgr");
        de.setLocale(parsession.getLocale());
        de.setTitle(errorTitle);
        throw de;
     }

     boolean readyToConstructURL = false;
     boolean urlConstructed = false;
     boolean connectionValid = false;
     LoginCredentials usernamePassword =
        new LoginCredentials(null, null, null, null, null);
     if (hostname != null) {
        /* If there is a hostname on the request, this is the second
         * visit to the JSP and the user has provided the parameter.
         * First, add the hostname to the parsession.  Then raise the
         * readyToConstructURL flag.
         */
        if (parsession.getUsernamePassword(parID) != null) {
           /* An entry for this partition already exists.
            * Override that entry with this one.
            */
           parsession.removeUsernamePassword(parID);
        }
        usernamePassword.setWBEMCredentials(hostname, null, null);
        parsession.addUsernamePassword(parID, usernamePassword);
        readyToConstructURL = true;
     } else {
        /* First, see if the parsession already has login data
         * for this partition.  If not, check to see if the current
         * partition is the local one and if it is, grab the WBEM
         * username and password from the parsession (and add it
         * to the list of known nPartition login information.  If
         * all of that fails, leave the readyToConstructURL flag
         * down so that a FORM can be loaded in the body of the
         * JSP that will prompt the user for the required data.
         */
        usernamePassword = parsession.getUsernamePassword(parID);
        if (usernamePassword != null) {
           hostname = usernamePassword.getWBEMHost();
           readyToConstructURL = true;
        } else if (nPartition.isLocalPartition()) {
           /* Local partition. */
           usernamePassword =
                 new LoginCredentials(parsession.getWBEMHost(),
                                      null, null, null, null);
           parsession.addUsernamePassword(parID, usernamePassword);
           readyToConstructURL = true;
        } else {
           /* Record the URL of this JSP so it can be used as the
            * action in the FORM that will now be generated.
            */
           String queryString = request.getQueryString();
           thisURL = request.getRequestURL();
              if (queryString != null) {
                 thisURL.append("?");
                 thisURL.append(request.getQueryString());
              }
           }
     }

     // get the fully qualified domain name for the host name.
     hostname = InetAddress.getByName(hostname).getCanonicalHostName();

     if (readyToConstructURL) {
        /* Time to assemble the CAT URL.  Need to add the hostname,
         * then tack on the desired CAT action, and the referring URL.
         */

        String currentHost = request.getServerName();
        int currentPort = request.getServerPort();

        currentURL = "https://" + currentHost + ":" +
                     currentPort + Util.URL_BASE_JSP + "/" + fromURL;

        /*
         * Check to see if it's possible to establish a connection
         * to the target system.  Try the standard pdweb server port
         * first, then try the autostart port.  If neither works,
         * flag the connection as invalid so an attempt to connect
         * via SMH can be attempted.
         */

        int catPort = STANDARD_PDWEB_SERVER_PORT;
        String protocol = HTTPS;
        try {
            Socket catSocket = new Socket(hostname, catPort);
            connectionValid = true;
        } catch (Exception ioe) {
            connectionValid = false;
        }

        if (!connectionValid) {
            catPort = AUTOSTART_PDWEB_SERVER_PORT;
            protocol = HTTP;
            try {
                Socket catSocket = new Socket(hostname, catPort);
                connectionValid = true;
            } catch (Exception ioe) {
                connectionValid = false;
            }
        }

        /*
         * SMH connection attempt
         *
         * If the standard pdweb connection attempt failed, then check to see
         * if it's possible to establish an SMH connection to the target
         * system.  If neither works, flag the connection as invalid so
         * an error page can be displayed.
         */

        if(!connectionValid)
        {
            catPort = STANDARD_SMH_SERVER_PORT;
            protocol = HTTPS;
            try {
                Socket catSocket = new Socket(hostname, catPort);
                connectionValid = true;
            } catch (Exception ioe) {
                connectionValid = false;
            }
        }

        if (!connectionValid) {
            catPort = AUTOSTART_SMH_SERVER_PORT;
            protocol = HTTP;
            try {
                Socket catSocket = new Socket(hostname, catPort);
                connectionValid = true;
            } catch (Exception ioe) {
                connectionValid = false;
            }
        }

        /* If the connection is valid, build the CAT URL. */

        if (connectionValid) {

            /* Use the "CLOSE" argument for the returnURL parameter
             * to inform pdweb that all "Done", "Log off", and
             * "Cancel" buttons should close their own windows.
             */

            String returnURL = "CLOSE";

            Object [] msgArgs = { protocol,
                                  hostname,
                                  new Integer(catPort).toString(),
                                  actionID,
                                  slotID,
                                  returnURL };

            MessageFormat msgFormat;
            if ((catPort == STANDARD_PDWEB_SERVER_PORT) ||
                (catPort == AUTOSTART_PDWEB_SERVER_PORT))
            {
                msgFormat =
                    new MessageFormat(optionsBundle.getString("Cat.url"));
            } else {
                // changed interface format from using "+" as delimiter to
                // using ":" as delimiter to work around a cleansweep issue.
                msgFormat =
                    new MessageFormat(optionsBundle.getString("Cat1205.url"));
            }
            catURL = msgFormat.format(msgArgs);
        }

        urlConstructed = true;
     }

%>

<HTML>
<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<%@ include file="session_keep_alive.jsp" %>
<script type='text/javascript' language='JavaScript' src='<%=Util.URL_PARMGR_JS%>'
></script>

<SCRIPT language="JavaScript" type="text/javascript">
function singleOnSubmit()
{
    document.getElementById("HOSTNAME_BUTTON").disabled = true;
    return true;
}
function logon_onLoad()
{
    document.hostnameForm.hostname.focus();
}
</SCRIPT>

<link type='text/css' rel='stylesheet' href='<%=Util.URL_PARMGR_CSS%>'>
</HEAD>

<%
   /* If the CAT URL has been constructed and the connection is
    * valid, display a new page.  If the connection is invalid,
    * show an error page.  All of the tasks replace the entire window.
    */

   if (urlConstructed) {
      if (connectionValid) {
%>

<BODY onLoad='document.location.replace("<%=catURL%>")'>

<%

      } else {

         /* Unable to connect to I/O system.  Display an error page. */
%>

<BODY>
   <img src="<%=Util.URL_BASE_IMAGES + "/failed.gif"%>">
   <font class="errHeading">
      <i18n:message key="io.unavailable.label" bundle="<%=parmgrBundle%>" />
   </font>
   <p class="errText">
      <i18n:message key="io.unavailable.description" bundle="<%=parmgrBundle%>" />
   </p>

<TABLE>
   <td valign="top">
      <INPUT class="button" type="submit"
             value="<%=parmgrBundle.getString("button.close.str")%>"
             name="closeWindow"
             onClick="window.close()" />
   </td>
   <td valign="top">
      <INPUT class="button" type="button"
             value="<%=parmgrBundle.getString("button.help.str")%>"
             name="help"
             onClick="showHelp('<%=parsession.getHelpURL("ioUnavailable")%>')" />
   </td>
</TABLE>

<%
      }
   } else {

   String defaultIPValue = "";
   String adminIPAddress = nPartition.getAdminIPAddress();
   if (adminIPAddress != null) {
       if (! adminIPAddress.equals("0.0.0.0")) {
           defaultIPValue = adminIPAddress;
       }
   }
   String hostnameLabel = parmgrBundle.getString("login.hostname.label");
   String connectLabel = parmgrBundle.getString("login.connect.label");
%>

<BODY onLoad="logon_onLoad()">

   <TABLE class="parmgr-regular-text" border="0" cellspacing="0" cellpadding="0">
      <FORM name="hostnameForm" method="POST" action="<%=thisURL%>" onSubmit="return singleOnSubmit()" >
         <TR>
            <TD>
               <%=hostnameLabel%>
            </TD>
         </TR>
         <TR>
             <TD>

<%
   Object [] loginMsgArg = { nPartition.getLongLabel() };
   MessageFormat loginMsgFormat =
      new MessageFormat(parmgrBundle.getString("login.cat.prompt.format"));
   String loginMsg = loginMsgFormat.format(loginMsgArg);
%>

               <%=loginMsg%>
             </TD>
             <TD><input type="text" name="hostname" style="width:200" value="<%=defaultIPValue%>"></TD>
         </TR>
          <TR>
             <TD colspan="2" align="right"><input id="HOSTNAME_BUTTON" type="submit" class="button" value="<%=connectLabel%>" ></TD>
          </TR>

      </FORM>
   </TABLE>
<% } %>

</BODY>
</HTML>





