<%--
/**
 * The JSP page for turning on/off cell LEDs.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/cell_LED.jsp,v 1.23 2008/03/05 12:16:10 sridharav Exp $
 * @risk Medium
 *
 * @task name = Cell Task: turn off cell LED
 *      cycle = 5
 *      phase = test
 *      owner = Ming
 *      status = DONE
 *      description = none.
 *
 * @task name = Cell Task: turn on cell LED
 *      cycle = 5
 *      phase = test
 *      owner = Ming
 *      status = done
 *      description = none.
 */
--%>
<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<%@ page import="java.text.MessageFormat" %>
<%@ page import="java.util.TreeSet" %>
<%@ page import="java.util.List" %>
<%@ page import="java.util.Iterator" %>
<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.Cabinet" %>
<%@ page import="com.hp.parmgr.model.CellSlot" %>
<%@ page import="com.hp.parmgr.ui.SelectionManager" %>
<%@ page import="com.hp.parmgr.ui.SessionTimeOut" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page import="com.hp.parmgr.util.FRULedTask" %>
<%@ page import="com.hp.parmgr.util.Log" %>
<%@ page import="com.hp.parmgr.util.TaskUtil" %>
<%@ page import="com.hp.parmgr.util.CommandFailedException" %>
<%@ page import="com.hp.parmgr.util.CommandIOException" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<jsp:useBean id="parsession" scope="session"
             class="com.hp.parmgr.ui.ParmgrSession" />

<i18n:bundle baseName="com.hp.parmgr.ui.Parmgr" id="bundle" />

<%
    Log.summary(Log.EVENT_START, Log.RESULT_NONE,
            "Starting the cell attention LED task.");

    Discovery discovery = parsession.getDiscovery();

    // Determine what kind of cabinet we're dealing with since
    // the smaller cabinets do not have cabinet LCDs
    boolean changeCabinetLCD = true;
    String cabType = "sd";
    List cabs = discovery.getCabinets();
    for (Iterator iter = cabs.iterator(); iter.hasNext();) {
        Cabinet cab = (Cabinet)iter.next();
        int type = cab.getType();
        if (type == Cabinet.KEYSTONE || type  == Cabinet.MATTERHORN) {
            changeCabinetLCD = false;
            cabType="rome";
        }
    }

    boolean hasErr=true;
    String errMsg = null;

    // get the task the user has selected.
    String cellLEDTask = request.getParameter("cellLEDTask");

    /*
       The following variables are used to determine the display messages and
       task operation.
        - cellLEDTitle: title displayed for the task.
        - cellLEDIntro: Intro string displayed for the task.
        - turnOnLED: boolean that indicates to turn on or off the LEDs.
        - helpPage: the page ID to access help
     */
    String cellLEDTitle=null;
    String cellLEDIntro=null;
    String cellLEDIntroLog=null;
    String cellLEDLog=null;
    boolean turnOnLED=false;
    String helpPage=null;
    if (cellLEDTask.equals("on")){
        cellLEDTitle = bundle.getString("cellLED.on.title");
        cellLEDIntro = "LED.on.intro.fmt." + cabType;
        cellLEDIntroLog = "LED.on.intro.log.fmt." + cabType;
        cellLEDLog = "cellLED.on.log.fmt";
        turnOnLED = true;
        helpPage = "cellLEDOn";
    } else if (cellLEDTask.equals("off")){
        cellLEDTitle = bundle.getString("cellLED.off.title");
        cellLEDIntro = "LED.off.intro.fmt." + cabType;
        cellLEDIntroLog = "LED.off.intro.log.fmt." + cabType;
        turnOnLED = false;
        helpPage = "cellLEDOff";
        cellLEDLog = "cellLED.off.log.fmt";
    } else {
        errMsg = bundle.getString("error.badArgs.str");
        Log.error("The URL contains unknown parameters.");
    }
%>

<HTML>

<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<%@ include file="session_keep_alive.jsp" %>
    <link type="text/css" rel="stylesheet" href="<%=Util.URL_PARMGR_CSS%>">
    <script type='text/javascript' language='JavaScript'
            src='<%=Util.URL_PARMGR_JS%>'></script>

    <%=SessionTimeOut.javascript(session, bundle.getLocale())%>
    <title>
        <%=cellLEDTitle%>
    </title>
</HEAD>

<BODY>

<p align="left" class="subwindow-heading">
    <%=cellLEDTitle%>
</p>

<%
    String fromURL = parsession.getFromURL();
    String complexName = null;
    SelectionManager selectedCheckboxes = parsession.getSelectedCheckboxes();

    String pageIntro;
    TreeSet cellIDs = selectedCheckboxes.getSelectedCells();
    int cellID = parsession.getImplicitCellID();
    String fruledCommand = null;
    String introString = null;

    // get the cell ID.
    if (errMsg == null) {
        // only get the cell ID if the arguments are correct.
        if (cellID == -1){
            // there is no implicit selection, get the cell ID from
            // selection manager.
            if (cellIDs.size() == 1){
                // right now the menu only supports single selection, i.e. the
                // cell LED task is only enabled when there is exactly one cell
                // selected.
                cellID = ((Integer) cellIDs.first()).intValue();
            } else if (cellIDs.size() > 1){
                // there is more than one cell selected.
                errMsg = bundle.getString("error.tooManyCellSelected.str");
                Log.error("There is more than one cell selected.  Only one cell may be selected to use this task.");
            } else {
                // there is no cell selection, either implicit or otherwise.
                errMsg = bundle.getString("error.noCellSelected.str");
                Log.error("No cell has been selected.");
            }
        }
    }

    if (cellID != -1){
        // format the fruled command.
        CellSlot cellSlot = discovery.getCellSlot(cellID);
        if (cellSlot == null){
            errMsg = bundle.getString("error.noSuchSlot.fmt");
            Log.error("The specified slot does not exist in the complex.  The data that Partition Manager (parmgr) has must be old. Perform a refresh via the parmgr [Refresh] button (not the browser refresh button).");
        } else {

            FRULedTask fruled = FRULedTask.getCellLedTask(
                                            cellSlot,
                                            changeCabinetLCD,
                                            turnOnLED,
                                            parsession.getLoginCredentials());

            try {
                // log start of command.
                Object [] logArgs = {cellSlot.getLongLabel()};
                MessageFormat logFormat =
                    new MessageFormat(bundle.getString(cellLEDLog));
                Log.detail(Log.EVENT_INTERMEDIATE, Log.RESULT_NONE,
                        logFormat.format(logArgs));

                fruledCommand = fruled.getCommand();
                fruled.run();
                hasErr = false;

                // format the success message;
                Cabinet cabinet = discovery.getCabinet(cellSlot.getCabinetID());
                String cellLabel = cellSlot.getLongLabel();

                Object [] msgArgs = new Object[2];

                msgArgs[0] = cellSlot.getLongLabel();

                if (changeCabinetLCD) {
                    msgArgs[1] = new String(cabinet.getLabel());
                }

                MessageFormat myFormat =
                    new MessageFormat(bundle.getString(cellLEDIntro));
                introString = myFormat.format(msgArgs);

                msgArgs[0] = discovery.getComplex().getName();
                msgArgs[1] = null;
                myFormat = new MessageFormat(bundle.getString("complexName.fmt"));
                complexName = myFormat.format(msgArgs);

                // log the success completion of the command.
                MessageFormat myFormatLog =
                    new MessageFormat(bundle.getString(cellLEDIntroLog));
                String introStringLog = myFormat.format(msgArgs);
                Log.detail(Log.EVENT_INTERMEDIATE, Log.RESULT_SUCCESS,
                        "Complex: " + msgArgs[0]);
            }
            catch (CommandIOException ioe) {
                Object [] ioeArgs = {ioe.getMessage()};
                MessageFormat ioeFormat =
                    new MessageFormat(bundle.getString("error.ioError.format"));
                errMsg = ioeFormat.format(ioeArgs);
            } catch (CommandFailedException ce) {
                errMsg = ce.getMessage();
            }
        }
    }
%>

<p align="left">
<%
    if (hasErr){
        TaskUtil.displayError(errMsg, bundle, out);
        // log failure.
        Log.summary(Log.EVENT_DONE, Log.RESULT_FAILURE,
                "Failed the cell attention LED task.");
    } else {
%>
<p align="left"><%=complexName%></p>
<p align="left"><%=introString%></p>
<%
        // log success.
        Log.summary(Log.EVENT_DONE, Log.RESULT_SUCCESS,
                "Completed the cell attention LED task.");
    }
%>
</p>

<% if (fruledCommand != null) { %>

<p>
<i18n:message key="parmgr.commandTitle.str" bundle="<%=bundle%>" />
</p>

<p align="left"><code>
<%=fruledCommand%>
</code>
</p>

<p align=left>
    <INPUT class="button" type="button"
        value="<%=bundle.getString("button.ok.str")%>"
        name="Ok"
        onClick="window.location.replace('<%=fromURL%>');"
    />
    <INPUT class="button" type="button"
        value="<%=bundle.getString("button.help.str")%>"
        name="Help"
        onClick="showHelp('<%=parsession.getHelpURL(helpPage)%>')"
    />
</p>

<% } %>

</BODY>
</HTML>

