<%--
/**
 * The JSP page for turning on/off chassis LEDs.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/chassis_LED.jsp,v 1.20 2008/03/05 12:33:01 sridharav Exp $
 *
 * @task name = Chassis Task: turn on/off cell LED
 *      cycle = 5
 *      phase = TEST
 *      owner = Paul
 *      status = Done
 *      description = None
 */
--%>
<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<%@ page import="java.text.MessageFormat" %>
<%@ page import="java.util.TreeSet" %>
<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.Cabinet" %>
<%@ page import="com.hp.parmgr.model.CellSlot" %>
<%@ page import="com.hp.parmgr.model.IOChassisSlot" %>
<%@ page import="com.hp.parmgr.ui.SelectionManager" %>
<%@ page import="com.hp.parmgr.ui.SessionTimeOut" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page import="com.hp.parmgr.util.FRULedTask" %>
<%@ page import="com.hp.parmgr.util.CommandFailedException" %>
<%@ page import="com.hp.parmgr.util.CommandIOException" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<jsp:useBean id="parsession" scope="session"
             class="com.hp.parmgr.ui.ParmgrSession" />

<i18n:bundle baseName="com.hp.parmgr.ui.Parmgr" id="parmgrTagResources" />

<%
    boolean hasErr=true;
    /* Determine the desired state of the chassisLED. */
    String chassisLEDState = request.getParameter("chassisLED");

    /*
       The following variables are used to determine the display messages and
       task operation.
        - chassisLEDTitle: title displayed for the task.
        - chassisLEDIntro: Intro string displayed for the task.
        - activateLED: boolean that indicates whether or not to turn on the
                       LED.
        - pageID: the page ID to access help
     */
    String chassisLEDTitle;
    String chassisLEDIntro;
    boolean activateLED;
    String pageID;

    chassisLEDTitle = parmgrTagResources.getString("chassisLED." +
        chassisLEDState + ".title");
    // We can use the superdome message since the ROME cabinets
    // do not have chassis LEDs
    chassisLEDIntro = "LED." + chassisLEDState + ".intro.fmt.sd";
    activateLED = chassisLEDState.equals("on");
    pageID = "chassisLED" + chassisLEDState;

    String fromURL = parsession.getFromURL();
    Discovery discovery = parsession.getDiscovery();
    String complexName = null;
    SelectionManager selectedCheckboxes = parsession.getSelectedCheckboxes();

    String errMsg = null;

    String pageIntro;
    TreeSet chassisIDs = selectedCheckboxes.getSelectedChassis();
    int chassisID = parsession.getImplicitChassisID();
    FRULedTask fruled = null;
    String introString = null;

    /* Get the chassis ID. */
    if (chassisID == -1){
        /* There is no implicit selection.  Get the chassis ID from
         * SelectionManager.
         */

        if (chassisIDs.size() > 0){
            /* assert(chassisIDs.size == 1); Should be single-select. */
            chassisID = ((Integer) chassisIDs.first()).intValue();
        } else {
            /* No chassis have been selected. */
            errMsg =
               parmgrTagResources.getString("error.noChassisSelected.str");
            // throw exception;
            return;
        }
    }

    /* Format the fruled command. */
    IOChassisSlot chassisSlot = discovery.getChassisSlot(chassisID);
    fruled = FRULedTask.getIOChassisLEDTask(chassisSlot,
                                            true, /* Change the cabinet LCD in
                                                   * the same manner as the
                                                   * chassis LED.
                                                   */
                                            activateLED,
                                            parsession.getLoginCredentials());

    try {
        fruled.run();
        hasErr = false;

        /* Format the success message. */
        Cabinet cabinet = discovery.getCabinet(chassisSlot.getCabinetID());
        String chassisLabel = chassisSlot.getLongLabel();

        Object [] msgArgs = {chassisSlot.getLongLabel(),
                             new String(cabinet.getLabel())};
        MessageFormat myFormat =
            new MessageFormat(parmgrTagResources.getString(chassisLEDIntro));
        introString = myFormat.format(msgArgs);

        msgArgs[0] = discovery.getComplex().getName();
        msgArgs[1] = null;
        myFormat = new MessageFormat(parmgrTagResources.getString("complexName.fmt"));
        complexName = myFormat.format(msgArgs);
    }
    catch (CommandIOException ioe) {
        Object [] ioeArgs = {ioe.getMessage()};
        MessageFormat ioeFormat =
            new MessageFormat(parmgrTagResources.getString("error.ioError.format"));
        errMsg = ioeFormat.format(ioeArgs);
        // throw exception
    } catch (CommandFailedException ce) {
        errMsg = ce.getMessage();
        // throw exception
    }
%>

<HTML>

<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<%@ include file="session_keep_alive.jsp" %>
    <link type="text/css" rel="stylesheet" href="<%=Util.URL_PARMGR_CSS%>">
    <script type='text/javascript' language='JavaScript'
            src='<%=Util.URL_PARMGR_JS%>'></script>
<%=SessionTimeOut.javascript(session, parmgrTagResources.getLocale())%>
    <title>
        <%=chassisLEDTitle%>
    </title>
</HEAD>

<BODY>

<p class="subwindow-heading">
    <%=chassisLEDTitle%>
</p>

<p>
<%
    if (hasErr){
        out.println("<img src=\"" + Util.URL_BASE_IMAGES + "/failed.gif\">");
        out.print("<font class=\"errHeading\">");
        out.print(parmgrTagResources.getString("error.operationFailed.title.str"));
        out.println("</font>");
        out.print("<p class=\"errText\">");
        out.print(errMsg);
        out.println("</p>");
    } else {
%>
<p align="left"><%=complexName%></p>
<p align="left"><%=introString%></p>
<%
    }
%>
</p>

<% if (fruled != null) { %>

<p>
<%=parmgrTagResources.getString("parmgr.commandTitle.str")%>
</p>

<p align="left"><code>
<%=fruled.getCommand()%>
</code>
</p>

<p align=left>
    <INPUT class="button" type="button"
        value="<%=parmgrTagResources.getString("button.ok.str")%>"
        name="ok"
        onClick="window.location.replace('<%=fromURL%>');"
    />
    <INPUT class="button" type="button"
        value="<%=parmgrTagResources.getString("button.help.str")%>"
        name="help"
        onClick="showHelp('<%=parsession.getHelpURL(pageID)%>')"
    />
</p>

<% } %>


</BODY>
</HTML>
