<%--
/**
 * The JSP page for powering on an I/O chassis.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/chassis_power.jsp,v 1.24 2008/03/05 12:33:01 sridharav Exp $
 * @risk Medium
 *
 * @task name = Chassis Task: power on
 *       cycle = 5
 *       phase = test
 *       owner = Ming
 *       status = done
 *       description = none.
 *
 * @task name = Chassis Task: power off
 *       cycle = 5
 *       phase = test
 *       owner = Ming
 *       status = done
 *       description = none.
 */
--%>

<%@ page errorPage="exception.jsp" %>

<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<%@ page import="java.text.MessageFormat" %>
<%@ page import="java.util.TreeSet" %>
<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.Cabinet" %>
<%@ page import="com.hp.parmgr.model.CellSlot" %>
<%@ page import="com.hp.parmgr.model.IOChassisSlot" %>
<%@ page import="com.hp.parmgr.ui.SelectionManager" %>
<%@ page import="com.hp.parmgr.ui.SessionTimeOut" %>
<%@ page import="com.hp.parmgr.ui.UnauthorizedUserException" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page import="com.hp.parmgr.util.WaitForCellToPowerOn" %>
<%@ page import="com.hp.parmgr.util.TaskUtil" %>
<%@ page import="com.hp.parmgr.util.FRUPowerTask" %>
<%@ page import="com.hp.parmgr.util.CommandIOException" %>
<%@ page import="com.hp.parmgr.util.CommandFailedException" %>
<%@ page import="com.hp.parmgr.util.StringFormatUtil" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<jsp:useBean id="parsession" scope="session"
             class="com.hp.parmgr.ui.ParmgrSession" />

<jsp:useBean id="powerUtil" scope="session"
             class="com.hp.parmgr.util.IOChassisPowerUtil" />

<i18n:bundle baseName="com.hp.parmgr.ui.Parmgr" id="bundle" />
<i18n:bundle baseName="com.hp.parmgr.util.WaitMessages" id="waitBundle" />

<%
FRUPowerTask cellCommand=null;
FRUPowerTask cellOffCommand=null;
FRUPowerTask chassisCommand=null;

Discovery discovery = parsession.getDiscovery();

Object [] msgArgs = { discovery.getComplex().getName() };
MessageFormat format = new MessageFormat(bundle.getString("complexName.fmt"));
String complexName = format.format(msgArgs);

// Do this after getting discovery
if (!parsession.userAuthorizedToMakeChanges("frupower")) {
    throw new UnauthorizedUserException(parsession.getFromURL(), parsession.getLocale());
}

// get ok and cancel buttons and the URL to return to.
String okButton = request.getParameter("ok");
String cancelButton = request.getParameter("cancel");
String fromURL = parsession.getFromURL();

if (cancelButton != null) {
    // return to from URL and exit the task.  the user canceled or is done
    // with the task, use javascript to get out of the task and return to
    // from URL.
%>
    <html>
    <body onLoad="window.location.replace('<%=fromURL%>');" >
    </body>
    </html>
<%
} else {

    /*
       The following variables are used to determine the state of the jsp:
       - powerState: indicates if this is a power on or off operation.
       - hasErr: True if there is an error that prevents the user from
                 continuing with the power task.  If this flag is set, then
                 the user will not get a confirmation dialog, just the error
                 and an [OK] button that will exit out of the task.
     */
    boolean powerState = true;
    boolean hasErr = true;
    String msg = null;

    /*
       The following variables are used to determine the display messages:
       - helpPage: what to display when [Help] is clicked.
       - chassisTaskTitle: the displayed title.
     */
    String helpPage = null;
    String chassisTaskTitle = null;
    // JAGaf88550: guard against cross site scripting vulnerabilities,
    // by encoding parameters that may be output to the HTML stream:
    String chassisPowerTask =
        StringFormatUtil.encodeHtmlEntities(request.getParameter("chassisPowerTask"));

    if (chassisPowerTask.equals("off")) {
        powerState = false;
        helpPage = "chassisPowerOff";
        chassisTaskTitle = bundle.getString("chassisPowerOff.title");
    } else if (chassisPowerTask.equals("on")) {
        powerState = true;
        helpPage = "chassisPowerOn";
        chassisTaskTitle = bundle.getString("chassisPowerOn.title");
    } else {
        msg = bundle.getString("error.badArgs.str");
    }
%>
    <HTML>
    <HEAD>
    <META http-equiv="Content-Type" content="text/html;charset=UTF-8">
        <%@ include file="session_keep_alive.jsp" %>
        <link type="text/css" rel="stylesheet" href="<%=Util.URL_PARMGR_CSS%>">
        <script type='text/javascript' language='JavaScript'
                src='<%=Util.URL_PARMGR_JS%>'></script>
        <%=SessionTimeOut.javascript(session, bundle.getLocale())%>
        <title>
            <%=chassisTaskTitle%>
        </title>
    </HEAD>

<%
    if (okButton == null) {
        // print the confirmation or error dialog.

        // this is the first time into this dialog. Need to log the start of
        // the task.

        // reset the IOChassisPowerUtil bean.
        powerUtil.reset();

        // get the selected objects.
        SelectionManager selectedCheckboxes = parsession.getSelectedCheckboxes();

        TreeSet chassisIDs = selectedCheckboxes.getSelectedChassis();

        // get implict chassisID.
        int chassisID = parsession.getImplicitChassisID();

        // only get the chassis ID if the parameters from the URL are correct.
        if (msg == null) {
            if (chassisID == -1){
                if (chassisIDs.size() == 0) {
                    msg = bundle.getString("error.noChassisSelected.str");
                } else if (chassisIDs.size() == 1){
                    chassisID = ((Integer) chassisIDs.first()).intValue();
                } else {
                    // too many chassis selections.
                    msg = bundle.getString("error.tooManyChassisSelected.str");
                }
            }
        }

        if (chassisID != -1) {
            // we have validated that there is a chassis ID.  Now clear the
            // discovery cache.
            discovery.clearCache();

            // check to see if the power task can be performed based on the
            // state of the chassis and the state of any cells the chassis
            // might be connected to.
            int retVal = powerUtil.canPower(powerState,
                                            chassisID,
                                            discovery,
                                            bundle,
                                            parsession.getLoginCredentials());
            msg = powerUtil.getMsg();

            // The CANT_POWER and NO_OP return values are counted as errors,
            // and will be displayed as such.
            if (retVal > powerUtil.CANT_POWER){
                hasErr = false;

                // We need to get the commands here for display purposes. If
                // there is no command, a null will be returned.
                cellCommand = powerUtil.getCellCommand();
                cellOffCommand = powerUtil.getCellOffCommand();
                chassisCommand = powerUtil.getChassisCommand();
            }
        }
%>
        <body>
        <!-- print task title on the screen. -->
        <p align='left' class='subwindow-heading'>
        <%=chassisTaskTitle%>
        </p>

        <p align='left'>
        <%=complexName%>
        </p>

        <!-- print the form. -->
        <form action='chassis_power.jsp' method='post'>

        <!-- hidden field to remember if the chassis task was on or off. -->
        <INPUT type='hidden' value='<%=chassisPowerTask%>' name='chassisPowerTask' />
<%
        if (hasErr) {
            // log failure.

            // print error msg.
            TaskUtil.displayError(msg, bundle, out);
%>
            <!-- Okay button for the error dialog. -->
            <INPUT class='button' type='button'
                   value='<%=bundle.getString("button.ok.str")%>'
                   name='ok'
                   onClick="window.location.replace('<%=fromURL%>');" />
<%
        } else {
%>
            <p>
            <%=msg%>
            </p>

            <hr class=separator>

            <!-- okay button for the confirmation dialog. -->
            <INPUT class='button' type='submit'
                   value='<%=bundle.getString("button.ok.str")%>'
                   name='ok' />

            <!-- cancel button for the confirmation dialog. -->
            <INPUT class='button' type='submit'
                   value='<%=bundle.getString("button.cancel.str")%>'
                   name='cancel' />
<%
        }
%>
        <!-- help button for the task. -->
        <INPUT class='button' type='button'
               value='<%=bundle.getString("button.help.str")%>'
               name='help'
               onClick="showHelp('<%=parsession.getHelpURL(helpPage)%>');" />

        </form>
<%
        // print the commands if there isn't an error, and we have commands to
        // run after user confirmation.
        if (!hasErr) {
            // Show user the commands that will be executed.
            String buttonNameString = null;
            boolean displayCommandTable = cellCommand != null ||
                                          chassisCommand != null ||
                                          cellOffCommand != null;
            if (displayCommandTable) {
                // there is atleast one command we can display. We don't check
                // the cellOffcommand because it's always null when cellCommand
                // is null.
                buttonNameString = "["+bundle.getString("button.ok.str")+"]:";
%>
                <p><%=bundle.getString("commandPreview")%><br>
                <dl><dd>
                <table>
<%
            }

            if (cellOffCommand != null){
%>
                <tr>
                <td align=right class="parmgr-regular-text"><%=buttonNameString%></td>
                <td align=left><code><%=cellOffCommand.getCommand()%></code></td>
                </tr>
<%
                buttonNameString = "";
            }
            if (cellCommand != null) {
%>
                <tr>
                <td align=right class="parmgr-regular-text"><%=buttonNameString%></td>
                <td align=left><code><%=cellCommand.getCommand()%></code></td>
                </tr>
<%
                buttonNameString = "";
            }
            if (chassisCommand != null) {
%>
                <tr>
                <td align=right class="parmgr-regular-text"><%=buttonNameString%></td>
                <td align=left><code><%=chassisCommand.getCommand()%></code></td>
                </tr>
<%
            }

            if (displayCommandTable) {
%>
                </table>
                </dd></dl>
                </p>
<%
            }
        }
%>
        </body>
        </html>
<%
    } else {
        // the user hit ok, we need to do something to either a chassis or
        // a cell.  Run the commands and print the results page, including
        // logic to wait for a cell to power on.

        try {
            chassisCommand = powerUtil.getChassisCommand();
            if (chassisCommand == null){
                // we don't have a chassis command, it must mean that we need
                // to run a cell command.
                cellCommand = powerUtil.getCellCommand();
                cellOffCommand = powerUtil.getCellOffCommand();
                if (cellOffCommand != null){
                    // we are power cycling the cell, need to turn it off first.
                    cellOffCommand.run();
                }
                //assert(cellCommand != null);
                cellCommand.run();

            } else {
                // we only need to manage power to the chassis. Just run the
                // chassis command.
                chassisCommand.run();
            }
            discovery.clearCachedChassis(powerUtil.getChassisID());
            int cellID = powerUtil.getWaitForCellID();
            if (cellID != -1) {
                discovery.clearCachedCell(cellID);
            }
            hasErr = false;
            // log sucess for the task.

        } catch (CommandIOException ioe) {
            Object [] ioeArgs = {ioe.getMessage()};
            MessageFormat ioeFormat =
                new MessageFormat(bundle.getString("error.ioError.format"));
            msg = ioeFormat.format(ioeArgs);
        } catch (CommandFailedException ce) {
            msg = ce.getMessage();
        }

        boolean goToWait = powerUtil.waitForCell();
        if (goToWait && !hasErr ) {
            // if there are cells that need to be powered on, wait for them
            // here.
            int cellID = powerUtil.getWaitForCellID();
            WaitForCellToPowerOn wait =
                new WaitForCellToPowerOn(discovery, waitBundle, cellID);

            wait.resetStartTime();

            // set the URL the wait for should return to.
            wait.setForwardURL(fromURL);

            // lets the user interact with the wait for dialog.
            wait.setInteractive(true);

            // lets the user cancel/close the wait.
            wait.setOptional(true);

            parsession.setWaitBean(wait);

            // print HTML page to redirect to the wait for dialog.
%>
            <BODY onLoad="window.location.replace('wait.jsp')">
            <%=bundle.getString("note.transitioningToWaitDialog.str")%>
            </BODY>
            </html>
<%
        } else {
%>
            <BODY>

            <p align='left' class='subwindow-heading'>
            <%=chassisTaskTitle%>
            </p>

            <p align='left'>
            <%=complexName%>
            </p>

            <p>
<%
            if (hasErr){
                // log error and failure for the task

                // print error message content.
                TaskUtil.displayError(msg, bundle, out);
            } else {
                // print success msg.
%>
                <%=powerUtil.getSuccessMsg()%>
<%
            }
%>
            </p>
            <form>
            <!-- Okay button for the results dialog. -->
            <INPUT class='button' type='button'
                   value='<%=bundle.getString("button.ok.str")%>'
                   name='ok'
                   onClick="window.location.replace('<%=fromURL%>')" />

            <!-- Help button for the task.-->
            <INPUT class='button' type='button'
                   value='<%=bundle.getString("button.help.str")%>'
                   name='ok'
                   onClick="showHelp('<%=parsession.getHelpURL(helpPage)%>')" />
            </form>
            </BODY>
            </html>
<%
        }

    }
} // end printing results page.
%>
