<%--
/**
 * This included JSP checks for the presence of a pending profile,
 * if necessary.  The JSP forwards control to the pending_profile.jsp 
 * page, if a check is required AND a pending profile exists.  If 
 * "continue" is on the request, the user has been to the pending 
 * profile screen and has chosen to ignore the pending profile and
 * continue with the task.  The continue flag is an immediate indicator.
 * There are times when it will not be set (e.g. every time the user
 * goes to a new tab in the BPVs), even though the user has told us
 * to ignore the pending profile.  The second flag, checkPending, in
 * the parmgr session, is a more long term "memory" that indicates
 * whether it's necessary to check for a new pending proile.  In order
 * to avoid checking for a new pending profile on every tab change in
 * the BPVs, the check is conducted only on the first BPV visited.  If
 * no pending profile is discovered on the initial check, the checkPending
 * flag is lowered.  It will not be raised again until either the Refresh
 * button is pressed or a task is executed which depends on knowing if a 
 * pending profile is present (Create/Modify/Delete nPar and Name Complex).
 * If a pending profile is discovered, pending_profile.jsp is launched.
 * On that page, the user may take one of three actions:  wait for the
 * profile to be pushed out, cancel the profile, or continue without 
 * regard for the existence of the profile.  In the first two cases, the
 * checkPending flag continues to be raised.  In the latter case, it is
 * lowered and remains down until one of the cases listed above occurs.
 * Note that when the pending profile is checked at the outset of one
 * of the four tasks listed above, the checkPending flag will immediately
 * be lowered if no pending profile exists.  This has the added consequence
 * that a pending profile which is pushed out following, say, a successful 
 * creation of an nPar will be ignored when parmgr returns to the BPV.  In
 * addition, if the pending profile check passes successfully at the start
 * of a task and then a rogue pending profile shows up before the command
 * is executed, the command could fail with a pending profile error, but
 * parmgr won't produce the pending profile warning and its associated
 * screen until the Refresh button is pressed or the task is re-executed.
 *
 * Included by:
 * 
 * complex.jsp, partition.jsp, cell.jsp, chassis.jsp, create_npar.jsp,
 * modify_npar.jsp, delete_npar.jsp, name_complex.jsp, 
 * analyze_complex_health.jsp
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/check_pending_profile.jsp,v 1.7 2003/07/11 20:47:10 nkinney Exp $
 * @risk: Low
 *
 */
--%>

<%@ page import="com.hp.parmgr.model.Complex" %>
<%@ page import="com.hp.parmgr.model.Discovery" %>
<%
    if (request.getParameter("continue") == null &&
        parsession.getCheckPending())
    {
        // The user did not come here immediately following the
        // pending profile screen (by hitting the Continue button)
        // and a check for a pending profile hasn't already taken
        // place.

        // Can't use variable "discovery" because it may or may
        // not be set/used by the file that included this one.
        Discovery myDiscovery = parsession.getDiscovery();

        // See if the user hit Cancel on the pending profile screen.
        if (request.getParameter("cancelProfile") != null) {
            // pending_profile.jsp loaded this JSP with the
            // explicit instruction to cancel any existing
            // pending profile.
            myDiscovery.getComplex().cancelPendingProfile();
        } else {
            // Check for a pending profile to see if there is one there.
            myDiscovery.clearPendingProfileCache();
            Complex pcomplex = myDiscovery.getPendingComplex();

            if (pcomplex != null && !pcomplex.isErrorObject()) {
                // A pending profile exists.  Forward to the
                // pending_profile.jsp error page.
                pageContext.forward("pending_profile.jsp");
            } else {
                // No pending profile exists.  Lower the 
                // checkPending flag since we have now checked
                // for the pending profile once.
                parsession.setCheckPending(false);
            }
        }

    } else {
        // The user has just come from the pending profile screen
        // and hit the Continue button there.  The user wants to
        // proceed without checking for pending profiles.  (Note 
        // that this flag is reset when the complex data is refreshed 
        // or when a task that depends on a current picture of the
        // pending profile status is executed).
        parsession.setCheckPending(false);
    }
%>

