<script language='javascript'>
<%--
/*
 * run the cell option checks and display any error messages.
 *
 * @param myForm a form object that contains the cell options selectors.
 * @return true or false depending on if the checks where successful or not.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect a message dialog will be invoked with the error message if the
 *             user entered an inapropriate value.
 */
--%>
function validateCellOptions(myForm){
    if (doNotValidate){
        return true;
    }

    var hasErrorCellsValue

    if (isIE()) {
        hasErrorCellsValue = myForm.document.getElementById("haveErrorCells").value
    } else {
        hasErrorCellsValue = myForm.ownerDocument.getElementById("haveErrorCells").value
    }

    if (hasErrorCellsValue == "true") {
        var msg = validateErrorCoreCells(myForm);
        if (msg.length != 0) {
            alert(msg);
            return false;
        }

        msg = validateErrorBootCells(myForm);
        if (msg.length != 0) {
            alert(msg);
            <%-- let them continue in this case --%>
            return true;
        }

        return true;
    }

    var cells = new Array();
    var re = /coreCellChoice/;
    var cellRe = /\d+/;
    var j = 0;

    <%--
        Loop through the elements on the form.  The core cell choice is 
        identified with any row in the table that has a selector for core cell
        choices.  The actual choice does not matter.
    --%>
    for (var i = 0; i < myForm.elements.length; ++i)
    {
        <%-- get the elements name --%>
        var name = myForm.elements[i].name;

        <%--
            If this element is a coreCellChoice element store
            the cell number in the cells array.
        --%>
        if (name.search(re) != -1)
        {
           cells[j++] = myForm.elements[i].name.match(cellRe); 
        }
    }

    var msg = checkUseOnNextBoot(myForm, cells) + 
              checkFailureUsage(myForm, cells) + 
              checkCoreChoices();

    if (msg.length != 0){
        alert(msg);
        return false;
    }

    return checkAllCoreChoicesNone();
}

<%-- global defines for cell usage. --%>
var BASE = "2";
var BASE_NO_INTERLEAVE = "3";
var FLOATING = "4";
var DISABLE = "5";

<%--
/*
 * Make sure there is at least one viable core cell that will be used on 
 * failure.
 *
 * @param myForm a form object that contains the cell options selectors.
 * @return an empty string or an error message if the check failed.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
--%>
function checkFailureUsage(myForm, cells){

    var numDisabledCells = 0;
    var msg = '<%=bundle.getString("error.cellFailureUsage.message")%>';

    <%--
        Loop through the elements on the form and see if there is at least
        one core cell that will be used on next boot.
    --%>
    for (var i = 0; i < myForm.elements.length; ++i)
    {
        <%--
            Loop through the array of cells created above (in
            the function that called this one).  If one of the
            cells in the array is set to be used on next boot
            return true.  Otherwise we will fall through the loop.
        --%>
        for (var j = 0; j < cells.length; ++j)
        {
            var cellFailureUsage = "failureUsage" + cells[j];
            if ((myForm.elements[i].name == cellFailureUsage) && 
                (myForm.elements[i].value == DISABLE))
            {
                ++numDisabledCells;
            }
        }
    }

    if (numDisabledCells == cells.length){
        return msg + "\n\n";
    }

    return "";
}

<%--
/*
 * make sure there is at least one viable core cell that will be used on next 
 * boot
 *
 * @param myForm a form object that contains the cell options selectors.
 * @return an empty string or an error message if the check failed.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
--%>
function checkUseOnNextBoot(myForm, cells){
    var msg = '<%=bundle.getString("error.cellUseOnNextBoot.message")%>';

    <%--
    Loop through the elements on the form and see if there is at least
    one core cell that will be used on next boot.
    --%>
    for (i = 0; i < myForm.elements.length; ++i)
    {
        <%--
        Loop through the array of cells created above.  If
        one of the cells in the array set to be used on
        next boot return true.  Otherwise we will fall
        through the loop.
        --%>
        for (j = 0; j < cells.length; ++j)
        {
            var cellUseOnNextBoot = "useOnNextBoot" + cells[j];
            if ((myForm.elements[i].name == cellUseOnNextBoot) &&
                (myForm.elements[i].value == "true"))
            {
                return "";
            }
        }
    }

    return msg + "\n\n";
}

<%--
/**
 * Check to see if all the core cell choices are set to none.  If so, warn the
 * user.
 *
 * @precondition the function validateCoreChoices must be ran first.
 *
 */
--%>
function checkAllCoreChoicesNone(){
    var msg = '<%=bundle.getString("coreChoiceNone.warning")%>';

    if (noCoreChoices) {
        return confirm(msg);
    }

    return true;
}

<%--
/*
 * make sure there is no duplicate.  If there is a duplicate, then the cell 
 * that didn't trigger the event will have it's core choice set to none.
 *
 * @param selectedIndex the index number of the selected option.
 * @param numSelections the number options in the select menu.
 * @param rowIndex the row number of the cell in the cell table.
 * 
 * @return None.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect The core cell choices for cells showing on the screen might
 *               be modified to "none".
 */
--%>
function validateCoreChoices(selectedIndex, numSelections, rowIndex){

    <%--
    when the user sets a core choice, make sure to shift the core choices
    around when neccessary so there will not be duplicate core choices.
    --%>
    if (selectedIndex == 0){
        return true;
    }

    var re=/coreCellChoice/;
    var elements = document.setCellOptions.elements;
    var eLength = elements.length;
    var cells = new Array();
    var j = 0;

    <%--
    get the list of viable core cell choices, less the cell that triggered
    this event.
    --%>
    for (var i = 0; i < eLength; ++i){

        var name = elements[i].name;

        if (name.search(re) != -1){

            var row = elements[i].parentNode.parentNode;

            if ((row.rowIndex != rowIndex) && 

                (elements[i].selectedIndex == selectedIndex)){

                <%--
                this is a different core cell viable cell than the one
                that triggered the onChange event, and it has the same
                core choice. Set this cells choice to none.
                --%>
                elements[i].selectedIndex = 0;
            }
        }
    }
}

<%--
/*
 * Check to see if there are any holes in core cell choices, i.e. 1, 2, and 4
 * but no 3.  Also see if all the core choices are set to "None".
 *
 * @return an empty string or an error message if the check failed.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
--%>
var noCoreChoices = false;
function checkCoreChoices(){
    var re=/coreCellChoice/;
    var elements = document.setCellOptions.elements;
    var eLength = elements.length;
    var numSelections = 0;
    var coreNone = 0;
    var numCoreCells = 0;
    var msg = '<%=bundle.getString("error.coreChoices.message")%>';
    noCoreChoices = false;

    <%-- use flags for core cell choices. --%>
    var choices = [false, false, false, false, false];

    <%-- find out which cells have a core cell choice, and what is it. --%>
    for (var i = 0; i < eLength; ++i){
        var name = elements[i].name;
        if (name.search(re) != -1){
            <%-- get the number of core cell choices showing on the screen. --%>
            if (numSelections == 0){
                numSelections = elements[i].length;
            }
            <%-- set the used flag for this core cell choice to true. --%>
            choices[elements[i].selectedIndex] = true;

            <%-- 
                find out if this core capable cell does not have a core choice. 
            --%>
            if (elements[i].selectedIndex == 0){
                ++coreNone;
            }

            <%-- get the number of core capable cells. --%>
            ++numCoreCells;
        }
    }

    <%--
    if all of the core capable cells have "none" as the selection, then
    return and do not go through the continuity check below.  
    --%>
    if (coreNone == numCoreCells){
        noCoreChoices = true;
        return "";
    }

    <%--
    check to see if a higher choice is used without a lower choice being
    used.  Choice 0 does not matter because it is the choice for not using a
    cell as a core cell choice.
    --%>
    for (var i = numSelections; i > 0; --i){
        if (choices[i]){
           for(var j = 1; j < i; ++j){
             if(!choices[j]){
                return msg;
             }
           }
        }
    }

    return "";
}

<%--
/*
 * Adjust the failure usage options based on the user's cell type
 * selection.  If the user sets a cell's type to floating, only
 * give two failure usage options: "floating" or "do not use".
 *
 * @param type the user selection for the cell type
 * @param failureUsage the select object for setting the failure usage for
 *        this cell.
 * @return None.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect The failure usage choices for cells showing on the screen will
 *               be modified.
 */
--%>
function adjustCellFailureUsageOptions(type, failureUsage){

    TYPE_FLOATING="1";

    <%-- get the current user selected failure usage. --%>
    var sFailureUsage = failureUsage.selectedIndex;

    if (type.selectedIndex == TYPE_FLOATING){

        failureUsage.options[0].text = "Floating";
        failureUsage.options[0].value = FLOATING;
        failureUsage.options[1].text = "Disable";
        failureUsage.options[1].value = DISABLE;
        failureUsage.options[2] = null;
        failureUsage.options[3] = null;
        failureUsage.options.length = 2;
        
        <%-- 
            in this case, the user changed the cell from type base to floating.
            The indexes have now changed because none of the base options are available.
            Whatever index they had selected before should be preserved if it is still available.
            The if/else ladder should map the previously available indexes to the currently available indexes.
        --%>
        if (sFailureUsage == 2) {
            sFailureUsage = 0;
        } else if (sFailureUsage == 3) {
            sFailureUsage = 1;
        } else {
            sFailureUsage = 0;
        }

    } else {

        failureUsage.options[0].text = "Base";
        failureUsage.options[0].value = BASE;
        failureUsage.options[1].text = "Base, no interleave";
        failureUsage.options[1].value = BASE_NO_INTERLEAVE; 
        failureUsage.options[2] = new Option ("Floating", FLOATING);
        failureUsage.options[3] = new Option ("Disable", DISABLE); 
        <%-- 
            in this case, the user has changed the cell from type floating to type base.  
            The indexes have now changed because none of the base options are available.
            Whatever index they had selected before should be preserved if it is still available.
            The if/else ladder should map the previously available indexes to the currently available indexes.
        --%>
        if (sFailureUsage == 0) {
            sFailureUsage = 2;
        } else if (sFailureUsage == 1) {
            sFailureUsage = 3;
        } else {
            sFailureUsage = 0;
        }
    }

    failureUsage.selectedIndex = sFailureUsage;
}

<%--
/*
 * Validate that the partition has at least one core cell with use-on-next-boot
 * set to true.  The core cell may be an error cell.
 *
 * This function should be used when there is at least one error cell in the
 * partition.
 *
 * @param myForm a form object that contains the cell options
 * @return a message, which is the empty string if the validation passes.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
--%>
function validateErrorCoreCells(myForm) {
    var canBeCoreRE = /canBeCore/;
    var coreCells = new Array();
    var c = 0;
    var useOnNextBootRE = /useOnNextBoot/;
    var bootCells = new Array();
    var b = 0;

    var cellRE = /\d+/;

    <%--
    Get all the information we need to determine warnings.
    --%>
    for (j=0; j<myForm.elements.length; ++j) {
        var name = myForm.elements[j].name;
        var value = myForm.elements[j].value;

        if (name.search(canBeCoreRE) != -1) {
            coreCells[c++] = name.match(cellRE);
        } else if (name.search(useOnNextBootRE) != -1 && value == "true") {
            bootCells[b++] = name.match(cellRE);
        }
    }

    <%--
    One of the core cells must be a boot cell (or another way, one of
    the boot cells must be a core cell).
    --%>
    var found = false;
    for (j=0; j<coreCells.length; ++j) {
        for (k=0; k<bootCells.length; ++k) {
            if (coreCells[j][0] == bootCells[k][0]) {
                found = true;
            }
        }
    }

    var msg = "";
    if (!found) {
        msg = '<%=bundle.getString("error.cellUseOnNextBoot.message")%>';
    }

    return msg;
}

<%--
/*
 * Determine if the partition has one or more error cells that are marked
 * use-on-next-boot.
 *
 * This function should be used when there is at least one error cell in the
 * partition.
 *
 * @param myForm a form object that contains the cell options
 * @return a message, which is the empty string if the validation passes.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
--%>
function validateErrorBootCells(myForm) {
    var useOnNextBootRE = /useOnNextBoot/;
    var bootCells = new Array();
    var b = 0;
    var errorCellRE = /errorCell/;
    var errorCells = new Array();
    var e = 0;

    var cellRE = /\d+/;

    <%--
    Get all the information we need to determine warnings.
    --%>
    for (var j=0; j<myForm.elements.length; ++j) {
        var name = myForm.elements[j].name;
        var value = myForm.elements[j].value;

        if (name.search(useOnNextBootRE) != -1 && value == "true") {
            bootCells[b++] = name.match(cellRE);
        } else if (name.search(errorCellRE) != -1 && value == "true") {
            errorCells[e++] = name.match(cellRE);
        }
    }

    <%--
    We want to display a warning if any of the boot cells are
    also error cells (or put another way, if any of the error
    cells are marked to boot).
    --%>
    var found = false;
    for (var j=0; j<errorCells.length; ++j) {
        for (var k=0; k<bootCells.length; ++k) {
            if (errorCells[j][0] == bootCells[k][0]) {
                found = true;
            }
        }
    }

    var msg = "";
    if (found) {
        msg = '<%=bundle.getString("error.errorCellUseOnNextBoot.message")%>';
    }

    return msg;
}

</script>
