<script language='javascript'>
<%--
myDoc is a global for referring to the current document.  It is not  
needed when referring to a form by its name or via the forms array.
--%>

<%--
/*
 * Round a memory amount to the nearest .5 GB, for example:
 *	3.2 -> 3.0
 *	3.4 -> 3.5
 *
 * @param amount the number that should be rounded.
 * @return a float number that is rounded to the nearest .5 .
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
--%>
function roundMemAmount(amount){

    return Math.round(amount * 2) / 2.0;
}

<%--
/*
 * Format a number to a floating point number by * adding ".0" to the number 
 * if the number is a whole number. This is because * a float such as "3.0" 
 * will print to the screen as "3" by javascript.
 *
 * @param amount the number to be formatted.
 * @return a string holding a float number formatted to 1 decimal point.
 * 
 * @precondition None.
 * @postcondition None.
 * @sideeffect None.
 */
--%>
function formatFloat(amount){

    if (parseInt(amount) == parseFloat(amount)){

        return amount+".0";

    } else {

        return amount;
    }
}

// field name prefixes.
var cellAmountStr = "clmAmount";
var cellRatioStr = "clmFraction";
var cellTotalStr = "cellTotal";
var cellInterleaveStr = "cellInterleave";
var cellCLMStr = "cellCLM";
var numCLMTableHeaderRows = 1;
var numCLMTableFooterRows = 2;

<%--
//
// Update the total CLM and interleave value for the table based on
// changes in the requested CLM amount for a single cell.
//
--%>
function updatePerCellTotals(cellCLM, cellID){
    // update the requested interleave column for the cell.
    var cellTotalMem = getCellTotalMem(cellID);
    var newCellInterleave = cellTotalMem - cellCLM;
    var cellInterleave = document.getElementById(cellInterleaveStr+cellID);
    cellInterleave.innerHTML = formatFloat(newCellInterleave);

    // update the total CLM field and the total interleave field
    updateTotals();
}

<%--
Add up the totals for either requested CLM or interleaved memory.
--%>
function updateTotals(){
    var totalCLM = 0;
    var totalInt = 0;
    var bootableCLM = 0;
    var bootableInt = 0;
    var bootableMem = 0;
    var yesStr = "<%=bundle.getString("field.YES.label")%>";
    var noStr = "<%=bundle.getString("field.NO.label")%>";
    var clmStr;

    var clmTable = document.getElementById("CLMInputTable");

    // the interleave col is always the last col.
    var intCOL = clmTable.rows[0].cells.length - 1;
    var useOnNextBootCOL = 2;

    // need to skip the header row and the total row(s).
    for (var i = numCLMTableHeaderRows; i < clmTable.rows.length - numCLMTableFooterRows; ++i) {
        clmStr = clmTable.rows[i].cells[intCOL].innerHTML;  
        if (!isNaN(clmStr)) {
            var clm = parseFloat(clmStr);
            totalInt += clm;
            useOnNextBoot = clmTable.rows[i].cells[useOnNextBootCOL].innerHTML;
            if (useOnNextBoot == yesStr) {   
                bootableInt += clm;
            }
        }
    }

    var totalMem = getTotalMemory();
    totalCLM = totalMem - totalInt;

    var totalCLMField = document.getElementById("totalCLM");
    totalCLMField.innerHTML = formatFloat(totalCLM);

    var totalIntField = document.getElementById("totalInterleave");
    totalIntField.innerHTML = formatFloat(totalInt);
    
    var bootableMem = getTotalBootableMemory();
    bootableCLM = bootableMem - bootableInt;

    var bootableCLMField = document.getElementById("totalBootableCLM");
    bootableCLMField.innerHTML = formatFloat(bootableCLM);

    var bootableIntField = document.getElementById("totalBootableInterleave");
    bootableIntField.innerHTML = formatFloat(bootableInt);
}

<%--
Get the total good memory for a cell from the table.
--%>
function getCellTotalMem(cellID){
    var cellTotal = document.getElementById(cellTotalStr+cellID);
    var cellTotalMem = parseFloat(cellTotal.innerHTML);
    return cellTotalMem;
}

<%--
Get the total good memory for the partition from the total memory field.
--%>
function getTotalMemory(){
    var totalMemField = document.getElementById("totalMemory");
    var totalMem = parseFloat(totalMemField.innerHTML);
    return totalMem;
}
  
<%--
Get the total good bootable memory for the partition from the total bootable memory field.
--%>
function getTotalBootableMemory(){
    var bootableMemField = document.getElementById("totalBootableMemory");
    var bootableMem = parseFloat(bootableMemField.innerHTML);
    return bootableMem;
}

<%--
This function is called when the user changes a cell's CLM request via the 
fraction selector.
--%>
var CLM_FRACTION_DENOMINATOR = 8.0;
function updatePerCellFraction(selectList, cellID){
    var selectedIndex = selectList.selectedIndex;
    var cellAmount = document.getElementById(cellAmountStr+cellID);

    if (selectedIndex == selectList.length - 1){
        // set the CLM amount to 0 since the user selected NA for fraction.
        cellAmount.value = '0.0';
        return;
    }

    var fraction = selectList.options[selectedIndex].value;

    // get the total cell memory
    var cellTotalMem = getCellTotalMem(cellID);

    // calculate new CLM amount for the cell based on ratio.
    var newCellCLM = cellTotalMem * fraction / CLM_FRACTION_DENOMINATOR;

    // change the cell CLM amount input in table to N/A;
    cellAmount.value = '<%=bundle.getString("cell.NA.string")%>';

    // update the requested CLM and interleaved totals.
    updatePerCellTotals(newCellCLM, cellID);
}

<%--
This function is called when the user changes a cell's CLM request via the 
amount input.
--%>
function updatePerCellAmount(cellAmount, cellID){
    // get the total cell memory
    var newCellCLM = parseFloat(cellAmount.value);
    updatePerCellTotals(newCellCLM, cellID);
    
    // set the fraction choice to N/A.
    var cellRatio = document.getElementById(cellRatioStr+cellID);
    cellRatio.options[cellRatio.selectedIndex].selected = false;
    cellRatio.options[cellRatio.length - 1].selected = true;
}

<%--
//
// Validate the value in the text box that represents an amount
// of memory.  If the value is greater than the max, change it to
// the max.  If the value is less than the min, change it to the min.
// Then round it to the nearest 0.5 GB value. 
//
--%>
function validateMemAmount(max, mem, min){

    var totalMem = parseFloat(mem);
    var newMem = roundMemAmount(mem.value);

    if ((newMem < min) || isNaN(newMem)){
        mem.value = formatFloat(roundMemAmount(min));
    } else if (newMem > max) {
        mem.value = formatFloat(roundMemAmount(max));
    } else {
        mem.value = formatFloat(newMem);
    }
}

<%--
There are several things to check for:
1. The amount of total interleave memory is at least minZI and
   a power of 2 GB, 
2. On a PA-RISC npar, the amount of total interleave memory is at least
   1.5 GB + total_memory/32 (rounded up to nearest 0.5 GB).
2. The amount of requested interleave memory is the same for all base cells that
   are participating in the interleave.
3. The number of base cells participating in the interleave is a power of 2.
--%>
function checkTotalInterleave(myForm) {
    if (doNotValidate){
        return true;
    }

    <%--
        Make sure that for every cell, only fraction or amount is filled in.
    --%>
    tooMuchInformation = ! checkFractionAndAmount(myForm);
    if (tooMuchInformation) {
        alert("<%=bundle.getString("clm.fraction.and.amount.str")%>");
        return false;
    }

    <%-- Now do the rest of the memory checks --%>
    var totalBootableMem = getTotalBootableMemory();
    var totalBootableIntField;
    var totalBootableInt;

    totalBootableIntField = document.getElementById("totalBootableInterleave");
    totalBootableInt = parseFloat(totalBootableIntField.innerHTML);

    var msg = "";
    var error = false;
    var minOSInterleave = "<%=parConfig.getMinOSInterleave()%>";

    if( totalBootableInt < minOSInterleave ) {
            error = true;
            msg = "<%=bundle.getString("clm.minOSZI.1.warning")%>"
                  + " " + minOSInterleave + " " 
                  + "<%=bundle.getString("clm.minOSZI.2.warning")%>"
                  + " " + totalBootableMem + " " 
                  + "<%=bundle.getString("clm.minOSZI.3.warning")%>";
    }
    else {
        var minInterleave= "<%=parConfig.getMinInterleave()%>";
        if( totalBootableInt < parseFloat(minInterleave) ) {
            msg = "<%=bundle.getString("clm.minZI.1.warning")%>"
                  + minInterleave + " " 
                  + "<%=bundle.getString("clm.minZI.2.warning")%>";
        }
    }

    if ( ! isPowerOf2(totalBootableInt*2) ||
         ! isNumBaseCellsPowerOf2() ||
         ! isInterleaveAmountSameForAllBaseCells() )
    {
        if( msg != "" ) msg = msg + "\n\n----------\n\n";
        msg = msg 
              + "<%=bundle.getString("clm.totalInterleavedNotOptimal.warning")%>"
              + "\n" 
              + "<%=bundle.getString("clm.totalInterleaveRules.str")%>";
;
    } 
    if( msg != "" ) {
        if (error) {
            alert(msg);
            return false;
        }
        else {  
            return( confirm( msg + "\n\n" + "<%=bundle.getString("clm.clickOK")%>"));
        }
    }

    return true;
}

<%--
Check if the number of base cells that are participating in the interleave is
a power of 2.
@issue:11.31:uncomment the code when we support floating cells.
--%>
function isNumBaseCellsPowerOf2(){
    var clmTable = document.getElementById("CLMInputTable");

    // the cell type col is always the second col.
    var cellTypeCol = 1;
    var baseStr = "<%=bundle.getString("cell.usage.base")%>";
    var intCOL = clmTable.rows[0].cells.length - 1;
    var numBaseCells = 0;

    // need to skip the header row and the total row(s).
    for (var i = numCLMTableHeaderRows; i < clmTable.rows.length - numCLMTableFooterRows; ++i){
        cellTypeStr = clmTable.rows[i].cells[cellTypeCol].innerHTML; 
        intStr = clmTable.rows[i].cells[intCOL].innerHTML;

        if (
            (cellTypeStr == baseStr) && 
            (parseFloat(intStr) > 0)) 
        {
            numBaseCells ++;
        }
    }

    return isPowerOf2(numBaseCells);
}

<%--
Check if the amount of requested interleave is the same on all participating
base cells.
--%>
function isInterleaveAmountSameForAllBaseCells(){
    var clmTable = document.getElementById("CLMInputTable");

    // the interleave col is always the last col.
    var intCOL = clmTable.rows[0].cells.length - 1;
    var amount = 0;
    var intStr1;
    var intStr2;

    // find a value to compare against. no need to check cell type because 
    // floating cells will always have a "0" in the requested interleave column.
    for (var i = numCLMTableHeaderRows; i < clmTable.rows.length - numCLMTableFooterRows; ++i){
        intStr1 = clmTable.rows[i].cells[intCOL].innerHTML;
        if (parseFloat(intStr1) > 0){
            break;
        }
    }

    // need to skip the header row and the total row.
    for (var i = numCLMTableHeaderRows; i < clmTable.rows.length - numCLMTableFooterRows; ++i){
        intStr2 = clmTable.rows[i].cells[intCOL].innerHTML;

        if (parseFloat(intStr2) > 0){
            if (parseFloat(intStr1) != parseFloat(intStr2)){
                return false;
            }
        }
    }

    return true;
}

<%--
    Make sure that none of the cells have both fixed and fraction set.
    This could happen in very obscure circumstances, but it *can* happen.

    When the fraction is set and the user fills in a fixed value, then clicks
    on the fraction drop down and either chooses the same fraction or doesn't
    choose any fraction (clicks somewhere else to remove focus from the
    dropdown), this leaves the fraction set to what it was, and the fixed
    field filled in, with possibly incorrect calculated total interleave for
    the cell.  Further, when the user clicks OK, they get an exception and a
    stack trace.

    We can't prevent the user from getting both fields filled in through
    this strange mechanism, so we catch it here and prevent the user
    from leaving the form in this state.
--%>
function checkFractionAndAmount(myForm) {

    <%--
        Loop through the elements on the form and figure out what cells are
        displayed.  There will be one clmFraction element for each cell, so
        look at those (could just as easily look at clmAmount or some other
        element).
    --%>
    var re = /clmFraction/;
    var cellRe = /\d+/;

    for (var i=0; i < myForm.elements.length; i++) {

        <%-- get the element name --%>
        var name = myForm.elements[i].name;

        <%--
            If this element is a clm fraction element, get the cell id
            and then the clm amount.
        --%>
        if (name.search(re) != -1) {

            <%--
                This is a clmFractionx, where x is a cellID.  Get the actual
                form element for the fraction and the corresponding clm amount.
            --%>
            cellID = myForm.elements[i].name.match(cellRe); 
            var amount = document.getElementById("clmAmount"+cellID);
            var fraction = document.getElementById("clmFraction"+cellID);

            <%--
                If they both have a value, we throw up an alert and make them
                fix it.  For the fraction value, nine is the last element on the
                list, which is "-"
            --%>
            if ((fraction.value != 9) && (amount.value != "-")) {
                return false;
            }
        }
    }

    return true;
}

<%--
Log base 10 function.
--%>
function log10(amount){
    return Math.log(amount)/Math.log(10);
}

<%--
Log base 2 function.
--%>
function log2(amount){
    return Math.log(amount)/Math.log(2);
}

<%--
Check if a number is a power of 2.
0 is not really a power of 2, but is valid
in the cases where we use this check.
--%>
function isPowerOf2(amount){
    if( amount == 0 ) return true;
    var power = log2(amount);
    return (parseInt(power) == parseFloat(power));
}

</script>
