<%--
/**
 * The main JSP page for the "Complex Scope".
 * <p>
 * This page contains the main structure of the "Complex Scope".
 * It sets up the ParmgrSession (parsession) object and the
 * discovery variable used by other pages included via which
 * tab is selected. It also includes the parmgr javascript code
 * and stylesheets used in the complex scope.
 * <p>
 * Depending on which tab is selected, this JSP includes the
 * appropriate JSP for the body of the view:
 *   Tab      JSP
 *  Hardware complex_hardware.jsp
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/complex.jsp,v 1.65 2008/03/05 12:40:57 sridharav Exp $
 * @risk Medium
 *
 * @task name = Create "refresh" button
 *       cycle = 5
 *       phase = TEST
 *       owner = Paul
 *       status = Done
 *       description = Create mechanism to refresh data
 *
 * @task name = Create printer-friendly links for all Complex views
 *       cycle = 9
 *       phase = DESIGN
 *       owner = Paul
 *       status = Done
 *       description = Create JSP which does not include links or checkboxes.
 */
--%>
<%@ taglib uri="/com/hp/mx/taglibs/MxTabsTag" prefix="tabs" %>
<%@ taglib uri="/com/hp/parmgr/ui/tags/parmgr" prefix="parmgr" %>
<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<%@ page errorPage="exception.jsp" %>

<%@ page import="com.hp.parmgr.util.Menu" %>
<%@ page import="com.hp.parmgr.util.StringFormatUtil" %>
<%@ page import="com.hp.parmgr.ui.SessionTimeOut" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<i18n:bundle baseName="com.hp.parmgr.ui.ComplexView" id="bundle" />
<%@ include file="session_init.jsp" %>

<%
    // JAGaf88550: guard against cross site scripting vulnerabilities,
    // by encoding parameters that may be output to the HTML stream:
    String szSelected =
        StringFormatUtil.encodeHtmlEntities(request.getParameter("selection"));
    if (szSelected == null)
        szSelected = "HARDWARE";
    String fromURL="complex.jsp";

    com.hp.parmgr.model.Discovery discovery;
    com.hp.parmgr.ui.SelectionManager selectedCheckboxes;
    discovery = parsession.getDiscovery();

    if (request.getParameter("refresh") != null) {
       /* Must clear cache before generating the HTML. */
       discovery.clearCache();
       parsession.setCheckPending(true);
    }
%>

<%@ include file="run_analyze_complex_health.jsp" %>
<%@ include file="check_pending_profile.jsp" %>

<%
    String complexName = discovery.getComplex().getName();
    selectedCheckboxes = parsession.getSelectedCheckboxes();

    /* The following variables indicate to the scope_header.jsp
     * that we are in the complex scope.
     */
    String currentURL = "complex.jsp";
    String partitionName = null;
    String parID = null;
    String cellLocation = null;
    String chassisName = null;
    boolean isPartitionValid = false;
    boolean isCellIDPresent = false;
    boolean validChassisID = false;
%>

<HTML>
<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<META HTTP-EQUIV="expires" CONTENT="0">
<meta http-equiv="Cache-Control" CONTENT="no-cache">
<meta http-equiv="Pragma" CONTENT="no-cache">
<%@ include file="session_keep_alive.jsp" %>
<tabs:MxTabs dumpHeadContent="true" urlBase="<%=Util.URL_BASE%>"/>

<i18n:bundle baseName="com.hp.parmgr.ui.tags.tags" id="parmgrTagResources" />

<script type='text/javascript' language='JavaScript' src='<%=Util.URL_PARMGR_JS%>'></script>
<%=SessionTimeOut.javascript(session, bundle.getLocale())%>
<link type='text/css' rel='stylesheet' href='<%=Util.URL_PARMGR_CSS%>'>
<link type='text/css' rel='stylesheet' href='<%=Util.URL_HARDWARE_CSS%>'>
<TITLE>Partition Manager - Complex <%=complexName%></TITLE>
</HEAD>

<BODY>

<table border=0 cellpadding=0 cellspacing=0 width="100%" height="100%">
<tr>
    <td>
        <%@ include file="scope_header.jsp" %>
    </td>
</tr>
<tr><td class=smc-filler>
<table border="0" cellpadding="0" cellspacing="0" width="100%">
<tr><td width="100%">
<tabs:MxTabs selection="<%=szSelected%>" urlBase="<%=Util.URL_BASE%>">
    <selectors>
        <selector id="HARDWARE">
            <label><i18n:message key="tab.Hardware.label" bundle="<%=bundle%>" /></label>
            <url>
                <%=request.getRequestURI()%>?selection=HARDWARE
            </url>
        </selector>

        <selector id="PARTITIONS">
            <label><i18n:message key="tab.Partitions.label" bundle="<%=bundle%>" /></label>
            <url>
                <%=request.getRequestURI()%>?selection=PARTITIONS
            </url>
        </selector>

        <selector id="POWERCOOLING">
            <label><i18n:message key="tab.PowerCooling.label" bundle="<%=bundle%>" /></label>
            <url>
                <%=request.getRequestURI()%>?selection=POWERCOOLING
            </url>
        </selector>

        <selector id="CELLS">
            <label><i18n:message key="tab.Cells.label" bundle="<%=bundle%>" /></label>
            <url>
                <%=request.getRequestURI()%>?selection=CELLS
            </url>
        </selector>

        <selector id="IO">
            <label><i18n:message key="tab.IO.label" bundle="<%=bundle%>" /></label>
            <url>
                <%=request.getRequestURI()%>?selection=IO
            </url>
        </selector>
        <selector id="GENERAL">
            <label><i18n:message key="tab.General.label" bundle="<%=bundle%>" /></label>
            <url>
                <%=request.getRequestURI()%>?selection=GENERAL
            </url>
        </selector>
    </selectors>
</tabs:MxTabs>
</td>
<%@ include file="logofflink.jsp"%>
</tr>
</table>

</td></tr>

<tr height="100%"><td bgcolor=white>

<table border=0 cellpadding=0 cellspacing=8 width="100%" height="100%">
<tr>
  <td>

<%
  String pageID="";
  String srcJsp="";

  if (szSelected.equals("HARDWARE")) {
    pageID = "complexHardwareView";
    srcJsp = "complex_hardware.jsp";
  } else if (szSelected.equals("PARTITIONS")) {
    pageID = "complexPartitionView";
    srcJsp = "complex_partition.jsp";
  } else if (szSelected.equals("POWERCOOLING")) {
    pageID = "complexPowerCoolingView";
    srcJsp = "complex_power_cooling.jsp";
  } else if (szSelected.equals("CELLS")) {
    pageID = "complexCellView";
    srcJsp = "complex_cells.jsp";
  } else if (szSelected.equals("IO")) {
    pageID = "complexIOView";
    srcJsp = "complex_io.jsp";
  } else if (szSelected.equals("GENERAL")) {
    pageID = "complexGeneralView";
    srcJsp = "complex_general.jsp";
  }

  selectedCheckboxes.resetVisibleCheckboxes();
  parsession.setImplicitPartitionID(Menu.INVALID_PARTITION_ID);
  parsession.setImplicitCellID(Menu.INVALID_CELL_ID);
  parsession.setImplicitChassisID(Menu.INVALID_CHASSIS_ID);
  parsession.setPageID(pageID);
  parsession.setFromURL(fromURL + "?selection=" + szSelected);
  if (!discovery.isCached()) {
      srcJsp = Util.URL_BASE_JSP + "/loading.jsp?src=" + srcJsp;
  }

  // start a thread to cache partition info (first time only)
  parsession.initPartitionInfo();
%>

  <iframe name="FRAME_PARMGR_WORKSPACE" id="FRAME_PARMGR_WORKSPACE"
        scrolling=auto frameborder=0 width="100%" height="100%"
        src="<%=srcJsp%>">
  </iframe>
  </td>

<%-- /* Note that no source is specified for the menu actions frame in
      * this JSP.  The menu src will be loaded by the workspace frame
      * when it is done loading to ensure that the workspace has
      * completed rendering before the actions menu is created.
      */
--%>

  <td width=220>
    <BR>
    <iframe name="FRAME_PARMGR_ACTIONS" id="FRAME_PARMGR_ACTIONS"
        scrolling=auto frameborder=0 width="100%" height="100%"
        src="dummy.jsp">
    </iframe>
  </td>
</tr>
</table>

</td></tr>
</table>

</BODY>
</HTML>
