<%--
/**
 * The content for the complex details page.  This page is a full
 * listing of all the components in the complex with their status.
 * <p>
 * The layout of the page is as follows:
 *   Complex Level:
 *     Data from complex scope, general tab
 *     Table of cells in the complex
 *     Table of cpus and memory in the complex
 *     Table of io chassis in the complex
 *     Table of cabinets in the complex
 *     Table of power and cooling domains in the complex
 *   Partition Level (for each partition):
 *     Data from partition scope, general tab
 *     Table of cells in the partition
 *     Table of cpus and memory in the partition
 *     Table of io chassis in the partition
 *   Cell Level (for each cell):
 *     Data from cell scope, general tab
 *     Table of cpus on the cell
 *     Table of dimms on the cell
 *   IO Chassis Level (for each io chassis):
 *     Data similar to cell scope, general tab (but for io chassis)
 *     Table of io slots
 *   Power/Cooling Domain Level (for each cabinet):
 *     Table of cabinet cooling
 *     Table of io cooling in cabinet
 *     Table of backplane power boards in cabinet
 *     Table of bulk power supplies in cabient
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/complex_details.jsp,v 1.39 2008/03/05 12:40:57 sridharav Exp $
 * @risk: Low
 *
 * @task name = complex details
 * cycle = 4
 * phase = Test
 * owner = Bryan
 * status = Done
 * description = Implement the complex details page.
 */
--%>

<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>
<%@ taglib uri="/com/hp/parmgr/ui/tags/parmgr" prefix="parmgr" %>

<%@ page import="java.util.List" %>
<%@ page import="java.util.Arrays" %>
<%@ page import="java.util.ArrayList" %>
<%@ page import="java.util.ListIterator" %>
<%@ page import="java.util.Iterator" %>
<%@ page import="java.util.ResourceBundle" %>
<%@ page import="java.text.MessageFormat" %>
<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.HWPartition" %>
<%@ page import="com.hp.parmgr.model.Complex" %>
<%@ page import="com.hp.parmgr.model.Cell" %>
<%@ page import="com.hp.parmgr.model.DIMM" %>
<%@ page import="com.hp.parmgr.model.CellSlot" %>
<%@ page import="com.hp.parmgr.model.IOChassis" %>
<%@ page import="com.hp.parmgr.model.IOChassisSlot" %>
<%@ page import="com.hp.parmgr.model.IOSlot" %>
<%@ page import="com.hp.parmgr.model.IOCard" %>
<%@ page import="com.hp.parmgr.model.Domain" %>
<%@ page import="com.hp.parmgr.util.Log" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page import="com.hp.npar.mof.HP_NParPowerCoolingDomain" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<i18n:bundle baseName="com.hp.parmgr.ui.Global" id="globalResources" />
<i18n:bundle baseName="com.hp.parmgr.ui.ComplexDetails" id="complexDetailsResources" />
<i18n:bundle baseName="com.hp.parmgr.ui.tags.tags" id="parmgrTagResources" />
<%@ include file="session_init.jsp" %>

<%
    Log.summary("Displaying complex details page.");
    Discovery discovery = parsession.getDiscovery();
    Complex complex     = discovery.getComplex();
    List partitions     = discovery.getPartitions();
    List cells          = discovery.getCells();
    List cellSlots      = discovery.getCellSlots();
    List chassisSlots   = discovery.getChassisSlots();
    List cabinets       = discovery.getCabinets();
    List domains        = discovery.getDomains();
    String pageID       = "complexDetails"; // for help link
    String returnURL    = parsession.getFromURL();
    if (returnURL == null) {
        // The user came from the CLI, so there is no return URL.
        // And when they hit OK (to be sent to the complex BPV),
        // we don't want to display complex health again, since
        // it is displayed as part of complex details.  So clear
        // that flag.
        returnURL = "complex.jsp";
        parsession.setRunACH(false);
    }
%>

<HTML>
<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<META HTTP-EQUIV="expires" CONTENT="0">
<script type='text/javascript' language='JavaScript' src='<%=Util.URL_PARMGR_JS%>'></script>
<link type='text/css' rel='stylesheet' href='<%=Util.URL_PARMGR_CSS%>'>
<link type='text/css' rel='stylesheet' href='<%=Util.URL_MXTABLE_CSS%>'>
<link type='text/css' rel='stylesheet' href='<%=Util.URL_HARDWARE_CSS%>'>
<title><i18n:message key="ComplexDetails.PageTitle" bundle="<%=complexDetailsResources%>"/></title>
</HEAD>

<BODY>

<table class="properties" border="0" cellspacing="2" cellpadding="0">

<!-- begin application data -->
  <tr>
    <td colspan="2">
      <table class="properties" border="0" cellspacing="2" cellpadding="0" width="100%">
        <tr>
          <td class="partition-head" width="80%">
            <i18n:message key="Application.Name.long" bundle="<%=globalResources%>"/>
            (<i18n:message key="Application.Name.short" bundle="<%=globalResources%>"/>) -
            <i18n:message key="ComplexDetails.PageTitle" bundle="<%=complexDetailsResources%>"/><br>
            <%=discovery.getLastUpdateTime()%>
          </td>
        </tr>
        <tr>
          <td align="right" width="20%">
            <form method="post" action="<%=returnURL%>" name="complex_details_form">
            <p align="left">
              <input class="button" type="submit"
                     value="<%=parmgrTagResources.getString("Button.OK")%>"
                     name="OK" />
              <input class="button" type="button"
                     value="<%=parmgrTagResources.getString("Button.Help")%>"
                     name="help"
                     onclick="showHelp('<%=parsession.getHelpURL(pageID)%>')" />
            </p>
            </form>
          </td>
        </tr>
      </table>
    </td>
  </tr>
<!-- end application data -->
<!-- begin complex general data -->
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head">
    <i18n:message key="ComplexDetails.ComplexProperties" bundle="<%=complexDetailsResources%>">
        <i18n:messageArg value="<%= complex.getName() %>" />
    </i18n:message></td>
    </td>
  </tr>
  <tr>
    <td colspan="2">
      <hr class="separator">
    </td>
  </tr>
      <jsp:include page="complex_general_body.jsp" />
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>

  <tr>
    <td colspan="2" class="partition-head"><i18n:message key="ComplexDetails.AnalyzeComplex" bundle="<%=complexDetailsResources%>"/></td>
  </tr>
  <tr>
    <td colspan="2">
      <jsp:include page="analyze_complex_health.jsp">
        <jsp:param name="imbedded" value="true" />
      </jsp:include>
    </td>
  </tr>
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>

  <tr>
    <td colspan="2" class="partition-head"><i18n:message key="ComplexDetails.CellsInComplex" bundle="<%=complexDetailsResources%>"/></td>
  </tr>
  <tr>
    <td colspan="2">
      <parmgr:complexcelltable objects="<%=cellSlots%>"
                               resources="<%=parmgrTagResources%>"
                               discovery="<%=discovery%>" />
    </td>
  </tr>
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head"><i18n:message key="ComplexDetails.CPUsMemoryInComplex" bundle="<%=complexDetailsResources%>"/></td>
  </tr>
  <tr>
    <td colspan="2">
      <parmgr:cpumemorytable objects="<%=cellSlots%>"
                             resources="<%=parmgrTagResources%>"
                             discovery="<%=discovery%>" />
    </td>
  </tr>
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head"><i18n:message key="ComplexDetails.IOInComplex" bundle="<%=complexDetailsResources%>"/></td>
  </tr>
  <tr>
    <td colspan="2">
      <parmgr:iotable objects="<%=chassisSlots%>"
                      resources="<%=parmgrTagResources%>"
                      discovery="<%=discovery%>" />
    </td>
  </tr>
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head"><i18n:message key="ComplexDetails.CabinetsInComplex" bundle="<%=complexDetailsResources%>"/></td>
  </tr>
  <tr>
    <td colspan="2">
      <parmgr:cabinettable objects="<%=cabinets%>"
                           domains="<%=domains%>"
                           resources="<%=parmgrTagResources%>" />
    </td>
  </tr>
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head"><i18n:message key="ComplexDetails.PowerCoolingInComplex" bundle="<%=complexDetailsResources%>"/></td>
  </tr>
  <tr>
    <td colspan="2">
      <parmgr:domaintable objects="<%=domains%>"
                          resources="<%=parmgrTagResources%>" />
    </td>
  </tr>
<!-- end complex general data -->

<%-- *********************************************************
**** Partitions
**** **************************************************** --%>

<%  HWPartition nextPartition;
    List parChassisSlots;
    List parSlots;
    int parID;
    ListIterator partitionIter = partitions.listIterator();
    while( partitionIter.hasNext() ) {
        nextPartition = (HWPartition)partitionIter.next();
        parID = nextPartition.getID();
        String partitionName;
        if (nextPartition.isErrorObject()) {
            partitionName = nextPartition.getLabel();
        } else {
            partitionName = nextPartition.getName();
        }
        parSlots = discovery.getCellSlotsInPartition(parID);
        parChassisSlots = discovery.getChassisSlotsInPartition(parID);
%>
<!-- begin partition N data -->
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head">
    <i18n:message key="ComplexDetails.PartitionProperties" bundle="<%=complexDetailsResources%>">
        <i18n:messageArg value="<%= partitionName %>" />
    </i18n:message></td>
  </tr>
  <tr>
    <td colspan="2">
      <hr class="separator">
    </td>
  </tr>
  <tr>
    <td colspan="2">
      <jsp:include page="partition_general_body.jsp">
        <jsp:param name="partition" value="<%=parID%>" />
        <jsp:param name="makeLinks" value="false" />
      </jsp:include>
    </td>
  </tr>
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head"><i18n:message key="ComplexDetails.CellsInPartition" bundle="<%=complexDetailsResources%>"/></td>
  </tr>
  <tr>
    <td colspan="2">
      <parmgr:partitioncelltable objects="<%=parSlots%>"
                                 resources="<%=parmgrTagResources%>"
                                 discovery="<%=discovery%>" />
    </td>
  </tr>
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head"><i18n:message key="ComplexDetails.CPUsMemoryInPartition" bundle="<%=complexDetailsResources%>"/></td>
  </tr>
  <tr>
    <td colspan="2">
      <parmgr:cpumemorytable objects="<%=parSlots%>"
                             resources="<%=parmgrTagResources%>"
                             discovery="<%=discovery%>" />
    </td>
  </tr>
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head"><i18n:message key="ComplexDetails.IOInPartition" bundle="<%=complexDetailsResources%>"/></td>
  </tr>
  <tr>
    <td colspan="2">
      <parmgr:iotable objects="<%=parChassisSlots%>"
                      resources="<%=parmgrTagResources%>"
                      discovery="<%=discovery%>" />
    </td>
  </tr>
<!-- end partition N data -->
<% } // End while partitionIter.hasNext %>

<%-- *********************************************************
**** Cells
**** **************************************************** --%>
<%  ListIterator cellIter = cells.listIterator();
    List dimms = new ArrayList(16);
    while( cellIter.hasNext() ) {
        Cell nextCell = (Cell)cellIter.next();
        int cellID    = nextCell.getGlobalID();
        CellSlot slot = discovery.getCellSlot(cellID);
        List cpus     = null;
        dimms.clear();
        if (!nextCell.isErrorObject()) {
            cpus     = Arrays.asList(nextCell.getCPUs());
            DIMM [] dimmArray = nextCell.getDimms();
            for(int i = 0; i < dimmArray.length; ) {
                List pair = new ArrayList(2);
                pair.add(dimmArray[i]);
                i++;
                pair.add(dimmArray[i]);
                i++;
                dimms.add(pair);
            }
        }
%>
<!-- begin cell N data -->
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head">
    <i18n:message key="ComplexDetails.CellProperties" bundle="<%=complexDetailsResources%>">
        <i18n:messageArg value="<%= slot.getLongLabel() %>" />
    </i18n:message> </td>
  </tr>
  <tr>
    <td colspan="2">
      <hr class="separator">
    </td>
  </tr>
    <jsp:include page="cell_general_body.jsp">
      <jsp:param name="globalID" value="<%=cellID%>" />
      <jsp:param name="makeLinks" value="false" />
    </jsp:include>
<% if (!nextCell.isErrorObject()) { %>
  <tr>
    <td colspan="2">
      <parmgr:cputable objects="<%=cpus%>"
                       resources="<%=parmgrTagResources%>" />
    </td>
  </tr>
  <tr>
    <td colspan="2">
      <parmgr:dimmtable objects="<%=dimms%>"
                        resources="<%=parmgrTagResources%>" />
    </td>
  </tr>
<% } // if not error object %>
<!-- end cell N data -->
<% } // End while cellIter.hasNext %>

<%-- *********************************************************
**** I/O Chassis
**** **************************************************** --%>
<%  ListIterator chassisSlotIter = chassisSlots.listIterator();
    while( chassisSlotIter.hasNext() ) {
        IOChassisSlot nextSlot = (IOChassisSlot)chassisSlotIter.next();
        if( ! nextSlot.isPopulated() ) {
            continue;
        }
        IOChassis chassis = discovery.getChassis(nextSlot);
        List ioSlots = null;
        if (!chassis.isErrorObject()) {
            ioSlots = Arrays.asList(chassis.getIOSlots());
        }
%>
<!-- begin chassis N data -->
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head">
    <i18n:message key="ComplexDetails.IOChassisProperties" bundle="<%=complexDetailsResources%>">
        <i18n:messageArg value="<%= nextSlot.getLongLabel() %>" />
    </i18n:message></td>
  </tr>
  <tr>
    <td colspan="2">
      <hr class="separator">
    </td>
  </tr>
    <jsp:include page="chassis_general_body.jsp">
      <jsp:param name="chassisID" value="<%=nextSlot.getID()%>" />
      <jsp:param name="makeLinks" value="false" />
    </jsp:include>
<% if (!chassis.isErrorObject()) { %>
  <tr>
    <td colspan="2">

      <table class="item-list" width="100%">
      <thead>
        <tr>
          <td class="item-list-header-row">
            <i18n:message key="ComplexDetails.IOSlot" bundle="<%=parmgrTagResources%>" />
          </td>
          <td class="item-list-header-row">
            <i18n:message key="ComplexDetails.Description" bundle="<%=parmgrTagResources%>" />
          </td>
        </tr>
      </thead>
      <tbody>

<%

     for (Iterator iter = ioSlots.iterator(); iter.hasNext();)
     {
        IOSlot currentSlot = (IOSlot) iter.next();
        String currentDesc =
            parmgrTagResources.getString("IOSlot.empty.label");
        String currentSlotId = currentSlot.getLabel();

        if (currentSlot.isPopulated())  {
           IOCard currentCard = currentSlot.getCard();
           currentDesc = currentCard.getLabel();
        }
%>

        <tr>
          <td class="item-list-row-align-left">
            <%=currentSlotId%>
          </td>
          <td class="item-list-row-align-left">
            <%=currentDesc%>
          </td>
        </tr>

<%   } %>

      </tbody>
      </table>
    </td>
  </tr>
<% } // chassis is not an error object %>
<!-- end chassis N data -->
<% } // End while chassisSlotIter.hasNext %>

<%-- *********************************************************
**** Power / Cooling domains
**** **************************************************** --%>
<%
    ResourceBundle domainBundle = ResourceBundle.getBundle("com.hp.npar.mof." + Domain.OBJECT_PATH, parsession.getLocale());

    MessageFormat domainFormat = new MessageFormat(complexDetailsResources.getString("ComplexDetails.DomainProperties"));
    ListIterator domainIter = domains.listIterator();
    while( domainIter.hasNext() ) {
        Domain nextDomain = (Domain)domainIter.next();
        List devices = Arrays.asList(nextDomain.getDomainDevices());
        int cabID = nextDomain.getCabinetID();

        String domainSubType = domainBundle.getString(HP_NParPowerCoolingDomain.PROP_DomainSubtype + "_" + nextDomain.getSubType());
        String domainType = domainBundle.getString(HP_NParPowerCoolingDomain.PROP_DomainType + "_" + nextDomain.getDomainType());

        Object [] domainArgs = { domainSubType,
                                 domainType,
                                 Integer.toString(cabID) };
        String domainHeading = domainFormat.format(domainArgs);

%>
<!-- begin domain N data -->
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="partition-head"><%=domainHeading%></td>
  </tr>
  <tr>
    <td colspan="2">
      <hr class="separator">
    </td>
  </tr>
  <tr>
    <td colspan="2">
      <parmgr:domaindetailtable objects="<%=devices%>"
                                resources="<%=parmgrTagResources%>" />
    </td>
  </tr>
<!-- end domain N data -->
<% } // End while domainIter.hasNext %>

</table>

</BODY>
<% Log.summary("Finished displaying complex details page."); %>
