<%--
/**
 * The content for the body of the complex general tab and the
 * complex details page.
 *
 * @precondition The following style sheets must be loaded
 *               in the "head" section of the including jsp:
 *               .  /parmgr/css/parmgr.css
 *               .  /parmgr/css/hardware.css
 *               <p>
 *               The following java scripts must be loaded
 *               in the "head" section of the including jsp:
 *               .  /parmgr/js/parmgr.js
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/complex_general_body.jsp,v 1.14 2005/12/30 17:14:10 tmartin Exp $
 */
--%>

<%@ taglib uri="/com/hp/parmgr/ui/tags/parmgr" prefix="parmgr" %>
<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<%@ page import="java.util.List" %>
<%@ page import="java.util.ListIterator" %>
<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.DIMM" %>
<%@ page import="com.hp.parmgr.model.Cell" %>
<%@ page import="com.hp.parmgr.model.IOChassis" %>
<%@ page import="com.hp.parmgr.model.IOChassisSlot" %>
<%@ page import="com.hp.parmgr.model.Complex" %>
<%@ page import="com.hp.parmgr.model.Cabinet" %>
<%@ page import="com.hp.parmgr.model.ServiceProcessor" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>

<%@ page errorPage="exception.jsp" %>

<i18n:bundle baseName="com.hp.parmgr.ui.ComplexView" id="bundle" />
<i18n:bundle baseName="com.hp.parmgr.ui.Global" id="globalResources" />
<i18n:bundle baseName="com.hp.parmgr.ui.tags.tags" id="tagResources" />
<i18n:bundle baseName="com.hp.parmgr.ui.Arches" id="archesResources" />

<jsp:useBean id="parsession" scope="session" class="com.hp.parmgr.ui.ParmgrSession" />

<%
    Discovery discovery;
    discovery = parsession.getDiscovery();

    Complex complex = discovery.getComplex();
    int cpuCabinetsPresent = 0;
    int ioCabinetsPresent = 0;
    String activeServiceProcessorLocation = "none";

    /* Iterate through cabinets and find the active service processor and
     * types of cabinets.
     */
    List cabs = discovery.getCabinets();
    ListIterator iter = cabs.listIterator();
    Cabinet cab = null;

    while ( iter.hasNext() ) {

        Cabinet nextCab = (Cabinet)iter.next();
        ServiceProcessor [] serviceProcessors = nextCab.getServiceProcessors();

        if ( nextCab.getType() == Cabinet.IOEXPANSION ) {
            ioCabinetsPresent++;
        }
        else {
            if (cab == null){
                cab = nextCab;
            }
            cpuCabinetsPresent++;
        }

        for ( int i = 0; i < serviceProcessors.length; i++ ) {

            if ( serviceProcessors[i].getStatus() == ServiceProcessor.ACTIVE ) {
                activeServiceProcessorLocation = serviceProcessors[i].getLocation();
                break;
            }
        }
    }

    int numCells = 0;
    int numCPUs = 0;
    int numDIMMs = 0;
    float amountMem = 0;
    boolean missingMemoryInfo = false;
    boolean supportsGSM = false;

    List cells = discovery.getCells();
    ListIterator iterCell = cells.listIterator();

    numCells = cells.size();
    while ( iterCell.hasNext() ) {
        Cell nextCell = (Cell)iterCell.next();
        if (!nextCell.isErrorObject()) {
            if (nextCell.getStatus() < Cell.INACTIVE) {
                // We can't get memory information if the cell is
                // powered-off or in POST (all memory values will
                // be zero).
                missingMemoryInfo = true;
            }
            numCPUs += nextCell.getCPUCount();
            numDIMMs += nextCell.getInstalledDIMMsCount();
            amountMem += nextCell.getTotalMemoryInstalledInGB();

            if (nextCell.isGsmCapable()) {
                supportsGSM = true;
            }
        }
    }

    java.text.NumberFormat nf = DIMM.getMemoryNumberFormat(bundle.getLocale());
    String amountMemStr = nf.format(amountMem);

    int numChassis = 0;
    int numIOCards = 0;
    List chassisSlots = discovery.getChassisSlots();
    ListIterator iterChas = chassisSlots.listIterator();
    while (iterChas.hasNext()){
        IOChassisSlot nextChassisSlot = (IOChassisSlot)iterChas.next();
        if (nextChassisSlot.isPopulated()){
            numChassis++;
            IOChassis nextChassis = discovery.getChassis(nextChassisSlot);
            if (!nextChassis.isErrorObject()) {
                numIOCards += nextChassis.getPopulatedPCISlotCount();
            }
        }
    }

    String serialNumber = complex.getSerialNumber();

    /* get the type of cabinet so we can place the right graphic on the
       screen. */
    int cabType = cab.getType();
    String pictureURL;
    switch (cabType){
    case Cabinet.SUPERDOME:
    case Cabinet.DOMELITE:
        if (cab.isPairedCabinet()) {
            pictureURL = Util.URL_BASE_IMAGES + "/" +
                bundle.getString("cabinet.pic.sd64000");
        } else {
            pictureURL = Util.URL_BASE_IMAGES + "/" +
                bundle.getString("cabinet.pic.sd32000");
        }
        break;
    case Cabinet.KEYSTONE:
        pictureURL = Util.URL_BASE_IMAGES + "/" +
                bundle.getString("cabinet.pic.keystone");
        break;
    case Cabinet.MATTERHORN:
        pictureURL = Util.URL_BASE_IMAGES + "/" +
                bundle.getString("cabinet.pic.matterhorn");
        break;
    case Cabinet.UNKNOWN_CAB:
    case Cabinet.OTHER_CAB:
    default:
        pictureURL = Util.URL_BASE_IMAGES + "/" +
                bundle.getString("cabinet.pic.unknown");
    }

    String productName = complex.getProductName();

    if (productName == null) {
        // Older systems use model string
        productName = complex.getModelString();
    } else {
        // newer systems use OEMProductName or CreatorProductName
        String manufacturer = complex.getManufacturer();
        if (manufacturer != null) {
            productName =  manufacturer + " " + productName;
        }
    }
%>

  <tr>
    <td>
      <table class="properties" border=0 cellspacing=2 cellpadding=0>
        <tr>
        <td> <img src="<%=pictureURL%>"
              width="163"
              height="260"
              align="absmiddle"/>
        </td>
        </tr>
      </table>
    </td>
    <td>
    <table class="properties" border=0 cellspacing=2 cellpadding=0>
    <tr>
        <td colspan="2" class="prop-table-caption">
            <i18n:message key="generalTab.complex.generalProperties" bundle="<%=globalResources%>"/>
        </td>
    </tr>

    <tr>
        <td class="prop-table-header">
            <i18n:message key="field.ComplexName.label" bundle="<%=bundle%>" />
        </td>
        <td class="prop-table-data">
            <%=complex.getName()%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="field.Model.label" bundle="<%=bundle%>" />
        </td>
        <td class="prop-table-data">
            <%=productName%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="field.SerialNumber.label" bundle="<%=bundle%>" />
        </td>
        <td class="prop-table-data">
            <%=serialNumber%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="field.ProductNumberCurrent.label" bundle="<%=bundle%>" />
        </td>
        <td class="prop-table-data">
            <%=complex.getCurrentProductNumber()%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="field.ProductNumberOriginal.label" bundle="<%=bundle%>" />
        </td>
        <td class="prop-table-data">
            <%=complex.getOriginalProductNumber()%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="field.ComplexProfileRevision.label" bundle="<%=bundle%>" />
        </td>
        <td class="prop-table-data">
            <%=complex.getRevisionString()%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header-last">
            <i18n:message bundle="<%=bundle%>"
              key="field.ActiveServiceProcessorLocation.label"/>
        </td>
        <td class="prop-table-data-last">
              <%=activeServiceProcessorLocation%>
        </td>
    </tr>
    <tr>
        <td>&nbsp;</td>
    </tr>
    <tr>
        <td colspan="2" class="prop-table-caption">
            <i18n:message key="generalTab.summaryLabel" bundle="<%=globalResources%>"/>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="generalTab.summaryTable.numCabs" bundle="<%=globalResources%>"/>
        </td>
        <td align="right" class="prop-table-data">
            <%=cpuCabinetsPresent%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="generalTab.summaryTable.numIOCabs" bundle="<%=globalResources%>"/>
        </td>
        <td align="right" class="prop-table-data">
            <%=ioCabinetsPresent%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="generalTab.summaryTable.numNpars" bundle="<%=globalResources%>"/>
        </td>
        <td align="right" class="prop-table-data">
            <%=discovery.getPartitions().size()%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="generalTab.summaryTable.numCells" bundle="<%=globalResources%>"/>
        </td>
        <td align="right" class="prop-table-data">
            <%=numCells%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="generalTab.summaryTable.numCPUs" bundle="<%=globalResources%>"/>
        </td>
        <td align="right" class="prop-table-data">
            <%=numCPUs%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="generalTab.summaryTable.numDIMMs" bundle="<%=globalResources%>"/>
        </td>
        <td align="right" class="prop-table-data">
            <%=numDIMMs%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="generalTab.summaryTable.amountMem" bundle="<%=globalResources%>"/>
        </td>
        <td align="right" class="prop-table-data">
            <%=amountMemStr%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header">
            <i18n:message key="generalTab.summaryTable.numChassis" bundle="<%=globalResources%>"/>
        </td>
        <td align="right" class="prop-table-data">
            <%=numChassis%>
        </td>
    </tr>
    <tr>
        <td class="prop-table-header-last">
            <i18n:message key="generalTab.summaryTable.numIOCards" bundle="<%=globalResources%>"/>
        </td>
        <td align="right" class="prop-table-data-last">
            <%=numIOCards%>
        </td>
    </tr>

<%-- @TODO:  Un-comment for the official Montecito release
  <%  // only display if this nPar is GSM capable
      if (supportsGSM) {
  %>
    <tr>
        <td colspan="2">&nbsp;</td>
    </tr>
    <tr>
        <td colspan="2" class="prop-table-caption">
            <i18n:message key="complex.general.gsm.config.heading"
                          bundle="<%=archesResources%>"/></td>
    </tr>
    <tr>
        <td colspan="2" class="prop-table-data-last">
        <% if (complex.getGSMSharingType() == Complex.GSM_DISABLED) { %>
            <i18n:message key="complex.general.gsm.config.disabled"
                          bundle="<%=archesResources%>"/>
        <% } else if (complex.getGSMSharingType() == Complex.GSM_CPLX_WIDE) { %>
            <i18n:message key="complex.general.gsm.config.enabled"
                          bundle="<%=archesResources%>"/>
        <% } else { %>
          <parmgr:gsmConfigTable
              gsmConfigMatrix="<%=complex.getGSMSharing()%>"
              resources="<%=archesResources%>"
              discovery="<%=discovery%>" />
        <% } %>
        </td>
    </tr>
  <%
      } // if (supportsGSM)
  %>
--%>

<% if (missingMemoryInfo) {
    // Put this as the last row of the table, rather than outside
    // the table, so it is snugged right up against the bottom
    // of the last row.
%>
    <tr><td colspan=2 class=parmgr-small-text>
    <i18n:message key="General.missingMemoryInfo" bundle="<%=tagResources%>"/>
    </td></tr>
<% } %>

    </table>

    </td>
  </tr>
