<%--
/**
 * The JSP page that allows the user to configure Global Shared Memory for the current complex.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/configure_gsm.jsp,v 1.7 2008/03/05 12:48:29 sridharav Exp $
 * @risk Medium
 */
--%>
<%@ taglib uri="/com/hp/parmgr/ui/tags/parmgr" prefix="parmgr" %>
<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<%@ page errorPage="exception.jsp" %>

<%@ page import="java.util.List" %>
<%@ page import="java.util.ListIterator" %>

<%@ page import="java.text.MessageFormat" %>

<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.Complex" %>
<%@ page import="com.hp.parmgr.model.HWPartition" %>

<%@ page import="com.hp.parmgr.ui.UnauthorizedUserException" %>
<%@ page import="com.hp.parmgr.ui.SessionTimeOut" %>
<%@ page import="com.hp.parmgr.ui.DialogException" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>

<%@ page import="com.hp.parmgr.util.CommandFailedException" %>
<%@ page import="com.hp.parmgr.util.CommandIOException" %>
<%@ page import="com.hp.parmgr.util.ComplexModifyTask" %>
<%@ page import="com.hp.parmgr.util.WaitForProfile" %>
<%@ page import="com.hp.parmgr.util.Log" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<jsp:useBean id="parsession" scope="session"
    class="com.hp.parmgr.ui.ParmgrSession" />
<jsp:useBean id="gsmConfigBean" scope="session"
    class="com.hp.parmgr.util.GSMConfigBean"/>

<%-- contains properties/strings common to the app --%>
<i18n:bundle baseName="com.hp.parmgr.ui.Parmgr" id="bundleMain" />
<%-- contains new properties/strings added for Arches/Montecito support --%>
<i18n:bundle baseName="com.hp.parmgr.ui.Arches" id="bundleArches" />

<%
    Discovery discovery = parsession.getDiscovery();

    // Do this after getting discovery
    if (!parsession.userAuthorizedToMakeChanges("cplxmodify")) {
        throw new UnauthorizedUserException(parsession.getFromURL(), parsession.getLocale());
    }

    // URLs we might need from the session:
    String fromURL = parsession.getFromURL();
    String helpURL = parsession.getHelpURL("configGSM");

    // Strings to be displayed:
    String titleString 		= bundleArches.getString("gsm.config.title");
    String subTitle 		= bundleArches.getString("gsm.config.subtitle");
    String okButtonStr     	= bundleMain.getString("button.ok.str");
    String helpButtonStr   	= bundleMain.getString("button.help.str");
    String cancelButtonStr 	= bundleMain.getString("button.cancel.str");

    Object [] msgArgs = { discovery.getComplex().getName() };
    MessageFormat format =
        new MessageFormat(bundleMain.getString("complexName.fmt"));
    String complexName = format.format(msgArgs);

    // page control parameters
    boolean editOK        = (request.getParameter("EditOK") != null);
    boolean editCancel    = (request.getParameter("EditCancel") != null);
    boolean confirmOK     = (request.getParameter("ConfirmOK") != null);
    boolean confirmCancel = (request.getParameter("ConfirmCancel") != null);
    boolean changeSharing = (request.getParameter("ChangeSharing") != null);
    // current page state
    boolean firstPass = !(editOK || editCancel || confirmOK || confirmCancel ||
                          changeSharing);
    boolean editMode = !(editOK || editCancel || confirmOK) || confirmCancel ||
                         changeSharing;
    boolean confirmMode = editOK;
    boolean editCancelled = editCancel;
    boolean confirmCancelled = confirmCancel;
    boolean runCommand = confirmOK;

    // If this is the first time the user has come into this page
    // (from the menu, not from canceling out of the confirm dialog, etc.),
    // initialize the GSMConfigBean for this session:
    if (firstPass) {
        // This is the initial pass into this dialog (got here from the menu)

        // Initialize the GSM Config bean for this session with current data:
        gsmConfigBean.init(discovery.getComplex().getGSMSharingType(),
                           discovery.getComplex().getGSMSharing(),
                           parsession.getLoginCredentials());

        // Log that we are starting a new task:
        Log.summary(Log.EVENT_START, Log.RESULT_NONE,
            bundleArches.getString("gsm.config.start.log"));
    }
    else if (editMode || confirmMode) {
        // get a copy of the last used settings (to use as default settings)
        int sharingType = gsmConfigBean.getCurrGSMSharingType();
        boolean[][] sharingMatrix = gsmConfigBean.getCurrGSMSharingMatrix();
        // check for data updates in the request parameters
        boolean haveUpdates = false;
        String strConfigType = request.getParameter("GSMConfigType");
        if (strConfigType != null) {
            sharingType = Integer.parseInt(strConfigType);
            haveUpdates = true;
        }
        if (sharingType == Complex.GSM_PER_NPAR) {
            // check for per-npar control updates
            String[] parSelectedStrings = request.getParameterValues("nParSelect");
            if (parSelectedStrings != null) {
                // this means the "Change" button was clicked
                haveUpdates = true;
                int parIDSelected = Integer.parseInt(parSelectedStrings[0]);
                // first, clear out the array for the selected nPar
                for (int i = 0; i < sharingMatrix.length; ++i) {
                    // for non-existent nPars, leave the matrix as-is
                    // (otherwise, command options may be generated for
                    // non-existent nPars)
                    if (discovery.getPartition(i) == null) {
                        continue;
                    }
                    for (int j = 0; j < sharingMatrix[i].length; ++j) {
                        // for non-existent nPars, leave the matrix as-is
                        // (otherwise, command options may be generated for
                        // non-existent nPars)
                        if (discovery.getPartition(j) == null) {
                            continue;
                        }
                        if (i == j || i == parIDSelected || j == parIDSelected) {
                            sharingMatrix[i][j] = false;
                        }
                    }
                }
                // get the updated sharing array for the selected nPar
                String[] sharesWithStrings =
                    request.getParameterValues("sharesWithSelect");
                if (sharesWithStrings != null) {
                    for (int i = 0; i < sharesWithStrings.length; ++i) {
                        int parSharesWith = Integer.parseInt(sharesWithStrings[i]);
                        if (parSharesWith != parIDSelected) {
                            // turn on sharing between these 2 nPars
                            sharingMatrix[parIDSelected][parSharesWith] = true;
                            // (both directions)
                            sharingMatrix[parSharesWith][parIDSelected] = true;
                        }
                    }
                }
            }
        }
        if (haveUpdates) {
            // update our bean with the new info
            gsmConfigBean.setCurrGSM(sharingType, sharingMatrix);
        }
    }

%>

<HTML>
<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<%@ include file="session_keep_alive.jsp" %>
    <link type="text/css" rel="stylesheet" href="<%=Util.URL_PARMGR_CSS%>">
    <script type='text/javascript' language='JavaScript'
            src='<%=Util.URL_PARMGR_JS%>'></script>
<%=SessionTimeOut.javascript(session, bundleMain.getLocale())%>
    <title>
    <%=titleString%>
    </title>
</HEAD>

<% if (editCancelled) {   // *** USER CANCELED FROM EDIT MODE ***
    Log.summary(Log.EVENT_DONE, Log.RESULT_CANCELED,
        bundleArches.getString("gsm.config.end.cancel.log"));
%>
<BODY onLoad="window.location.replace('<%=fromURL%>');">
</BODY>

<% } else { %>
<BODY>
<script language="javascript">
function disableButtons() {
    var buttonOK  = document.getElementById("ConfirmOK");

    if (buttonOK != null) {
        var buttonCNX = document.getElementById("ConfirmCancel");
        var buttonHLP = document.getElementById("Help");

        buttonOK.disabled = true;
        buttonCNX.disabled = true;
        buttonHLP.disabled = true;
    }

    return true;
}
</script>

    <table width=100% border=0 cellspacing=0 cellpadding=2>
    <tr><td width="100%" align="left" nowrap bgcolor="#CCCCCC"
        style="font-size:14.0pt;font-family:Arial;color:black">
        <%=titleString%>
    </td></tr>
    <tr><td width="100%" align="left" nowrap bgcolor="#CCCCCC"
        style="font-size:8.0pt;font-family:Arial;color:black">
        <%=subTitle%>
    </td></tr>
    <tr><td width="100%" align="left" nowrap bgcolor="#CCCCCC"
        style="font-size:11.0pt;font-family:Arial;color:black">
        <%=complexName%>
    </td></tr>
    </table>

    <%
    ComplexModifyTask cplxModifyTask = null;
    String commandString = "";
    if (runCommand || confirmMode) {
        // First, get the absolute latest GSMSharing matrix from Discovery,
        // in case this has changed:
        discovery.clearComplexProfileCache();
        gsmConfigBean.setInitGSM(discovery.getComplex().getGSMSharingType(),
            discovery.getComplex().getGSMSharing());

        // Get the updated cplxmodify tas object from our bean:
        cplxModifyTask = gsmConfigBean.getComplexModifyTask();
        commandString = "<code>"+cplxModifyTask.getHTMLFormattedCommand()+"</code>";
    }
    if (runCommand) { // *** USER CLICKED "OK" ON CONFIRM DIALOG ***
        // Execute the command here
        boolean hasErr = true;  // assume the worst
        String errMsg = "";

        try {
            // execute the cplxmodify command:
            cplxModifyTask.run();

            // if we get past the last line of code, we have succeeded.
            hasErr = false;

            // clear the Discovery cache, so data will be refreshed
            discovery.clearComplexProfileCache();

            // Give the service processor a chance to push out the
            // pending profile.
            WaitForProfile.waitFixedTime(discovery);

            // Reset the check pending flag so that if the service
            // processor did not complete pushing out the pending
            // profile, we warn the user that it is still out there.
            parsession.setCheckPending(true);

            // log successful completion of the task
            Log.summary(Log.EVENT_DONE, Log.RESULT_SUCCESS,
                bundleArches.getString("gsm.config.end.success.log"));

        } catch (CommandIOException ioe) {
            Object [] ioeArgs = {ioe.getMessage()};
            MessageFormat ioeFormat =
                new MessageFormat(bundleMain.getString("error.ioError.format"));
            errMsg = ioeFormat.format(ioeArgs);
            hasErr = true;
        } catch (CommandFailedException ce) {
            errMsg = ce.getMessage();
            hasErr = true;
        }

        if (hasErr) {
            Object[] arg = { errMsg };
            String logString = new MessageFormat(
                bundleArches.getString("gsm.config.end.error.log.fmt")).format(arg);
            Log.summary(Log.EVENT_DONE, Log.RESULT_FAILURE, logString);

            DialogException de = new DialogException(errMsg, fromURL,
                                                     "com.hp.parmgr.ui.Arches");
            de.setLocale(parsession.getLocale());
            de.setTitle("gsm.config.end.error.title");
            de.setImage(Util.URL_BASE_IMAGES + "/status_critical_lg_lt.gif");
            de.setHelp("configureGSM");
            throw de;
        }
    %>
        <%-- Display success message here --%>
        <p align="left" class="subwindow-heading">
            <%=bundleArches.getString("gsm.config.end.success.title")%>
        </p>
        <p>
            <%=bundleArches.getString("gsm.config.end.success.msg")%>
        </p>
        <%-- Show the command that was run --%>
        <table width="100%">
            <tr>
                <%=bundleMain.getString("parmgr.commandTitle.str")%>
            </tr>
            <tr>
                <%=commandString%>
            </tr>
        </table>
    <% } else if (confirmMode) { // *** USER CLICKED "OK" ON THE EDIT DIALOG ***
    %>
        <%-- Display confirmation message --%>
        <p>
            <%=bundleArches.getString("gsm.config.confirm.modify.msg")%>
        </p>
        <%-- Show the command that will be run --%>
        <table width="100%">
            <tr>
                <%=bundleMain.getString("commandPreview")%>
            </tr>
            <trs>
                <%=commandString%>
            </tr>
        </table>
    <% } %>

    <%-- Form Starts Here --%>
    <form method="post" action="configure_gsm.jsp" name="GSMConfig">

    <% if (editMode) {   // *** DISPLAY GSM CONFIG EDIT DIALOG ***
        String typeCplxEnableTitle =
            bundleArches.getString("gsm.config.edit.type.cplxwide.enable.title");
        String typeCplxEnableNote =
            bundleArches.getString("gsm.config.edit.type.cplxwide.enable.note");
        String typeCplxDisableTitle =
            bundleArches.getString("gsm.config.edit.type.cplxwide.disable.title");
        String typeCplxDisableNote =
            bundleArches.getString("gsm.config.edit.type.cplxwide.disable.note");
        String typePerNparTitle =
            bundleArches.getString("gsm.config.edit.type.per.npartition.title");
        String typePerNParNote =
            bundleArches.getString("gsm.config.edit.type.per.npartition.note");

        String advHeading =
            bundleArches.getString("gsm.config.edit.advanced.heading");
        String advPromptNpartition =
            bundleArches.getString("gsm.config.edit.advanced.npartition");
        String advPromptSharesWith =
            bundleArches.getString("gsm.config.edit.advanced.shareswith");
        String advChangeButtonLabel =
            bundleArches.getString("gsm.config.edit.advanced.changebutton");
        String advTableNote =
            bundleArches.getString("gsm.config.edit.advanced.table.note");

        List partitions = discovery.getPartitions();

    %>
<script language="javascript">
function showOrHideTable(optButton) {
    if (optButton.value == "<%=Complex.GSM_PER_NPAR%>"){
        showObject("nParTable");
    }
    else{
        hideObject("nParTable");
    }
}
function selectSharesWithOption(value, selected, disabled) {
    var sharesWithSelect = document.getElementById("sharesWithSelect");
    var numPars = sharesWithSelect.length;

    // Step through the Shares With matrix & select/de-select the
    // option with the given value:
    for (var i = 0; i < numPars; i++) {
        if (sharesWithSelect.options[i].value == value) {
            // found the option -- set its selected state & bail
            sharesWithSelect.options[i].selected = selected;
            sharesWithSelect.options[i].disabled = disabled;
            break;
        }
    }
}
function onParSelectChange() {
    // first, disable the Change Sharing button until the user
    // changes something:
    var buttonChange = document.getElementById("ChangeSharing");
    buttonChange.disabled = true;

    // get the currently selected nPar ID:
    var nParSelect = document.getElementById("nParSelect");
    var selectedIdx = nParSelect.selectedIndex;
    if (selectedIdx == -1) {
        // no npar selected -- just return
        return;
    }
    var selectedID = nParSelect.options[selectedIdx].value;

 <% boolean[][] matrix = gsmConfigBean.getCurrGSMSharingMatrix();
    int maxPars = matrix.length;
    for (int i = 0; i < maxPars; ++i) { %>
        if (<%=i%> == selectedID) {
         <% for (int j = 0; j < maxPars; ++j) {
                 if (j == i) { %>
                     // this is the same as the "selected" nPar
                     // show this option selected & disabled:
                     selectSharesWithOption(<%=j%>, true, true);
             <%  } else if (matrix[i][j]) { %>
                    // Select the Shares With option whose value==j
                    selectSharesWithOption(<%=j%>, true, false);
             <% } else { %>
                    // De-select the Shares With option whose value==j
                    selectSharesWithOption(<%=j%>, false, false);
             <% }
            }
// can't do this, unfortunately (doesn't make sense in the generated script)
//            break;
//              or
//            return;
            %>
        }
 <% } %>
}
function onSharesWithSelectChange() {
    // enable the Change Sharing button, now that the user
    // has changed something:
    var buttonChange = document.getElementById("ChangeSharing");
    buttonChange.disabled = false;

    // ensure the currently selected npar always remains selected
    // in the sharesWith list (npars "always" share with themselves ;)
    var nParSelect = document.getElementById("nParSelect");
    var selectedIdx = nParSelect.selectedIndex;
    if (selectedIdx != -1) {
        var selectedID = nParSelect.options[selectedIdx].value;
        selectSharesWithOption(selectedID, true, true);
    }
}
</script>
        <%-- GSM Sharing Type radio buttons --%>
        <table border="0" width="100%" id="GSMTypeTable">
            <tr style="font-size:11.0pt;font-family:Arial;color:black">
                <td colspan=2>
                <input type="radio" name="GSMConfigType"
                    value="<%=Complex.GSM_DISABLED%>"
                    onclick="showOrHideTable(this)"
                    <%if(gsmConfigBean.getCurrGSMSharingType() ==
                         Complex.GSM_DISABLED){%>checked<%}%>/>
                <%=typeCplxDisableTitle%></td>
            </tr>
            <tr style="font-size:8.0pt;font-family:Arial;color:black">
                <td width="20px">&nbsp;</td>
                <td><%=typeCplxDisableNote%></td>
            </tr>
            <tr style="font-size:11.0pt;font-family:Arial;color:black">
                <td colspan=2>
                <input type="radio" name="GSMConfigType"
                    value="<%=Complex.GSM_CPLX_WIDE%>"
                    onclick="showOrHideTable(this)"
                    <%if(gsmConfigBean.getCurrGSMSharingType() ==
                         Complex.GSM_CPLX_WIDE){%>checked<%}%>/>
                <%=typeCplxEnableTitle%></td>
            </tr>
            <tr style="font-size:8.0pt;font-family:Arial;color:black">
                <td width="20px">&nbsp;</td>
                <td><%=typeCplxEnableNote%></td>
            </tr>
            <tr style="font-size:11.0pt;font-family:Arial;color:black">
                <td colspan=2>
                <input type="radio" name="GSMConfigType"
                    value="<%=Complex.GSM_PER_NPAR%>"
                    onclick="showOrHideTable(this)"
                    <%if(gsmConfigBean.getCurrGSMSharingType() ==
                         Complex.GSM_PER_NPAR){%>checked<%}%>/>
                <%=typePerNparTitle%></td>
            </tr>
            <tr style="font-size:8.0pt;font-family:Arial;color:black">
                <td width="20px">&nbsp;</td>
                <td><%=typePerNParNote%></td>
            </tr>
            <tr style="font-size:11.0pt;font-family:Arial;color:black">
                <td width="20px">&nbsp;</td>
                <td><div id="nParTable"
                       class="<%=(gsmConfigBean.getCurrGSMSharingType() ==
                                 Complex.GSM_PER_NPAR)? "show" : "hide"%>" >
                <%-- Per nPar config (advanced section) goes here --%>
                <table style="font-size:11.0pt;font-family:Arial;color:black">
                    <tr>
                        <td colspan=5>
                            <%=advHeading%>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <%=advPromptNpartition%>
                        </td>
                        <td>
                            <%-- Select nPar List Box --%>
                            <select id="nParSelect" name="nParSelect"
                                    size="<%=partitions.size()%>"
                                    onchange="onParSelectChange()">
                                <%
                                ListIterator iter = partitions.listIterator();
                                while(iter.hasNext()) {
                                    HWPartition npar = (HWPartition)iter.next();
                                %>
                                    <option value="<%=npar.getID()%>">
                                            <%=npar.getLongLabel()%>
                                    </option>
                                <% } %>
                            </select>
                        </td>
                        <td>
                            <%=advPromptSharesWith%>
                        </td>
                        <td>
                            <%-- Shares With List Box --%>
                            <select id="sharesWithSelect" name="sharesWithSelect"
                                    size="<%=partitions.size()%>" multiple
                                    onchange="onSharesWithSelectChange()">
                                <%
                                ListIterator iter2 = partitions.listIterator();
                                while(iter2.hasNext()) {
                                    HWPartition npar = (HWPartition)iter2.next();
                                %>
                                    <option value="<%=npar.getID()%>">
                                            <%=npar.getLongLabel()%>
                                    </option>
                                <% } %>
                            </select>
                        </td>
                        <td>
                            <input class="button" type="submit"
                               value="<%=advChangeButtonLabel%>"
                               name="ChangeSharing" />
                        </td>
                    </tr>
                    <tr style="font-size:8.0pt;font-family:Arial;color:black">
                        <td colspan=5>
                            <%=advTableNote%>
                        </td>
                    </tr>
                </table>
                <parmgr:gsmConfigTable
                    gsmConfigMatrix="<%=gsmConfigBean.getCurrGSMSharingMatrix()%>"
                    resources="<%=bundleArches%>"
                    discovery="<%=discovery%>" />
                </div></td>
            </tr>
        </table>

    <% } %>

    <%-- Button Row --%>
    <p align="left">
    <hr class="separator">

    <% if (editMode) {            // Edit Mode Buttons %>
    <input class="button"               type="submit"
           value="<%=okButtonStr%>"     name="EditOK" />
    <input class="button"               type="submit"
           value="<%=cancelButtonStr%>" name="EditCancel" />

    <% } else if (confirmMode) {  // Confirm Mode Buttons %>
    <input class="button"               type="submit"
           value="<%=okButtonStr%>"     name="ConfirmOK" />
    <input class="button"               type="submit"
           value="<%=cancelButtonStr%>" name="ConfirmCancel" />

    <% } else {                   // Command Result Buttons %>
    <input class="button"               type="button"
           value="<%=okButtonStr%>"     name="ResultOK"
           onClick="window.location.replace('<%=fromURL%>')" />
    <% }                          // Common Buttons %>
    <input class="button"               type="button"
           value="<%=helpButtonStr%>"   name="Help"
           onclick="showHelp('<%=helpURL%>')" />
    </p>

</form>
</BODY>
<% } %>

</HTML>
