<%--
/**
 * The JSP page for deleting partitions.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/delete_npar.jsp,v 1.37 2008/03/05 12:48:29 sridharav Exp $
 * @risk Medium
 *
 *  @task name = Implement delete nPar UI
 *       cycle = 5
 *       phase = TEST
 *       owner = Terry
 *      status = Done
 * description = Create a JSP which validates, confirms, and executes the delete nPar task.
 */
--%>
<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<%@ page errorPage="exception.jsp" %>

<%@ page import="java.lang.Integer" %>
<%@ page import="java.text.MessageFormat" %>
<%@ page import="java.util.TreeSet" %>
<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.HWPartition" %>
<%@ page import="com.hp.parmgr.model.NParComponent" %>
<%@ page import="com.hp.parmgr.ui.SelectionManager" %>
<%@ page import="com.hp.parmgr.ui.UnauthorizedUserException" %>
<%@ page import="com.hp.parmgr.ui.SessionTimeOut" %>
<%@ page import="com.hp.parmgr.ui.DialogException" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page import="com.hp.parmgr.util.HWPartitionRemoveTask" %>
<%@ page import="com.hp.parmgr.util.CommandFailedException" %>
<%@ page import="com.hp.parmgr.util.CommandIOException" %>
<%@ page import="com.hp.parmgr.util.Menu" %>
<%@ page import="com.hp.parmgr.util.Log" %>
<%@ page import="com.hp.parmgr.util.WaitForProfile" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<jsp:useBean id="parsession" scope="session"
             class="com.hp.parmgr.ui.ParmgrSession" />

<i18n:bundle baseName="com.hp.parmgr.ui.Parmgr" id="bundle" />

<%
    Discovery discovery = parsession.getDiscovery();

    // Do this after getting discovery
    if (!parsession.userAuthorizedToMakeChanges("parremove")) {
        throw new UnauthorizedUserException(parsession.getFromURL(), parsession.getLocale());
    }

    // actual strings to be displayed (in this order), if not null
    String titleString = bundle.getString("npar.delete.title");
    String message = null;
    String displayCommand = null;

    String okButtonStr     = bundle.getString("button.ok.str");
    String helpButtonStr   = bundle.getString("button.help.str");
    String cancelButtonStr = bundle.getString("button.cancel.str");

    // page control parameters
    boolean cancelled  = (request.getParameter("Cancel") != null);
    boolean runCommand = (request.getParameter("Ok") != null);

    // confirm == true --> display OK and cancel to confirm that the
    // user really wants to delete the partition.
    boolean confirm      = false;

    // hasErr == true --> invalid partition or a command failed.
    // display errMsg.
    boolean hasErr       = false;
    String errMsg        = null;

    String fromURL = parsession.getFromURL();
    String helpURL = parsession.getHelpURL("deleteNPar");


    // Get the ID of the partition to delete and handle any
    // error messages if no partition is selected or the
    // partition is invalid or does not exist.
    SelectionManager selectedCheckboxes = parsession.getSelectedCheckboxes();
    TreeSet parIDs = selectedCheckboxes.getSelectedPartitions();
    int implicitPar = parsession.getImplicitPartitionID();

    int parID = Menu.INVALID_PARTITION_ID;
    if (parIDs.size() == 0) {
        parID = parsession.getImplicitPartitionID();
    } else {
        // should only be 1 partition selected:
        // (assert() currently not suported in Tomcat?)
        // assert(parIDs.size() == 1);
        parID = ((Integer) parIDs.first()).intValue();
    }

    // get the partition object & attributes
    HWPartition partition = discovery.getPartition(parID);

    if (partition == null || !discovery.partitionExists(parID)) {
        // non-existent partition -- maybe the user typed in a bogus url?
        Object[] parIDArg = { new Integer(parID) };
        errMsg = new MessageFormat(
            bundle.getString("error.noSuchPartition.fmt")).format(parIDArg);
        hasErr = true;
        // selection manager may be messed up - reset it
        selectedCheckboxes.deselectAllCheckboxes();

    } else if (!cancelled) {

        // have a valid partition - proceed

        boolean isActivePar =
            (discovery.getPartitionStatus(parID) == NParComponent.ACTIVE);
        boolean isLocalPar = partition.isLocalPartition();
        boolean isInterparSecEnabled = discovery.getComplex().isInterpartitionSecurityEnabled();
        boolean isRemoteComplex = discovery.isRemoteComplex();

        // format a pretty partition name
        String formattedPartitionName;
        if (partition.isErrorObject()) {
            formattedPartitionName = "nPar"+partition.getID();
        } else {
            formattedPartitionName = partition.getName()+" (nPar"+partition.getID()+")";
        }
        Object[] parNameArg = { formattedPartitionName };

        HWPartitionRemoveTask deleteParTask =
            new HWPartitionRemoveTask(parID, true, parsession.getLoginCredentials());

        String shutdownCommandString = "<code>/etc/shutdown -RH</code>" +
                            "<br>" + "&nbsp;&nbsp;&nbsp;&nbsp;" +
                            bundle.getString("or.string") + "<br>" +
                            "<code>/etc/reboot -RH</code>";
        String commandString = "<code>"+deleteParTask.getCommand()+"</code>";

        if (!runCommand) {
            // first time through

            discovery.clearCache();
            isActivePar =
                (discovery.getPartitionStatus(parID) == NParComponent.ACTIVE);

            // First, check to see if a pending profile currently exists.
            // Raise the checkPending flag to ensure that Discovery is
            // accessed.
            parsession.setCheckPending(true);
%>

<%@ include file="check_pending_profile.jsp" %>

<%
            String logString = new MessageFormat(
                    bundle.getString("npar.delete.start.log.fmt")).format(parNameArg);
            Log.summary(Log.EVENT_START, Log.RESULT_NONE, logString);

            // validate that the selected partition is not active
            if (isActivePar && !isLocalPar && !isRemoteComplex) {

                // log the unsuccessful completion of the task
                Log.summary(Log.EVENT_DONE, Log.RESULT_NONE,
                        bundle.getString("npar.delete.abort.active.log"));

                // format "cannot delete an active partition" error message
                StringBuffer buffer = new StringBuffer();
                buffer.append("<p>");
                buffer.append(bundle.getString("npar.delete.error.active.desc"));
                buffer.append("</p><p>");
                buffer.append(new MessageFormat(
                    bundle.getString("npar.delete.error.active.instr")).format(parNameArg));
                buffer.append("</p><p>");

                // A numbered list of steps
                buffer.append("<ol>");
                buffer.append("<li>"+bundle.getString("shutdown.step.one.str")+"</li>");
                buffer.append("<li>"+bundle.getString("shutdown.step.two.str"));
                // Different instructions for hp-ux and windows.
                buffer.append("<dl><dt><i>HP-UX</i></dt>");
                buffer.append("<dd>");
                Object[] args = { "<code>-RH</code>",
                                  "<code>shutdown(1M)</code>",
                                  "<code>reboot(1M)</code>" };
                buffer.append(new MessageFormat(
                    bundle.getString("shutdown.step.two.hpux.fmt")).format(args));

                // windows specific instructions
                buffer.append("</dd><dt><i>Windows</i></dt>");
                buffer.append("<dd>");
                buffer.append(bundle.getString("shutdown.step.two.windows"));

                // linux specific instructions
                buffer.append("</dd><dt><i>Linux</i></dt>");
                buffer.append("<dd>");
                Object[] argsLin = { "<code>-r</code>",
                                     "<code>-h</code>",
                                     "<code>shutdown(8)</code>" };
                buffer.append(new MessageFormat(
                    bundle.getString("shutdown.step.two.linux.fmt")).format(argsLin));
                buffer.append("</dd></dl></li></ol>");

                // A final note
                buffer.append("</p><p>");
                buffer.append(bundle.getString("npar.delete.error.active.note"));
                buffer.append("</p>");

                DialogException de = new DialogException(buffer.toString(), fromURL,
                                                        "com.hp.parmgr.ui.Parmgr");
                de.setLocale(parsession.getLocale());
                de.setTitle("npar.delete.title");
                de.setImage(Util.URL_BASE_IMAGES + "/status_critical_lg_lt.gif");
                de.setHelp("deleteNPar");
                throw de;

            // InterpartitionSecurity check
            } else if (isInterparSecEnabled && !isLocalPar && !isRemoteComplex) {

                // log the unsuccessful completion of the task
                Log.summary(Log.EVENT_DONE, Log.RESULT_NONE,
                        bundle.getString("npar.delete.abort.sec.log"));

                // format "cannot delete, due to Interpartition Security" error message
                StringBuffer buffer = new StringBuffer();
                buffer.append("<p>");
                buffer.append(bundle.getString("error.ips.desc"));
                buffer.append("</p><p>");
                buffer.append(new MessageFormat(
                    bundle.getString("npar.delete.error.ips")).format(parNameArg));
                buffer.append("</p><p>");
                buffer.append(bundle.getString("error.ips.note"));
                buffer.append("</p>");

                DialogException de = new DialogException(buffer.toString(), fromURL,
                                                        "com.hp.parmgr.ui.Parmgr");
                de.setLocale(parsession.getLocale());
                de.setTitle("npar.delete.title");
                de.setImage(Util.URL_BASE_IMAGES + "/status_critical_lg_lt.gif");
                de.setHelp("interParSecurity");
                throw de;

            } else {
                // The user can delete the partition

                // display confirmation message

                StringBuffer buffer = new StringBuffer();

                // If this partition is active (and we're allowing it to be deleted), warn the user
                buffer.append("<p>");
                if (isActivePar) {
                    buffer.append(bundle.getString("npar.delete.warning.activepar"));
                }
                // If this is the last partition in the complex, warn the user
                if (discovery.getPartitions().size() == 1) {
                    if (isActivePar) {
                        // Appending to previous warning
                        buffer.append("<br>");
                    }
                    buffer.append(bundle.getString("npar.delete.warning.lastpar"));
                }
                buffer.append("</p>");

                buffer.append("<p>");
                buffer.append(new MessageFormat(
                    bundle.getString("npar.delete.confirm.fmt")).format(parNameArg));
                buffer.append("</p>");

                message = buffer.toString();
                displayCommand = commandString;
                confirm = true;
            }

        } else {
            // 2nd pass: the user has hit OK -- run the command

            try {

                // execute the parremove command:
                deleteParTask.run();

                // if we get past the last line of code, we have succeeded.
                hasErr = false;

                // Reset the selection manager
                selectedCheckboxes.deselectAllCheckboxes();

                // clear the Discovery cache, so data will be refreshed
                discovery.clearCache();

                // Give the service processor a chance to push out the
                // pending profile.
                WaitForProfile.waitFixedTime(discovery);

                // Reset the check pending flag so that if the service
                // processor did not complete pushing out the pending
                // profile, we warn the user that it is still out there.
                parsession.setCheckPending(true);

                // log successful completion of the task
                String logString = new MessageFormat(
                    bundle.getString("npar.delete.end.success.log.fmt")).format(parNameArg);
                Log.summary(Log.EVENT_DONE, Log.RESULT_SUCCESS, logString);

                // format the success message
                StringBuffer buffer = new StringBuffer();
                buffer.append("<p>");
                if (isActivePar) {
                    buffer.append(new MessageFormat(bundle.getString("npar.delete.success.active")).format(parNameArg));
                    buffer.append("<p>");

                    // display "need to reboot for changes to take effect" msg
                    buffer.append(bundle.getString("shutdown.description.str"));
                    buffer.append("</p><p>");

                    // A numbered list of steps
                    buffer.append("<ol>");
                    buffer.append("<li>"+bundle.getString("shutdown.step.one.str")+"</li>");
                    buffer.append("<li>"+bundle.getString("shutdown.step.two.str"));
                    // Different instructions for hp-ux and windows.
                    buffer.append("<dl><dt><i>HP-UX</i></dt>");
                    buffer.append("<dd>");
                    Object[] args = { "<code>-RH</code>",
                                      "<code>shutdown(1M)</code>",
                                      "<code>reboot(1M)</code>" };
                    buffer.append(new MessageFormat(
                        bundle.getString("shutdown.step.two.hpux.fmt")).format(args));

                    // windows specific instructions
                    buffer.append("</dd><dt><i>Windows</i></dt>");
                    buffer.append("<dd>");
                    buffer.append(bundle.getString("shutdown.step.two.windows"));
                    buffer.append("</dd></dl></li>");

                    // linux specific instructions
                    buffer.append("</dd><dt><i>Linux</i></dt>");
                    buffer.append("<dd>");
                    Object[] argsLin = { "<code>-r</code>",
                                         "<code>-h</code>",
                                         "<code>shutdown(8)</code>" };
                    buffer.append(new MessageFormat(
                        bundle.getString("shutdown.step.two.linux.fmt")).format(argsLin));
                    buffer.append("</dd></dl></li></ol>");

                    buffer.append("<p>");
                    buffer.append(bundle.getString("shutdown.postscript.str"));
                    // No </p> here - it is below

                    // log the shutdown/reboot warning
                    Log.warning(bundle.getString("npar.delete.warn.active.log")+
                                bundle.getString("shutdown.description.str"));
                    // Also, log the shutdown/reboot steps as details
                    Log.detail(bundle.getString("shutdown.steps.log"));

                } else {

                    buffer.append(new MessageFormat(
                        bundle.getString("npar.delete.success")).format(parNameArg));
                }
                buffer.append("</p>");

                message = buffer.toString();

            } catch (CommandIOException ioe) {
                Object [] ioeArgs = {ioe.getMessage()};
                MessageFormat ioeFormat =
                    new MessageFormat(bundle.getString("error.ioError.format"));
                errMsg = ioeFormat.format(ioeArgs);
                hasErr = true;
            } catch (CommandFailedException ce) {
                errMsg = ce.getMessage();
                hasErr = true;
            }
        }
    }

    if (hasErr) {
        Object[] arg = { errMsg };
        String logString = new MessageFormat(
                    bundle.getString("npar.delete.end.failure.log.fmt")).format(arg);
        Log.summary(Log.EVENT_DONE, Log.RESULT_FAILURE, logString);

        DialogException de = new DialogException(errMsg, fromURL,
                                                "com.hp.parmgr.ui.Parmgr");
        de.setLocale(parsession.getLocale());
        de.setTitle("error.operationFailed.title.str");
        de.setImage(Util.URL_BASE_IMAGES + "/status_critical_lg_lt.gif");
        de.setHelp("deleteNPar");
        throw de;
    }
%>

<HTML>

<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<%@ include file="session_keep_alive.jsp" %>
    <link type="text/css" rel="stylesheet" href="<%=Util.URL_PARMGR_CSS%>">
    <script type='text/javascript' language='JavaScript'
            src='<%=Util.URL_PARMGR_JS%>'></script>
<%=SessionTimeOut.javascript(session, bundle.getLocale())%>
    <title>
        <%=titleString%>
    </title>
</HEAD>

<% if (cancelled) {
        Log.summary(Log.EVENT_DONE, Log.RESULT_CANCELED,
                bundle.getString("npar.delete.end.cancel.log"));
%>
<BODY onLoad="window.location.replace('<%=fromURL%>');">
</BODY>

<% } else { %>

<BODY>

    <p align="left" class="subwindow-heading">
        <%=titleString%>
    </p>

<%
    Object [] msgArgs = { discovery.getComplex().getName() };
    MessageFormat format = new MessageFormat(bundle.getString("complexName.fmt"));
    String complexName = format.format(msgArgs);
%>

    <p>
    <%=complexName%>
    </p>

    <form method="post" action="delete_npar.jsp" name="nParDelete" >

    <p><%=message%></p>
<% } %>

    <%-- button row --%>
    <p align="left">
<% if (confirm) { %>
        <INPUT class="button"               type="submit"
               value="<%=okButtonStr%>"    name="Ok" />

        <INPUT class="button"               type="submit"
               value="<%=cancelButtonStr%>"     name="Cancel" />

<% } else { %>

        <INPUT class="button"               type="button"
               value="<%=okButtonStr%>"     name="Ok"
               onClick="window.location.replace('complex.jsp');" />
<% } %>
        <INPUT class="button"               type="button"
               value="<%=helpButtonStr%>"   name="Help"
               onclick="showHelp('<%=helpURL%>')" />
    </p>

    </form>
<% if (confirm) { %>
    <p align="left">
    <%=bundle.getString("commandPreview")%>
        <br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
        [<%=okButtonStr%>]:&nbsp;<code><%=displayCommand%></code>
    </p>
<% } %>
</BODY>

</HTML>
