<%--
/**
 * This page handles exceptions that aren't specifically caught by other java code
 * in parmgr.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/exception.jsp,v 1.36 2008/08/22 13:48:58 sridharav Exp $
 * @risk: Low
 *
 * @task name = Create exception handling page
 *       cycle = 5
 *       phase = TEST
 *       owner = Mike
 *       status = Done
 *       description = Create the page which handles exceptions which make it back to tomcat
 *
 * @task name = Create standard exception handling dialog page
 *       cycle = 7
 *       phase = CODE
 *       owner = Nancy
 *       status = In Progress
 *       description = Catch exceptions and create an appropriate page for
 *                     each complete with OK button, help if appropriate,
 *                     and appropriate text and return URL.
 */
--%>

<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>
<link type='text/css' rel='stylesheet' href='<%=Util.URL_PARMGR_CSS%>'>
<jsp:useBean id="parsession" scope="session" class="com.hp.parmgr.ui.ParmgrSession" />
<i18n:bundle baseName="com.hp.parmgr.ui.tags.tags" id="parmgrTagResources" />
<i18n:bundle baseName="com.hp.parmgr.ui.Parmgr" id="providerErrorResources" />


<%@ page isErrorPage="true" %>
<%@ page import="com.hp.parmgr.model.LockedObjectError" %>
<%@ page import="com.hp.parmgr.util.ParmgrException" %>
<%@ page import="com.hp.parmgr.util.ParmgrError" %>
<%@ page import="com.hp.parmgr.util.ParmgrThrowable" %>
<%@ page import="com.hp.parmgr.util.StringFormatUtil" %>
<%@ page import="com.hp.parmgr.model.DiscoveryError" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page import="com.hp.npar.error.NParErrorUtil" %>
<%@ page import="com.hp.npar.error.NParErrors" %>
<%@ page import="java.text.MessageFormat" %>
<%@ page import="com.hp.parmgr.util.Log" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<%
// Set some defaults in case something comes through that we don't
// specifically handle here.
String title = parmgrTagResources.getString("ErrorMsg.InternalError");
String image = Util.URL_BASE_IMAGES + "/status_critical_lg_lt.gif";
String errorMessage = exception.toString();
String errorMessageLog = exception.toString();

String url = parsession.getFromURL();
if (request.getQueryString() != null) {
    String joinQueryChar = "?";
    if (url.matches(".*[?].*")) joinQueryChar = "&";
    url = url + joinQueryChar + request.getQueryString();
}
// JAGaf88550: guard against cross site scripting vulnerabilities,
// by encoding parameters that may be output to the HTML stream:
url = StringFormatUtil.encodeHtmlEntities(url);

String helpPage = null;
boolean showDetails = true;
boolean displayOK = true;
Throwable realReason = null;
int errorCode = 0;
boolean logError = true;

// First handle the exceptions that are exceptions (those that we don't
// handle in a generic way).
if (exception instanceof LockedObjectError) {
    // Send the user to the break lock JSP.  (Don't display a dialog.)
    session.setAttribute("lockError", exception);
    session.setAttribute("fromURL", url);
    // Leave this page without continuing.
    pageContext.forward("break_lock.jsp");
    return;

} else if (exception instanceof ParmgrThrowable) {

    ParmgrThrowable pmt = (ParmgrThrowable)exception;

    pmt.setLocale(parsession.getLocale());

    title = pmt.getTitle();
    image = pmt.getImage();
    errorMessage = pmt.getDisplayMessage();
    errorMessageLog = pmt.getMessage();
    showDetails = pmt.displayDetails();
    displayOK = pmt.displayOKButton();
    helpPage = pmt.getHelp();
    url = pmt.getGotoURL(url);
    logError = pmt.getLogError();

    // There are a couple of special things for discovery errors
    // since they can involve user authentication.
    if (pmt instanceof DiscoveryError) {
        parsession.disconnect();
        DiscoveryError de = (DiscoveryError) pmt;
        Object tryDB = session.getAttribute("parmgrTryingCredDB");
        if (de.isAccessDeniedError() &&
            tryDB != null &&
            tryDB.equals((Object)parsession.getWBEMHost()))
        {
            StringBuffer fwURL = request.getRequestURL();
            int index = fwURL.lastIndexOf("/");
            fwURL.setLength(index+1);
            fwURL.append(url);

            String retryURL = de.getRetryURL(fwURL.toString());

            // Leave this page without continuing.
            pageContext.forward(retryURL.toString());
            return;
        }
        realReason = de.getThrowable();

        errorCode = NParErrorUtil.parseErrorCode(errorMessage);
        if (errorCode == NParErrors.TIME_OUT ||
            errorCode == NParErrors.PLATFORM_NOT_SUPPORTED ||
            errorCode == NParErrors.INCOMPATIBLE_IPMI_VERSION ||
            errorCode == NParErrors.LAN_ERROR )
        {
            String hostname = parsession.getIPMIHost();
            if (hostname == null)
                hostname = parsession.getWBEMHost();
            Object[] args = { new String(hostname) };
            try {
                   errorMessage = MessageFormat.format(providerErrorResources.getString("provider.npar.error." + errorCode), args);
                   errorMessageLog = MessageFormat.format(providerErrorResources.getString("provider.npar.error.log." + errorCode), args);
            }
            catch(java.util.MissingResourceException ex) {
            }
        }
        else if (errorCode > 0)
        {
            errorMessage = NParErrorUtil.getString(errorCode);
            // @TODO Don't know what this returns or whether it is suitable for a log file.
            errorMessageLog = errorMessage;
        }

        //
        // Display a little better message for access denied to root.
        // The reason could be because the CIM server doesn't allow
        // remote root access.
        //
        if (de.isAccessDeniedError() &&
            "root".equals(parsession.getWBEMUsername()))
        {
            // Set the Help URL to this page id:
            // <a href=/mxhelp/parmgr/en/starting.html#cimserver target="_popup">configuring the CIM server</a> 
			// ?? parsession.getHelpURL(helpPage);

            errorMessage = parmgrTagResources.getString("ErrorMsg.InvalidUserOrPwdForRoot");
            errorMessageLog = "The WBEM password is invalid, or WBEM on the remote host is not configured to allow remote privileged user access.<p>See the online help regarding SSL Certificates for details on configuring remote privileged user access for WBEM.";
        }

    }
}
%>

<html>
<head>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<META HTTP-EQUIV="expires" CONTENT="0">
<%@ include file="session_keep_alive.jsp" %>
<link type="text/css" rel="stylesheet" href="<%=Util.URL_PARMGR_CSS%>" />

    <title><%=title%></title>

    <script type='text/javascript' language='JavaScript' src='<%=Util.URL_PARMGR_JS%>'></script>

</head>

<body>

<p align="left" class="subwindow-heading">
<% if (image == null) { %>
<font size="+2"><b><%=title%></b></font>
<% } else { %>
<img src="<%=image%>"> <font size="+2"><b><%=title%></b></font>
<% } %>
</p>

<p align="left">
<%=errorMessage%>
</p>

<% if (logError) {
       Log.error(errorMessageLog);
   } %>

<%  if (displayOK || helpPage != null || showDetails) { %>

<form method="get" action="<%=url%>" name="exception_form">
    <p align="left">
<%  if (displayOK) { %>
    <input class="button" type="submit"
           value="<%=parmgrTagResources.getString("Button.OK")%>"
           name="OK" />
<%  } %>

<%  if (helpPage != null) { %>
    <input class="button" type="button"
           value="<%=parmgrTagResources.getString("Button.Help")%>"
           name="Help"
           onClick="showHelp('<%=parsession.getHelpURL(helpPage)%>')" />
<%  } %>
</p>
<%  if (!showDetails) { %>

</form>

<%  } else {

        String showDetailsStr =
            parmgrTagResources.getString("ErrorMsg.ShowDetails") + " >>";
        String hideDetailsStr =
            parmgrTagResources.getString("ErrorMsg.HideDetails") + " <<";

        // Get the stack trace in string form.
        java.io.StringWriter writer = new java.io.StringWriter();
        java.io.PrintWriter test = new java.io.PrintWriter(writer);
        exception.printStackTrace(test);
        String stackTrace = writer.toString();
        if (logError) {
            Log.error(stackTrace);
        }
%>
<p align="left">
    <div id="show_details">
        <input class="button" type="button" value="<%=showDetailsStr%>"
               onclick='hideObject("show_details"); showObject("hide_details"); showObject("exceptiondetails")' />
    </div>
    <div class="hide" id="hide_details">
        <input class="button" type="button" value="<%=hideDetailsStr%>"
               onclick='hideObject("hide_details"); showObject("show_details"); hideObject("exceptiondetails")'/>
    </div>
    </p>
</form>

<div class="hide" id="exceptiondetails">

    <pre>
<%
    java.io.PrintWriter wout = new java.io.PrintWriter(out);
    exception.printStackTrace(wout);
%>
    </pre>

<%  if (realReason != null) {

                // Get the stack trace in string form.
        writer = new java.io.StringWriter();
        test = new java.io.PrintWriter(writer);
        realReason.printStackTrace(test);
        stackTrace = writer.toString();
        if (logError) {
            Log.error(stackTrace);
        }
%>
        <b>Root cause:</b><br>
        <pre>
<%      realReason.printStackTrace(wout); %>
        </pre>
<%  } %>

</div>

<%  }  // show details %>
<%  }  // if any button %>

</body>
</html>

