<%--
/**
 * The content for the body of the partition general tab and the
 * complex details page.
 *
 * @precondition A parameter named "partition" must be set to
 *               the id of the partition.
 *               <p>
 *               The boolean variable "makeLinks" must
 *               be set to either true or false depending
 *               on whether items such as the core cell
 *               should be displayed as a link.
 *               If make links is not null, and is equal,
 *               ignoring case, to the string "true", then
 *               the following will happen:
 *               - links and memory tables will be displayed.
 *               - show/hide memory buttons will be hidden.
 *               See the constructor for Boolean(String).
 *               <p>
 *               The following style sheets must be loaded
 *               in the "head" section of the including jsp:
 *               .  /parmgr/css/parmgr.css
 *               .  /parmgr/css/hardware.css
 *               <p>
 *               The following java scripts must be loaded
 *               in the "head" section of the including jsp:
 *               .  /parmgr/js/parmgr.js
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/partition_general_body.jsp,v 1.46 2006/12/11 23:49:11 scheetz Exp $
 */
--%>

<%@ taglib uri="/com/hp/parmgr/ui/tags/parmgr" prefix="parmgr" %>
<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<%@ page import="java.util.List" %>
<%@ page import="java.lang.Boolean" %>
<%@ page import="java.util.ListIterator" %>
<%@ page import="com.hp.parmgr.model.Complex" %>
<%@ page import="com.hp.parmgr.model.Cell" %>
<%@ page import="com.hp.parmgr.model.NParComponent" %>
<%@ page import="com.hp.parmgr.model.CellSlot" %>
<%@ page import="com.hp.parmgr.model.HWPartition" %>
<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.OperatingSystem" %>
<%@ page import="com.hp.parmgr.util.ClmWarning" %>
<%@ page import="com.hp.parmgr.util.StringFormatUtil" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>


<%@ page errorPage="exception.jsp" %>

<i18n:bundle baseName="com.hp.parmgr.ui.PartitionView" id="bundle" />
<i18n:bundle baseName="com.hp.parmgr.ui.Global" id="globalResources" />
<i18n:bundle baseName="com.hp.parmgr.ui.CreateModifyNpar" id="memTable" />
<i18n:bundle baseName="com.hp.parmgr.ui.tags.tags" id="parmgrTagResources" />
<i18n:bundle baseName="com.hp.parmgr.ui.Arches" id="archesResources" />

<jsp:useBean id="parsession" scope="session" class="com.hp.parmgr.ui.ParmgrSession" />

<%
    String parIDStr = request.getParameter("partition");
    int parID = Integer.parseInt(parIDStr);
    String makeLinksStr = request.getParameter("makeLinks");
    boolean makeLinks = new Boolean(makeLinksStr).booleanValue();

    Discovery discovery = parsession.getDiscovery();
    Complex complex = discovery.getComplex();
    HWPartition partition = discovery.getPartition(parID);

    boolean isErrorPartition = partition.isErrorObject();

    String unknownStr = globalResources.getString("Application.Unknown.field");
    String naStr      = globalResources.getString("Application.NA.field");
    String yesStr     = globalResources.getString("Application.Yes.field");
    String noStr      = globalResources.getString("Application.No.field");

    String partitionName;
    if (isErrorPartition) {
        partitionName = unknownStr;
    } else {
        partitionName = partition.getName();
    }


    List cellSlots = discovery.getCellSlotsInPartition(parID);
    ListIterator iter = cellSlots.listIterator();

    int numCPUs = 0;
    int numDIMMs = 0;
    float amountMem = 0;
    int totalCells = 0;
    int numGoodCells = 0;
    Cell cellInPar = null;

    int numLockedCells = 0;
    int numReadErrorCells = 0;
    int numMinorErrorCells = 0;
    int activeCells = 0;
    int inactiveCells = 0;
    boolean atLeastOneGoodCell = false;
    boolean missingMemoryInfo = false;
    boolean supportsGSM = false;
    boolean supportsHyperThreading = false;
    boolean hyperThreadingActive = false;

    // Even if the partition is an error object, we can still get cells,
    // since that information is in the complex profile.  And since we can
    // get cells, we can get chassis.
    while ( iter.hasNext() ) {

        CellSlot nextCellSlot = (CellSlot)iter.next();
        if ( nextCellSlot.isPopulated() ) {
            totalCells++;
            Cell nextCell = discovery.getCell(nextCellSlot.getGlobalID());
            if (! nextCell.isErrorObject()) {
                // find a cell that has valid detailed information.
                if (cellInPar == null && nextCell.getStatus() > Cell.INACTIVE) {
                    cellInPar = nextCell;
                }

                if (nextCell.getStatus() == Cell.ACTIVE) {
                    activeCells++;
                }

                if( nextCell.getStatus() == Cell.INACTIVE ) {
                    inactiveCells++;
                }

                if (nextCell.getStatus() < Cell.INACTIVE) {
                    // Can't get memory information from cells that
                    // are powered-off or in POST (memory totals will
                    // be zero).
                    missingMemoryInfo = true;
                }

                numCPUs += nextCell.getCPUCount();
                numDIMMs += nextCell.getInstalledDIMMsCount();
                amountMem += nextCell.getTotalMemoryInstalledInGB();
                numGoodCells ++;

                if (nextCell.isGsmCapable()) {
                    supportsGSM = true;
                }
                if (nextCell.isHyperThreadCapable()) {
                    supportsHyperThreading = true;
                }
                if (nextCell.isHyperThreadingActive()) {
                    hyperThreadingActive = true;
                }
            }
            else {
                if (nextCell.getLockOwner() != null) {
                    numLockedCells++;
                }
                else {
                    numReadErrorCells++;
                }
            }

            if (nextCell.hasFailedOrDeconfiguredDevices()) {
                numMinorErrorCells++;
            }
        }
    }

    atLeastOneGoodCell = (numGoodCells > 0);

    String       hwStatusString  = "";
    StringBuffer hwStatusImage   = new StringBuffer(Util.URL_BASE_IMAGES + "/");
    String       hwStatusAltText = "";

    if( (activeCells == 0 && inactiveCells == 0) || totalCells == 0 ) {
        //
        // In this case, one of the following is true:
        // .  there are cell slots assigned, but no cells in the cell slots, or
        // .  all cells are powered off or powering on
        // So display an unknown image and string, as we don't know the
        // hardware status.
        //
        hwStatusString  =    parmgrTagResources.getString("Partition.hwstatus.unknown.label");
        hwStatusAltText =    parmgrTagResources.getString("Partition.hwstatus.unknown.altText");
        hwStatusImage.append(parmgrTagResources.getString("Partition.hwstatus.unknown.image"));
    }
    else if( numMinorErrorCells > 0 ) {
        hwStatusString  =    parmgrTagResources.getString("Partition.status.minor.label");
        hwStatusAltText =    parmgrTagResources.getString("Partition.status.minor.altText");
        hwStatusImage.append(parmgrTagResources.getString("Partition.status.minor.image"));
    }
    else if( numLockedCells > 0 ) {
        hwStatusString  =    parmgrTagResources.getString("Partition.status.locked.label");
        hwStatusAltText =    parmgrTagResources.getString("Partition.status.locked.altText");
        hwStatusImage.append(parmgrTagResources.getString("Partition.status.locked.image"));
    }
    else if( numReadErrorCells > 0 ) {
        hwStatusString  =    parmgrTagResources.getString("Partition.status.readerror.label");
        hwStatusAltText =    parmgrTagResources.getString("Partition.status.readerror.altText");
        hwStatusImage.append(parmgrTagResources.getString("Partition.status.readerror.image"));
    }
    else {
        hwStatusString  =    parmgrTagResources.getString("Partition.hwstatus.ok.label");
        hwStatusAltText =    parmgrTagResources.getString("Partition.hwstatus.ok.altText");
        hwStatusImage.append(parmgrTagResources.getString("Partition.hwstatus.ok.image"));
    }

    String amountMemStr;
    if (totalCells == 0 || numGoodCells > 0) {
        java.text.NumberFormat nf =
                com.hp.parmgr.model.DIMM.getMemoryNumberFormat(discovery.getLocale());
        amountMemStr = nf.format(amountMem);
    }
    else {
        amountMemStr = unknownStr;
    }


    // If there are no good cells (or no cells at all), there's no point
    // in looking for chassis, since the search is based on chassis that
    // are connected to cells (looking at the cells to find these, not the
    // chassis)

    boolean haveNonErrorChassis = false;
    int numChassis = 0;
    int numIOCards = 0;
    if (numGoodCells != 0) {

        // This method gets the cell slots and gets the chassis through
        // the cells.
        List chassisSlots = discovery.getChassisSlotsInPartition(parID);
        ListIterator iterChas = chassisSlots.listIterator();
        while (iterChas.hasNext()) {
            com.hp.parmgr.model.IOChassisSlot nextChassisSlot =
                                (com.hp.parmgr.model.IOChassisSlot)iterChas.next();
            if (nextChassisSlot.isPopulated()) {
                numChassis++;
                com.hp.parmgr.model.IOChassis nextChassis =
                                            discovery.getChassis(nextChassisSlot);
                if (!nextChassis.isErrorObject()) {
                    numIOCards += nextChassis.getPopulatedPCISlotCount();
                    haveNonErrorChassis = true;
                }
            }
        }
    }

    //
    // Decide whether to print chassis/cell info or whether to print
    // "unknown" strings.  This code could probably be condensed, but
    // for clarity (and maintainability), I've stretched it a bit and
    // added lots of comments.
    //
    // For those who like it condensed:
    //
    // boolean printCellInfo = (totalCells == 0 || numGoodCells > 0);
    // boolean printChassisInfo = (printCellInfo
    //                         && (numChassis == 0 || haveNonErrorChassis));
    //
    boolean printCellInfo = false;
    boolean printChassisInfo = false;
    if (totalCells == 0) {
        // No cells associated with the partition; perhaps only cell slots.
        // In this case there truly are no cells and chassis and we should
        // print 0.
        printCellInfo = true;
        printChassisInfo = true;
    } else if (numGoodCells > 0) {
        // There are cells, and there is at least one that is not an error
        // object: print the cell information, then check the chassis.
        // (In the case where there are cells, but all are error objects,
        // we print the unknown string for both cells and chassis by
        // leaving the booleans set to false.)
        printCellInfo = true;
        if (numChassis == 0) {
            // There are cells, so we looked for chassis, but there were
            // no chassis attached to them.
            printChassisInfo = true;
        } else if (haveNonErrorChassis) {
            // There were cells with chassis connected to them.  And there
            // was at least one chassis that was not an error object.
            printChassisInfo = true;
        }
        // else there were cells with chassis attached to them, but all of
        // chassis were error objects, so print the unknown string by
        // leaving the booleans set to false.
    }


    // Initialize so compiler doesn't complain.  This is never used if the
    // partition is an error object.
    int coreCellNo = -1;
    if (! isErrorPartition) {
        coreCellNo = partition.getCoreCellID();
    }


%>

<%if( makeLinks ) {%>
<script language="javascript">
function showMem(){
    showObject("memoryConfigTables");
    showObject("hideMemButton");
    hideObject("showMemButton");
}

function hideMem(){
    hideObject("memoryConfigTables");
    hideObject("hideMemButton");
    showObject("showMemButton");
}
</script>
<%}%>

<table class="properties" border="0" cellspacing="2" cellpadding="0">
  <tr>
    <td colspan="2" class="prop-table-caption">
      <i18n:message key="generalTab.partition.generalProperties" bundle="<%=globalResources%>"/>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.PartitionName.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
      <%=partitionName%>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.Status.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
      <%@ include file="partition_status_field.jsp" %>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.PartitionNumber.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
      <%=parID%>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.ActiveCoreCell.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
<% if (isErrorPartition) { %>
      <%=unknownStr%>
<% } else if( coreCellNo == HWPartition.INVALID_CORE_CELL_ID ) { %>
      <%=naStr%>
<% } else {
      CellSlot coreCellSlot = discovery.getCellSlot(coreCellNo);
      String coreCellLabel = coreCellSlot.getLongLabel();
      if(makeLinks) {
%>
        <a class="link" href="cell.jsp?globalID=<%=coreCellNo%>" target="_parent"><%=coreCellLabel%></a>
<%    } else { %>
        <%=coreCellLabel%>
<%
     }
   } %>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.OperatingSystem.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">

<%
    String osName = unknownStr;
    String hostName = unknownStr;

    if ( ! isErrorPartition) {

        OperatingSystem os = parsession.getOperatingSystem(partition);
        if (os != null) {
            osName = os.getNameAndVersion();
            hostName = os.getFullyQualifiedDomainName();
        }

    }

    out.println(osName);
%>

  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.HostName.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
    <%=StringFormatUtil.breakWithBr(hostName, 50)%>
  </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.IPAddress.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
<%
    String ipAddress;
    String typeString;
    if ( ! isErrorPartition) {
        ipAddress = partition.getAdminIPAddress();
        if (ipAddress == null || ipAddress.length() == 0) {
            ipAddress = globalResources.getString("Application.NA.field");
        }
        typeString = partition.getTypeString(discovery.getLocale());
    } else {
        ipAddress = unknownStr;
        typeString = unknownStr;
    }

    out.print(ipAddress);
%>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.CPUArchitecture.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
      <%=typeString%>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.Compatibility.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
<%
    /* get cell compatibility label */
    String compatibility;
    if (isErrorPartition || cellInPar == null) {
        compatibility = unknownStr;
    } else {
        compatibility = cellInPar.getCellCompatibilityLabelString();
    }
    out.print(compatibility);
%>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.FirmwareRevision.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
<%
    /* get firmware revision */
    String firmwareRevision;
    if (isErrorPartition || cellInPar == null) {
        firmwareRevision = unknownStr;
    } else {
        firmwareRevision = cellInPar.getFirmwareRevision();
    }

    out.print(firmwareRevision);

    String primaryBootPath;
    String haAlternateBootPath;
    String alternateBootPath;
    if (! isErrorPartition) {
        primaryBootPath = partition.getPrimaryBootPath();
        if (primaryBootPath == null) {
            primaryBootPath = naStr;
        }
        haAlternateBootPath = partition.getHAAlternateBootPath();
        if (haAlternateBootPath == null) {
            haAlternateBootPath = naStr;
        }
        alternateBootPath = partition.getAlternateBootPath();
        if (alternateBootPath == null) {
            alternateBootPath = naStr;
        }

    } else {
        primaryBootPath = unknownStr;
        haAlternateBootPath = unknownStr;
        alternateBootPath = unknownStr;
    }
%>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.HardwareStatus.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
      <img src='<%=hwStatusImage.toString()%>' alt='<%=hwStatusAltText%>'> <%=hwStatusString%>
    </td>
  </tr>
  <%  // display only if this npar is multithread capable
      if (supportsHyperThreading) {
  %>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="partition.general.hyperthread.enabled"
                    bundle="<%=archesResources%>" />
    </td>
    <td class="prop-table-data">
      <%=partition.isHyperThreadingEnabled() ? yesStr : noStr%>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="partition.general.hyperthread.active"
                    bundle="<%=archesResources%>" />
    </td>
    <td class="prop-table-data">
      <%=hyperThreadingActive ? yesStr : noStr%>
    </td>
  </tr>
  <%
      } // if (supportsHyperThreading)
  %>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.PrimaryBootPath.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
      <%=primaryBootPath%>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="field.HAAlternateBootPath.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data">
      <%=haAlternateBootPath%>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header-last">
      <i18n:message key="field.AlternateBootPath.label" bundle="<%=bundle%>" />
    </td>
    <td class="prop-table-data-last">
      <%=alternateBootPath%>
    </td>
  </tr>
  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>
  <tr>
    <td colspan="2" class="prop-table-caption">
      <i18n:message key="generalTab.summaryLabel" bundle="<%=globalResources%>"/>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="generalTab.summaryTable.numCells" bundle="<%=globalResources%>"/>
    </td>
    <td align="right" class="prop-table-data">
    <%=totalCells%>
    </td>
  </tr>


  <tr>
    <td class="prop-table-header">
      <i18n:message key="generalTab.summaryTable.numCPUs" bundle="<%=globalResources%>"/>
    </td>
    <td align="right" class="prop-table-data">
<% if (printCellInfo) { %>
    <%=numCPUs%>
<% } else { %>
    <%=unknownStr%>
<% } %>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="generalTab.summaryTable.numDIMMs" bundle="<%=globalResources%>"/>
    </td>
    <td align="right" class="prop-table-data">
<% if (printCellInfo) { %>
    <%=numDIMMs%>
<% } else { %>
    <%=unknownStr%>
<% } %>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="generalTab.summaryTable.amountMem" bundle="<%=globalResources%>"/>
    </td>
    <td align="right" class="prop-table-data">
      <%=amountMemStr%>
  </td>
  </tr>
  <tr>
    <td class="prop-table-header">
      <i18n:message key="generalTab.summaryTable.numChassis" bundle="<%=globalResources%>"/>
    </td>
    <td align="right" class="prop-table-data">
<% if (printChassisInfo) { %>
      <%=numChassis%>
<% } else { %>
      <%=unknownStr%>
<% } %>
    </td>
  </tr>
  <tr>
    <td class="prop-table-header-last">
      <i18n:message key="generalTab.summaryTable.numIOCards" bundle="<%=globalResources%>"/>
    </td>
    <td align="right" class="prop-table-data-last">
<% if (printChassisInfo) { %>
      <%=numIOCards%>
<% } else { %>
      <%=unknownStr%>
<% } %>
    </td>
  </tr>

<% if (missingMemoryInfo) {
    // Put this as the last row of the table, rather than outside
    // the table, so it is snugged right up against the bottom
    // of the last row.
%>
    <tr><td colspan=2 class=parmgr-small-text>
    <i18n:message key="General.missingMemoryInfo" bundle="<%=parmgrTagResources%>"/>
    </td></tr>
<% } %>

  <tr>
    <td colspan="2">&nbsp;</td>
  </tr>

</table>

<%
    // draw the interleave memory range table and the CLM configuration table
    // for this partition.
    boolean renderActual;
    int parStatus = discovery.getPartitionStatus(parID);
    if (parStatus == NParComponent.ACTIVE) {
        renderActual = true;
    } else {
        renderActual = false;
    }

    String tableDivStyle = "hide";
    String showMemButtonDivStyle = "show";
    String hideMemButtonDivStyle = "hide";

    if (!makeLinks) {
    // we are in complex details. Have to hide the buttons and always show
    // memory tables.
        tableDivStyle = "show";
        showMemButtonDivStyle = "hide";
        hideMemButtonDivStyle = "hide";
    }
%>


<table border="0" cellpadding="0" cellspacing="0">


<%if( makeLinks ) { %>
  <tr>
    <td>
      <div id="showMemButton" class="<%=showMemButtonDivStyle%>">
        <input type="button"
               name="AdvancedOptions"
               class="button"
               onClick="showMem();"
               value="<%=bundle.getString("button.showMem.label")%>"
        />
      </div>

      <div id="hideMemButton" class="<%=hideMemButtonDivStyle%>">
        <input type="button"
               name="AdvancedOptions"
               class="button"
               onClick="hideMem();"
               value="<%=bundle.getString("button.hideMem.label")%>"
        />
      </div>
    </td>
  </tr>
  <tr>
  <td>&nbsp;</td>
  </tr>
<%}%>

  <tr>
    <td class="parmgr-regular-text">
      <%if( makeLinks ) {%>
        <div id="memoryConfigTables" class="<%=tableDivStyle%>">
      <%}%>

      <parmgr:InterleaveRangeTable discovery="<%=discovery%>"
                                   resources="<%=memTable%>"
                                   showLinks="<%=makeLinks%>"
                                   parID="<%=parID%>" />
      <p>

      <parmgr:CLMTable discovery="<%=discovery%>"
                       resources="<%=memTable%>"
                       parID="<%=parID%>"
                       showLinks="<%=makeLinks%>"
                       renderActual="<%=renderActual%>"
                       renderInputs="false" />

<%-- @TODO:  Un-comment for the official Montecito release
  <%  // display only if this npar is GSM capable
      if (supportsGSM) {
  %>
      <p>
      <table>
      <tr><td class="prop-table-caption">
      <i18n:message key="partition.general.gsm.config.heading"
                      bundle="<%=archesResources%>"/>
      </td></tr>
      <tr><td class="prop-table-data-last">
      <% if (complex.getGSMSharingType() == Complex.GSM_DISABLED) { %>
            <i18n:message key="partition.general.gsm.config.disabled"
                          bundle="<%=archesResources%>"/>
      <% } else if (complex.getGSMSharingType() == Complex.GSM_CPLX_WIDE) { %>
            <i18n:message key="partition.general.gsm.config.enabled"
                          bundle="<%=archesResources%>"/>
      <% } else { %>
            <i18n:message key="partition.general.gsm.config.per.npar"
                          bundle="<%=archesResources%>"/>
            </td></tr>
            <%
            boolean[][] matrix = complex.getGSMSharing();
            String sharesWithStr = "";
            for (int i = 0; i < matrix[parID].length; ++i) {
                if (i != parID && matrix[parID][i]) {
                    HWPartition hwPar = discovery.getPartition(i);
                    // bits could be set for non-existent nPars (hwPar == null)
                    // - do not even attempt to display these!!!
                    // (would get null-pointer exception anyway):
                    if (hwPar != null) {
                        if (sharesWithStr != null && !sharesWithStr.equals("")) {
                            // add a separator
                            sharesWithStr += "; ";
                        }
                        sharesWithStr += hwPar.getLongLabel();
                    }
                }
            }
            %>
            <td class="prop-table-data-last">
                <%=sharesWithStr%>
      <% } %>
      </td></tr>
      </table>
  <%  // display only if this npar is GSM capable
      } // if (supportsGSM)
  %>
--%>

      <%if( makeLinks ) {%>
        </div>
      <%}%>

<%=ClmWarning.printCLMWarnings(discovery, parID, false, bundle.getLocale())%>

    </td>
  </tr>
</table>
