<%--
/**
 * The content rendered for the partition's single partition tab.
 *
 * @task name = Implement single-partition view for Partition Scope.
 * cycle = 4
 * owner = Paul
 * phase = TEST
 * status = Done
 * description = Single-partition view for Partitions.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/partition_partition.jsp,v 1.24 2008/03/05 13:05:41 sridharav Exp $
 * @risk: Low
 *
 */
--%>

<%@ taglib uri="/com/hp/parmgr/ui/tags/parmgr" prefix="parmgr" %>
<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>

<%@ page import="java.util.Iterator" %>
<%@ page import="java.util.List" %>
<%@ page import="java.util.ArrayList" %>
<%@ page import="java.text.NumberFormat" %>
<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.HWPartition" %>
<%@ page import="com.hp.parmgr.model.CellSlot" %>
<%@ page import="com.hp.parmgr.model.Cell" %>
<%@ page import="com.hp.parmgr.model.DIMM" %>
<%@ page import="com.hp.parmgr.model.Complex" %>
<%@ page import="com.hp.parmgr.model.IOChassisSlot" %>
<%@ page import="com.hp.parmgr.model.IOChassis" %>
<%@ page import="com.hp.parmgr.model.OperatingSystem" %>
<%@ page import="com.hp.parmgr.ui.SelectionManager" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page import="com.hp.parmgr.util.StringFormatUtil" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<%@ page errorPage="exception.jsp" %>

<i18n:bundle baseName="com.hp.parmgr.ui.ComplexView" id="bundle" />
<i18n:bundle baseName="com.hp.parmgr.ui.Parmgr" id="parmgrProperties" />
<i18n:bundle baseName="com.hp.parmgr.ui.tags.tags" id="parmgrTagResources" />

<jsp:useBean id="parsession" scope="session" class="com.hp.parmgr.ui.ParmgrSession" />

<%

    Discovery discovery = parsession.getDiscovery();
    SelectionManager selectedCheckboxes = parsession.getSelectedCheckboxes();
    NumberFormat nf = DIMM.getMemoryNumberFormat(discovery.getLocale());

    String parID = request.getParameter("partition");
    int partitionID = Integer.parseInt(parID);
    HWPartition partition = discovery.getPartition(partitionID);

    /*
     * The partitionkey taglib expects a list of partitions, but in
     * this case we don't have a list.  So we'll build one and stick
     * our single partition in it.  The partition is guaranteed to be
     * valid because it's checked in partition.jsp prior to loading
     * the partition_partition.jsp workspace iframe.
     */

    List currentPartition = new ArrayList();
    if (partition != null) {
       currentPartition.add(partition);
    }

        String osName = "?";
        String hostName = "?";

    OperatingSystem os = parsession.getOperatingSystem(partition);
    if (os != null) {
        osName = os.getNameAndVersion();
        hostName = os.getFullyQualifiedDomainName();
    }

    //
    // Count how many active cells we have in this partition and
    // figure out how much OK memory is in those cells.
    //
    List cellSlots = discovery.getCellSlotsInPartition(partitionID);
    int numLockedCells = 0;
    int numReadErrorCells = 0;
    int numMinorErrorCells = 0;
    int activeCells = 0;
    int totalCells = 0;
    double totalMem = 0.0;
    boolean atLeastOneGoodCell = false;
    boolean missingMemoryInfo = false;
    for (Iterator iter = cellSlots.iterator(); iter.hasNext();)
    {
        CellSlot cellSlot = (CellSlot) iter.next();
        if (cellSlot.isPopulated())
        {
            Cell cell = discovery.getCell(cellSlot.getID());
            if (cell != null) {
                totalCells ++;
                if (!cell.isErrorObject()) {
                    atLeastOneGoodCell = true;
                    if (cell.getStatus() == Cell.ACTIVE) {
                        activeCells++;
                    }
                    if (cell.getStatus() < Cell.INACTIVE) {
                        // Can't get memory information from cells that
                        // are powered-off or in POST (memory amounts will
                        // be zero).
                        missingMemoryInfo = true;
                    }
                    totalMem += (double) cell.getTotalMemoryOK();
                }
                else {
                    if (cell.getLockOwner() != null) {
                        numLockedCells++;
                    }
                    else {
                        numReadErrorCells++;
                    }
                }

                if(cell.hasFailedOrDeconfiguredDevices()) {
                    numMinorErrorCells++;
                }
            }
        }
    }
    String memoryOK = nf.format(totalMem/1024.0);

    //
    // Count how many active chassis we have in this partition.
    //
    // No point in doing this if there are no good cells, since
    // getChassisSlotsInPartition uses the cells to get the chassis.
    //
    int activeChassis = 0;
    if (atLeastOneGoodCell) {
        List chassisSlots = discovery.getChassisSlotsInPartition(partitionID);

        for (Iterator iter = chassisSlots.iterator(); iter.hasNext();)
        {
            IOChassisSlot chassisSlot = (IOChassisSlot) iter.next();
            if (chassisSlot.isPopulated())
            {
                IOChassis chassis = discovery.getChassis(chassisSlot);
                if (chassis != null) {
                    if (!chassis.isErrorObject()) {
                        if (chassis.getStatus() == IOChassis.ACTIVE) {
                            activeChassis++;
                        }
                    }
                }
            }
        }
    }

%>

<HTML>
<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<%@ include file="session_keep_alive.jsp" %>
<script type='text/javascript' language='JavaScript' src='<%=Util.URL_PARMGR_JS%>'></script>
<link type='text/css' rel='stylesheet' href='<%=Util.URL_PARMGR_CSS%>'>
<link type='text/css' rel='stylesheet' href='<%=Util.URL_HARDWARE_CSS%>'>
</HEAD>

<BODY onLoad="loadMenuFrame('<%=Util.URL_BASE_JSP%>')">

  <!-- begin partition key -->
  <parmgr:partitionkey partitions="<%=currentPartition%>"
                       showCheckboxes="false"
                       resources="<%=parmgrTagResources%>"
                       selectedCheckboxes="<%=selectedCheckboxes%>"/>
  <!-- end partition key -->

  <table cellspacing="5" bgcolor="ffffff" border=0 width="100%">
    <tr><td>
      <hr class="separator">
      <table class="partition" cellSpacing="4" cellPadding="0" border="0" width="100%">
        <tr>
          <td class="partition-head" width="100%">
            <table cellSpacing="4" cellPadding="0" border="0" width="100%">
              <tr>
                <td class="partition-head" width="100%">
<% if (partition.isErrorObject()) { %>
                  <a class="link" href="partition.jsp?partition=<%=partitionID%>" target="_parent"><font size="+1"><%=partition.getLabel()%></font></a>
<% } else { %>
                  <a class="link" href="partition.jsp?partition=<%=partitionID%>" target="_parent"><font size="+1"><%=partition.getName()%>&nbsp;(<%=partition.getLabel()%>)</font></a>
<% } %>
                </td>
              </tr>
            </table>
          </td>
        </tr>
        <tr>
          <td>
            <!-- table just to put some space around properties table -->
            <table width="50%" cellSpacing=10 cellPadding=0 border=0>
              <tr>
                <td width="100%">
                  <!-- Properties table -->
                  <table width="100%" cellSpacing=0 cellPadding=0 border=0>
                    <tr>
                      <td class="prop-table-header"><i18n:message key="field.Status.label" bundle="<%=bundle%>" /></td>
                      <td class="prop-table-data" align="left" width="100%">
                        <%@ include file="partition_status_field.jsp" %>
                      </td>
                    </tr>
                    <tr>
                      <td class="prop-table-header"><i18n:message key="field.Hostname.label" bundle="<%=bundle%>" /></td>
                      <td class="prop-table-data" align="left" width="100%"><%=hostName%></td>
                    </tr>
                    <tr>
                      <td class="prop-table-header"><i18n:message key="field.OSRevision.label" bundle="<%=bundle%>" /></td>
                      <td class="prop-table-data" align="left" width="100%"><%=osName%></td>
                    </tr>
                    <tr>
                      <td class="prop-table-header"><i18n:message key="field.ActiveCells.label" bundle="<%=bundle%>" /></td>
                      <td class="prop-table-data" align="left" width="100%"><%=activeCells%></td>
                    </tr>
                    <tr>
                      <td class="prop-table-header"><i18n:message key="field.ActiveIOChassis.label" bundle="<%=bundle%>" /></td>
                      <td class="prop-table-data" align="left" width="100%"><%=activeChassis%></TD>
                    </tr>
                    <tr>
                      <td class="prop-table-header-last"><i18n:message key="field.MemoryOK.label" bundle="<%=bundle%>" /></td>
                      <td class="prop-table-data-last" align="left" width="100%"><%=memoryOK%></td>
                    </tr>
<% if (missingMemoryInfo) {
    // Put this as the last row of the table, rather than outside
    // the table, so it is snugged right up against the bottom
    // of the last row.
%>
                    <tr>
                      <td colspan=2 class="parmgr-small-text">
                            <i18n:message key="General.missingMemoryInfo" bundle="<%=parmgrTagResources%>"/>
                      </td>
                    </tr>
<% } %>
                  </table>
                  <!-- end of Properties table -->
                </td>
              </tr>
            </table>
            <!-- end of space table around Properties table -->
          </td>
        </tr>
<%
    //
    // Draw a row for each cellSlot in the partition.
    //
    for (Iterator iter = cellSlots.iterator(); iter.hasNext();)
    {
        CellSlot cellSlot = (CellSlot) iter.next();
        int cellID = cellSlot.getID();
%>

        <tr>
          <td>
            <!-- table containing a cell, connector and chassis graphics -->
            <table cellSpacing=0 cellPadding=0 border=0>
              <tr>
                <td valign="top">
                  <parmgr:cellinfo discovery="<%=discovery%>"
                                   slot="<%=cellSlot%>"
                                   showPartition="true"
                                   showCPUs="true"
                                   resources="<%=parmgrTagResources%>"
                                   selectedCheckboxes="<%=selectedCheckboxes%>"/>

                </td>
<%
         //
         // If the cell is populated check for (and draw) any connected
         // IO Chassis.
         //
         if (cellSlot.isPopulated())
         {
             Cell cell = discovery.getCell(cellID);
             if (!cell.isErrorObject()) {
                 if (cell.isConnectedToIOChassis())
                 {
                     IOChassisSlot chassisSlot = discovery.getChassisSlot(cell.getConnectedIOChassisID());
%>
                <td><img height="12" width="28" src="<%=Util.URL_BASE_IMAGES + "/ch.gif"%>"
                     alt='<i18n:message key="field.ChassisConnector.text" bundle="<%=parmgrProperties%>" />'></td>
                <td>
                  <parmgr:chassisinfo discovery="<%=discovery%>"
                                      slot="<%=chassisSlot%>"
                                      showPartition="true"
                                      resources="<%=parmgrTagResources%>"
                                      selectedCheckboxes="<%=selectedCheckboxes%>"/>

                </td>
<%
                 }
             }
         }
%>
              </tr>
            </table>
          </td>
        </tr>
<%
       } // for cellSlots
%>

      </table>
    </td></tr>

<%

    //
    // Now do available resources
    //

%>
    <tr><td>
      <hr class="separator">
      <table class="partition" cellSpacing="4" cellPadding="0" border="0" width="100%">
        <tr>
          <td class="partition-head" width="100%">
            <table cellSpacing="4" cellPadding="0" border="0" width="100%">
              <tr>
                <td class="partition-head" width="100%">
                  <a class="link" href="partition.jsp?partition=-1" target="_parent"><font size="+1"><i18n:message key="PartitionKey.Available.label" bundle="<%=parmgrTagResources%>" /></font></a>
                </td>
              </tr>
            </table>
          </td>
        </tr>
<%
    //
    // Draw a row for each cellSlot in the partition.
    //
    Complex complex = discovery.getComplex();
    List allCellSlots = discovery.getCellSlots();

    for (Iterator iter = allCellSlots.iterator(); iter.hasNext();)
    {
        CellSlot cellSlot = (CellSlot) iter.next();
        int cellID = cellSlot.getID();
        if (complex.getCellType(cellID) == Complex.FREE)
        {
%>
        <tr>
          <td>
            <!-- table containing a cell, connector and chassis graphics -->
            <table cellSpacing=0 cellPadding=0 border=0>
              <tr>
                <td valign="top">
                  <parmgr:cellinfo discovery="<%=discovery%>"
                                   slot="<%=cellSlot%>"
                                   showPartition="true"
                                   showCPUs="true"
                                   resources="<%=parmgrTagResources%>"
                                   selectedCheckboxes="<%=selectedCheckboxes%>"/>

                </td>
<%
             //
             // If the cell is populated check for (and draw) any connected
             // IO Chassis.
             //
             if (cellSlot.isPopulated())
             {
                 Cell cell = discovery.getCell(cellID);
                 if (!cell.isErrorObject()) {
                     if (cell.isConnectedToIOChassis())
                     {
                         IOChassisSlot chassisSlot = discovery.getChassisSlot(cell.getConnectedIOChassisID());
%>
                <td><img height="12" width="28" src="<%=Util.URL_BASE_IMAGES + "/ch.gif"%>"
                     alt='<i18n:message key="field.ChassisConnector.text" bundle="<%=parmgrProperties%>" />'></td>
                <td>
                  <parmgr:chassisinfo discovery="<%=discovery%>"
                                      slot="<%=chassisSlot%>"
                                      showPartition="true"
                                      resources="<%=parmgrTagResources%>"
                                      selectedCheckboxes="<%=selectedCheckboxes%>"/>

                </td>
<%
                     }
                 }
             }
%>
              </tr>
            </table>
          </td>
        </tr>
<%
       } // if cellSlot is free

     } // for cellSlots

%>

      </table>
    </td></tr>
  </table>

</BODY>
</HTML>
