<%--
/**
 * Utility jsp to determine the status of a partition and display
 * the status and icon.
 *
 * NOTE: This jsp relies on the following variables being set:
 *
 *   boolean  atLeastOneGoodCell  true if there is at least 1
 *                                non-read-error cell in partition
 *   int      activeCells         the number of active cells in
 *                                the partition
 *   int      totalCells          the total number of cells in
 *                                the partition
 *   int      numMinorErrorCells  the number of cells with minor
 *                                errors in the partition
 *   int      numReadErrorCells   the number of cells with read
 *                                errors in the partition
 *   int      numLockedCells      the number of cells that were locked
 *                                when the read was attempted
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/partition_status_field.jsp,v 1.3 2005/11/09 19:14:08 tmartin Exp $
 * @risk: Low
 */
--%>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>

<%
//
// The following table shows the possible values for each variable used
// to determine the state of a partition.
//
// state | atLeastOneGoodCell   |  activeCells   | totalCells   |  Status
// =======================================================================
//   1   |          0           |      >0        |      >0      |  Error
//   2   |          0           |      >0        |      ==0     |  Inactive
//   3   |          0           |      ==0       |      >0      |  Error
//   4   |          0           |      ==0       |      ==0     |  Inactive
//   5   |          1           |      >0        |      >0      |  Active
//   6   |          1           |      >0        |      ==0     |  Active
//   7   |          1           |      ==0       |      >0      |  Inactive
//   8   |          1           |      ==0       |      ==0     |  Inactive
        //
        // Build the status string.  This series of if statements could
        // be condensed for efficiency, but for clarity and maintainability,
        // it was made more explicit regarding the possible states of partitions
        // within the partition.
        //
        // NOTE: the order of these if statements matters!
        //
        String partitionStatusString   = "";
        if( atLeastOneGoodCell && activeCells > 0 ) {
            //
            // See states 5 & 6 in the table above.
            //
            partitionStatusString = parmgrTagResources.getString("Partition.status.active.label");
        }
        else if( totalCells == 0 || (atLeastOneGoodCell && activeCells == 0) ) {
            //
            // See states 2, 4, 7 & 8 in the table above.  State 6 is
            // handled by the first condition.
            //
            partitionStatusString = parmgrTagResources.getString("Partition.status.inactive.label");
        }
        else {
            //
            // See states 1 & 3 in the table above.
            //
            if( numLockedCells > 0 ) {
                partitionStatusString = parmgrTagResources.getString("Partition.status.locked.label");
            }
            else {
                partitionStatusString = parmgrTagResources.getString("Partition.status.readerror.label");
            }
        }

        //
        // Build the status image.  The following rules (in order) are
        // applied for displaying the correct image:
        // 1) the minor error image if the partition contains least 1 known
        //    minor error cell
        // 2) the unknown image if the partition contains any read-error cells
        // 3) the unknown image if the partition contains any locked cells
        // 4) the active image if the partition contains at least 1 active cell
        // 5) the inactive image otherwise
        //
        // Note: the above rules are for determining which icon is displayed
        // next to the status string.  Therefore, an active partition with
        // >0 cells with a read error will result in a status field of
        // "? Active" (the ? is an icon, not a text character).  The intent
        // is to make ALL irregularities show up in the icon and alt text,
        // but display the status string to show the actual partition status.
        //
        StringBuffer partitionStatusImage   = new StringBuffer(Util.URL_BASE_IMAGES + "/");
        String       partitionStatusAltText = "";
        if( numMinorErrorCells > 0 ) {
            partitionStatusImage.append(parmgrTagResources.getString("Partition.status.minor.image"));
            partitionStatusAltText = parmgrTagResources.getString("Partition.status.minor.altText");
        }
        else if( numReadErrorCells > 0 ) {
            partitionStatusImage.append(parmgrTagResources.getString("Partition.status.readerror.image"));
            partitionStatusAltText = parmgrTagResources.getString("Partition.status.readerror.altText");
        }
        else if( numLockedCells > 0 ) {
            partitionStatusImage.append(parmgrTagResources.getString("Partition.status.locked.image"));
            partitionStatusAltText = parmgrTagResources.getString("Partition.status.locked.altText");
        }
        else if( activeCells > 0 ) {
            partitionStatusImage.append(parmgrTagResources.getString("Partition.status.active.image"));
            partitionStatusAltText = parmgrTagResources.getString("Partition.status.active.altText");
        }
        else {
            //
            // There are no minor errors, read errors, locked cells, or active cells,
            // so the partition must be inactive.
            //
            partitionStatusImage.append(parmgrTagResources.getString("Partition.status.inactive.image"));
            partitionStatusAltText = parmgrTagResources.getString("Partition.status.inactive.altText");
        }
%>
<img src='<%=partitionStatusImage.toString()%>' alt='<%=partitionStatusAltText%>'> <%=partitionStatusString%>
