<%--
/**
 * This page reports the existence of a pending profile, as well as any
 * details about the profile that may exist.  It also gives the user
 * the option of cancelling the profile, waiting for it to push out,
 * or continuing.
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/pending_profile.jsp,v 1.17 2008/03/05 13:05:41 sridharav Exp $
 * @risk: Low
 *
 * @task name = Create pending profile report page
 *       cycle = 6
 *       phase = TEST
 *       owner = Paul
 *       status = Done
 *       description = None
 *
 * @task name = Link pending profile checks into relevant JSPs
 *       cycle = 6
 *       phase = TEST
 *       owner = Paul
 *       status = Done
 *       description = None
 *
 */
--%>

<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>
<link type='text/css' rel='stylesheet' href='<%=Util.URL_PARMGR_CSS%>'>
<jsp:useBean id="parsession" scope="session" class="com.hp.parmgr.ui.ParmgrSession" />
<i18n:bundle baseName="com.hp.parmgr.ui.Parmgr" id="parmgrProperties" />
<i18n:bundle baseName="com.hp.parmgr.util.WaitMessages" id="waitMessages" />
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<HTML>
<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<%@ include file="session_keep_alive.jsp" %>
    <TITLE>
       <i18n:message key="pendingProfile.title" bundle="<%=parmgrProperties%>" />
    </TITLE>

    <script type='text/javascript' language='JavaScript' src='<%=Util.URL_PARMGR_JS%>'></script>

</HEAD>

<BODY>

<%@ page import="com.hp.parmgr.model.Complex" %>
<%@ page import="com.hp.parmgr.model.Discovery" %>
<%@ page import="com.hp.parmgr.model.CellSlot" %>
<%@ page import="com.hp.parmgr.model.HWPartition" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page import="com.hp.parmgr.util.WaitForProfile" %>
<%@ page import="com.hp.parmgr.util.StringFormatUtil" %>
<%@ page import="java.util.ResourceBundle" %>

<%

   Discovery discovery = parsession.getDiscovery();
   Complex pendingComplex = discovery.getPendingComplex();

   /* Determine forwarding URL. */
   String url = parsession.getFromURL();
   if (request.getQueryString() != null) 
   {
       String joinQueryChar = "?";
       if (url.matches(".*[?].*")) joinQueryChar = "&";
       url = url + joinQueryChar + request.getQueryString();
   }
   
   // JAGaf88550: guard against cross site scripting vulnerabilities,
   // by encoding parameters that may be output to the HTML stream:
   url = StringFormatUtil.encodeHtmlEntities(url);

   if ((pendingComplex == null) || (pendingComplex.isErrorObject())) {
       /* No pending complex profile anymore.  Lower the check
        * pending profile flag to ensure that a redundant pending
        * profile check is not conducted, then return to original URL. */
       parsession.setCheckPending(false);
       pageContext.forward(url);
   }


   String helpURL = parsession.getHelpURL("pendingProfile");

   int [] affectedCells = pendingComplex.getAffectedCells();
   int [] rebootPartitions = pendingComplex.getRebootPartitions();

   /* Create wait object in case user presses "Wait" button. */

   WaitForProfile wait = new WaitForProfile(discovery, waitMessages);
   wait.setForwardURL(url);
   parsession.setWaitBean(wait);

%>

<p class="subwindow-heading">
    <i18n:message key="pendingProfile.title" bundle="<%=parmgrProperties%>" />
</p>

<p>
<i18n:message key="error.pendingProfile.intro" bundle="<%=parmgrProperties%>" />

<%  if (affectedCells != null) {
%>

        <P>
        <i18n:message key="error.pendingProfile.affectedCells" bundle="<%=parmgrProperties%>" />
        <UL>

<%
            for (int i = 0; i < affectedCells.length; i++) {
              String longLabel =
                  discovery.getCellSlot(affectedCells[i]).getLongLabel();
%>

              <LI><%=longLabel%></LI>

<%
            }
%>

        </UL>

<%
    }
%>

<%  if (rebootPartitions != null) {
%>

        <P>
        <i18n:message key="error.pendingProfile.rebootPartitions" bundle="<%=parmgrProperties%>" />
        <UL>

<%
            for (int i = 0; i < rebootPartitions.length; i++) {
                HWPartition p = discovery.getPartition(rebootPartitions[i]);
                String label;
                if (p.isErrorObject()) {
                    label = p.getLabel();
                } else {
                    label = p.getLongLabel();
                }
%>

              <LI><%=label%></LI>

<%
            }
%>

        </UL>

<%
    }
%>

<P>
<i18n:message key="error.pendingProfile.button.description.intro" bundle="<%=parmgrProperties%>" />
    <ul>
    <li>
        <i18n:message key="error.pendingProfile.cancelButton.description" bundle="<%=parmgrProperties%>" />
    </li>
    <li>
        <i18n:message key="error.pendingProfile.waitButton.description" bundle="<%=parmgrProperties%>" />
    </li>
    <li>
        <i18n:message key="error.pendingProfile.continueButton.description" bundle="<%=parmgrProperties%>" />
        <ul>
        <li>
            <i18n:message key="error.pendingProfile.continueWarning.1" bundle="<%=parmgrProperties%>" />
        </li>
        <li>
            <i18n:message key="error.pendingProfile.continueWarning.2" bundle="<%=parmgrProperties%>" />
        </li>
        </ul>
    </li>
    </ul>
<P>
<p align=left>
<table>
   <tr>
      <td valign="top">
         <FORM action="<%=url%>" method="post">
            <INPUT class="button" type="submit"
                   value="<%=parmgrProperties.getString("button.cancelProfile.str")%>"
                   name="cancelProfile" />
         </FORM>
      </td>
      <td valign="top">
         <FORM action="wait.jsp" method="post">
            <INPUT class="button" type="submit"
                   value="<%=parmgrProperties.getString("button.wait.str")%>"
                   name="wait" />
         </FORM>
     </td>
      <td valign="top">
         <FORM action="<%=url%>" method="post">
            <INPUT class="button" type="submit"
                   value="<%=parmgrProperties.getString("button.continue.str")%>"
                   name="continue" />
         </FORM>
      </td>
      <td valign="top">
         <INPUT class="button" type="button"
                value="<%=parmgrProperties.getString("button.help.str")%>"
                name="help"
                onclick="showHelp('<%=helpURL%>')" />
      </td>
   </tr>
</table>
</p>

</BODY>
</HTML>
