<%--
/**
 * The JSP page for "unassign cells from nPar'
 *
 * @version $Header: /usr/local/vse/parmgr/src/web/jsp/unassign_cells_from_npar.jsp,v 1.23 2008/03/05 13:11:07 sridharav Exp $
 * @risk Medium
 *
 * @task name = Unassign cells from npar action
 *	cycle = 5
 *	owner = Kurt
 *	phase = test
 *	status = done
 *	description = GUI for unassign cells from npar action
 */
--%>

<%--
     Using the I18N tag here helps set the appropiate charset for the page.
     This is needed even though this reference to the bundle isn't used
     elsewhere in the page.
--%>
<%@ taglib uri="http://jakarta.apache.org/taglibs/i18n-1.0" prefix="i18n" %>
<i18n:bundle baseName="com.hp.parmgr.ui.Parmgr" id="bundle" />

<%@ page errorPage="exception.jsp" %>
<%@ page import="com.hp.parmgr.ui.SelectionManager,
                 com.hp.parmgr.ui.SessionTimeOut,
                 com.hp.parmgr.ui.UnauthorizedUserException,
                 com.hp.parmgr.model.CellSlot,
                 com.hp.parmgr.model.Complex,
                 com.hp.parmgr.model.HWPartition,
                 com.hp.parmgr.model.Discovery,
                 com.hp.parmgr.util.Log,
                 java.util.Iterator,
                 java.util.TreeSet" %>
<%@ page import="com.hp.parmgr.ui.tags.Util" %>
<%@ page contentType="text/html;charset=UTF-8"%>
<%@ page pageEncoding="UTF-8"%>

<jsp:useBean id="parsession" scope="session" class="com.hp.parmgr.ui.ParmgrSession" />

<%
    java.util.ResourceBundle pmBundle =
        java.util.ResourceBundle.getBundle( "com.hp.parmgr.ui.Parmgr",
                                            parsession.getLocale());
    java.util.ResourceBundle ucBundle =
        java.util.ResourceBundle.getBundle( "com.hp.parmgr.ui.UnassignCells",
                                            parsession.getLocale());
    java.util.ResourceBundle tagBundle =
        java.util.ResourceBundle.getBundle( "com.hp.parmgr.ui.tags.tags",
                                            parsession.getLocale());

    // set strings used in the page
    String nparOpt   = "nparID";
    String tgtPage   = "modify_npar.jsp";
    String thisPage  = "unassign_cells_from_npar.jsp";
    String pageTitle = ucBundle.getString("UnassignCells.title");
    String Help      = pmBundle.getString("button.help.str");
    String helpPage  = "unassignCells";
%>
<HTML>

<HEAD>
<META http-equiv="Content-Type" content="text/html;charset=UTF-8">
<%@ include file="session_keep_alive.jsp" %>
    <LINK type="text/css" rel="stylesheet" href="<%=Util.URL_PARMGR_CSS%>">
    <script type='text/javascript'
            language='JavaScript'
            src='<%=Util.URL_PARMGR_JS%>' >
    </script>
    <TITLE> <%=pageTitle%></TITLE>

    <%=SessionTimeOut.javascript(session, bundle.getLocale())%>
</HEAD>

<%
    Discovery discovery = parsession.getDiscovery();

    // Do this after getting discovery
    if (!parsession.userAuthorizedToMakeChanges("parmodify")) {
        throw new UnauthorizedUserException(parsession.getFromURL(), parsession.getLocale());
    }

    // get any possible parameters on the request
    boolean fromCancel   = (request.getParameter("cancel") != null);
    boolean fromContinue = (request.getParameter("continue") != null);
    String nparIdStr     = request.getParameter(nparOpt);

    // Are we coming on on a cancel button hit?
    if( fromCancel ) {
        // remove selections from session
        session.removeAttribute( "selectedCells" );
        // log exit, and go back
        Log.summary( Log.EVENT_DONE, Log.RESULT_CANCELED,
                     "Cancel Unassign Cells From nPartition" );
        // go back to the previous page
%>
<BODY onload="window.location.replace('<%=parsession.getFromURL()%>')">
</BODY>
</HTML>
<%
   } else if( fromContinue ) {
       // coming in from the continue button
       // leave cell selections on the session
       // log the transition
       // note that the nparID value must be non-null in this case.
       Log.summary(Log.EVENT_DONE, Log.RESULT_SUCCESS,
                   "Transition from Unassign Cells From nPartition to Modify nPartition." );
       // and go to modify npar
%>
<BODY onload="window.document.unassignCells.submit()">
    <p>
    <b>
    <%=ucBundle.getString("UnassignCells.goToModify")%>
    </b>
    </p>
    <FORM method="get" action="<%=tgtPage%>" name="unassignCells">
    <INPUT type="hidden" name="<%=nparOpt%>" value="<%=nparIdStr%>">
    </FORM>
</BODY>
</HTML>
<%
   } else {


    // first time into this page
    // log the task entry.
    Log.summary(Log.EVENT_START, Log.RESULT_NONE,
                "Start Unassign Cells from nPartition");

    // Validate entry conditions
    // Only cells assigned to the same partition are selected.  We will use
    // the partition those cells are assigned to and ignore any other
    // partition that might be selected, and always tell modify npar
    // which partition to modify.  Note that a cell might be
    // selected implicitly by being in the cell scope.

    int implicitCellID  = parsession.getImplicitCellID();
    Complex complex     = discovery.getComplex();

    boolean haveError   = false;    // flag for having some kind of error
    String errMsg       = "";

    int parID = -1;                 // partition to operate on.

    SelectionManager selectionMgr  = parsession.getSelectedCheckboxes();

    // Make a local copy of the selection manager to modify npar page
    SelectionManager localSelects = (SelectionManager) selectionMgr.clone();

    // add the implicit cell ID into the set of selected cells
    if( implicitCellID >= 0 ) {
        localSelects.addSelection(
            SelectionManager.getCellCheckboxID(implicitCellID) );
    }
    TreeSet selectedCells = localSelects.getSelectedCells();

    // Check error conditions
    if( selectedCells.size() <= 0 ) {
         // no cells selected
         haveError = true;
         errMsg = ucBundle.getString( "UnassignCells.error.noCells" );
         Log.error( "No cells have been selected.");
    } else {
         // check that all selected cells are assigned to the same partition
         Iterator iter = selectedCells.iterator();
         // assert: hasNext() is true since there's at least one cell selected
         int cellID = ((Integer) iter.next()).intValue();
         if( complex.getCellType(cellID) == Complex.FREE ) {
             // first cell in list is not in any partition
             haveError = true;
             Log.error( discovery.getCellSlot(cellID).getLongLabel() + " is selected, but already unassigned." );
             errMsg += ucBundle.getString( "UnassignCells.error.tooMany" );
         } else {
             parID  = complex.getCellAssignment(cellID);
             while( iter.hasNext() ) {
                 cellID = ((Integer) iter.next()).intValue();
                 int thisParID  = complex.getCellAssignment(cellID);
                 if( complex.getCellType(cellID) == Complex.FREE ) {
                     // cell is not in any partition
                     haveError = true;
                     errMsg =
                         ucBundle.getString( "UnassignCells.error.tooMany" );
                         Log.error( discovery.getCellSlot(cellID).getLongLabel() + " is selected, but already unassigned." );
                     break;
                 }
                 if( parID != thisParID ) {
                     // cell is in different partition than first one,
                     haveError = true;
                     errMsg =
                         ucBundle.getString( "UnassignCells.error.tooMany" );
                     Log.error( discovery.getCellSlot(cellID).getLongLabel() + " is selected, but is in a different nPartition than other selected cells." );
                     break;
                 }
             } // end while
         } // if first cell is free
    } // if there are cells at all
%>

<BODY>
<p class="subwindow-heading"> <%=pageTitle%> </p>
<p class="parmgr-regular-text">
<%=tagBundle.getString("ComplexScopeLabel")%>
<%=complex.getName()%>
</p>

<%
    if( haveError ) {
        // only need OK button, error message needs no additional help
%>
    <%=errMsg%>
    <form method="post" action="<%=thisPage%>" name="unassignCells">
    <p>
    <INPUT class="button"
           type="button"
           value="<%=pmBundle.getString("button.ok.str")%>"
           name="ok"
           onclick="window.location.replace('<%=parsession.getFromURL()%>')"
           />
    </p>
    </form>
<%
    } else {
        // no error, need Continue, Cancel, and Help
        // put the selected cells into the parsession
        session.setAttribute( "selectedCells", localSelects );
        String nParName;
        HWPartition partition = discovery.getPartition(parID);
        if (partition.isErrorObject()) {
            nParName = partition.getLabel();
        } else {
            nParName = partition.getLongLabel();
        }

%>
    <p>
    <%=ucBundle.getString("UnassignCells.intro")%>
    <%=nParName%>
    </p>
    <p><%=ucBundle.getString("UnassignCells.continue")%></p>
    <p><%=ucBundle.getString("UnassignCells.cancel")%></p>

    <FORM method="post" action="<%=thisPage%>" name="unassignCells">
    <INPUT type="hidden" name="<%=nparOpt%>" value="<%=parID%>" >
    <p>
    <INPUT class="button"
           type="submit"
           value="<%=pmBundle.getString("button.continue.str")%>"
           name="continue"
           />
    <INPUT class="button"
           type="submit"
           value="<%=pmBundle.getString("button.cancel.str")%>"
           name="cancel"
           />
    <INPUT class="button"
           type="button"
           value="<%=Help%>"
           name="help"
           onclick="showHelp('<%=parsession.getHelpURL(helpPage)%>')"
           />
    </p>
    </FORM>
<% } %>
</BODY>
</HTML>
<% } %>
